import com.sas.meta.SASOMI.IServer;
import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdOMRConnection;
import com.sas.tools.webappconfig.util.WebAppConfigException;

import java.io.File;
import java.io.FileReader;
import java.io.StringReader;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

Boolean debug = false;

// Get the script attributes from ant project
String _host 			= attributes.get( "host" );
String _port 			= attributes.get( "port" );
String _user 			= attributes.get( "user" );
String _password		= attributes.get( "password" );
String _backupconfig    = attributes.get( "backupconfig" );
String _userspec		= attributes.get( "userspec" );
String _userdir			= attributes.get( "userdir" );

String _outputPropName	= attributes.get( "outputproperty" );

// This variable holds the value that the returned ant property will be set to, 
// suitable for istrue condition task.  Values other than "True" and "False" are 
// error messages
String _output			= "True";

if (debug) {
   self.log( ">>>>> _host: " + _host );
   self.log( ">>>>> _port: " + _port );
   self.log( ">>>>> _user: " + _user );
   self.log( ">>>>> _password: " + _password );
   self.log( ">>>>> _backupconfig: " + _backupconfig );
   self.log( ">>>>> _userspec: " + _userspec );
   self.log( ">>>>> _userdir: " + _userdir );
}

try {
	//	 Connect to the metadata server
	MdFactory	m_mdFactory	= null;

	m_mdFactory = new MdFactoryImpl( false );
	MdOMRConnection connection = m_mdFactory.getConnection();
	
	IServer iServerConnection = connection.makeIServerConnection( _host, _port, _user, _password );
	String refreshOptions = readRefreshOptions( _backupconfig, _userspec, _userdir, debug );

	if (refreshOptions != null && refreshOptions.length() > 0)
	{
		if (debug) self.log( ">>>>> refreshOptions = \"" + refreshOptions + "\"." );
		iServerConnection.Refresh( refreshOptions );
	}
	else
		if (debug) self.log( ">>>>> refreshOptions is null." );

	// Disconnect from the metadata server
	m_mdFactory.closeOMRConnection();
	m_mdFactory.dispose();
}
catch (Throwable t) {
	t.printStackTrace();
	String throwableMessage = t.getLocalizedMessage();
	self.log("caught exception: " + throwableMessage);
	_output = "-1";
}

//Set the return code property
project.setNewProperty( _outputPropName, "" + _output );

public String readRefreshOptions( String metadataServerBackupConfigPath, String userSpecified, 
								  String userDirectory,					 boolean debug )
{

	String refreshOptions = "";
	StringBuffer metadataServerBackupConfigBuffer = new StringBuffer();
	try
	{
		File metadataServerBackupConfigFile = new File( metadataServerBackupConfigPath );
		FileReader metadataServerBackupConfigReader = new FileReader( metadataServerBackupConfigFile );
		int inchar = metadataServerBackupConfigReader.read();
		while (inchar != -1)
		{
			metadataServerBackupConfigBuffer.append((char) inchar);
			inchar = metadataServerBackupConfigReader.read();
		}
		DocumentBuilderFactory docBuildFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder = docBuildFactory.newDocumentBuilder();
		String metadataServerBackupConfigXML = metadataServerBackupConfigBuffer.toString();

		StringReader stringReader = new StringReader( metadataServerBackupConfigXML );
		Document parsedXML = docBuilder.parse(new org.xml.sax.InputSource( stringReader ));

		/*
		 * Look for <MetadataServerBackupConfiguration ....> tag
		 */
		NodeList xmlTags = parsedXML.getElementsByTagName( "MetadataServerBackupConfiguration" );
		NodeList scheduleTags, backupConfigTags;
		if (xmlTags != null && xmlTags.getLength() > 0)
		{
			if (debug) self.log( ">>>>> Found MetadataServerBackupConfiguration tag." );
			Element element  = (Element) xmlTags.item( 0 );
			// Look for <Schedule ....> tag
			scheduleTags     = element.getElementsByTagName( "Schedule" );
			// Look for <BackupConfiguration ....> tag
			backupConfigTags = element.getElementsByTagName( "BackupConfiguration" );
		}
		else
		{
			if (debug) self.log( ">>>>> Did not find MetadataServerBackupConfiguration tag." );
			// Look for <Schedule ....> tag
			scheduleTags     = parsedXML.getElementsByTagName( "Schedule" );
			// Look for <BackupConfiguration ....> tag
			backupConfigTags = parsedXML.getElementsByTagName( "BackupConfiguration" );
		}

		boolean dayExists = false;
		boolean locExists = false;
		boolean retExists = false;
		Element element;
		String scheduleOption = "";
		if (scheduleTags != null && scheduleTags.getLength() != 0)
		{
			if (debug) self.log( ">>>>> Found Schedule tag." );
			scheduleOption += "<Schedule";
			for (int i = 0; i < scheduleTags.getLength(); i++)
			{
				String weekday;
				element = (Element)scheduleTags.item( 0 );
				event = element.getAttribute( "Event" );
				if (event == null || !event.equalsIgnoreCase( "Backup" ))
					continue;

				if (debug) self.log( ">>>>> Found Event attribute Backup." );
				scheduleOption += " Event=\"Backup\"";
				weekday = element.getAttribute( "Weekday1" );
				if (weekday != null && weekday.length() != 0)
				{
					scheduleOption += " Weekday1=\"" + weekday + "\"";
					dayExists = true;
				}
				weekday = element.getAttribute( "Weekday2" );
				if (weekday != null && weekday.length() != 0)
				{
					scheduleOption += " Weekday2=\"" + weekday + "\"";
					dayExists = true;
				}
				weekday = element.getAttribute( "Weekday3" );
				if (weekday != null && weekday.length() != 0)
				{
					scheduleOption += " Weekday3=\"" + weekday + "\"";
					dayExists = true;
				}
				weekday = element.getAttribute( "Weekday4" );
				if (weekday != null && weekday.length() != 0)
				{
					scheduleOption += " Weekday4=\"" + weekday + "\"";
					dayExists = true;
				}
				weekday = element.getAttribute( "Weekday5" );
				if (weekday != null && weekday.length() != 0)
				{
					scheduleOption += " Weekday5=\"" + weekday + "\"";
					dayExists = true;
				}
				weekday = element.getAttribute( "Weekday6" );
				if (weekday != null && weekday.length() != 0)
				{
					scheduleOption += " Weekday6=\"" + weekday + "\"";
					dayExists = true;
				}
				weekday = element.getAttribute( "Weekday7" );
				if (weekday != null && weekday.length() != 0)
				{
					scheduleOption += " Weekday7=\"" + weekday + "\"";
					dayExists = true;
				}
				break;
			}
			scheduleOption += "/>";
			if (debug) self.log( ">>>>> Constructed Schedule option \"" + scheduleOption + "\"." );
			refreshOptions += scheduleOption;
		}

		String backupConfigOption = "";
		if (backupConfigTags != null && backupConfigTags.getLength() != 0)
		{
			if (debug) self.log( ">>>>> Found BackupConfiguration tag." );
			backupConfigOption += "<BackupConfiguration";

			element = (Element)backupConfigTags.item( 0 );
			String backupLocation = element.getAttribute( "BackupLocation" );
			if (userSpecified.equalsIgnoreCase( "true" ))
			    backupLocation = userDirectory;  /* user specified a location */
			else if (userDirectory != null && userDirectory.length() != 0)
			    backupLocation = userDirectory;  /* using the default location */
			if (backupLocation != null && backupLocation.length() != 0)
			{
				backupConfigOption += " BackupLocation=\"" + backupLocation + "\"";
				locExists = true;
			}
			String daysToRetainBackups = element.getAttribute( "DaysToRetainBackups" );
			if (daysToRetainBackups != null && daysToRetainBackups.length() != 0)
			{
				backupConfigOption += " DaysToRetainBackups=\"" + daysToRetainBackups + "\"";
				retExists = true;
			}
			backupConfigOption += "/>";
			if (debug) self.log( ">>>>> Constructed BackupConfiguration option \"" + backupConfigOption + "\"." );
			refreshOptions += backupConfigOption;
		}
		//Close reader
		if (metadataServerBackupConfigReader != null)
			metadataServerBackupConfigReader.close();
		return ( (dayExists && locExists && retExists)
				 ? refreshOptions
				 : "" );
	}
	catch (Throwable t) {
		t.printStackTrace();
		String throwableMessage = t.getLocalizedMessage();
		self.log("caught exception: " + throwableMessage);
		_output = "-1";
	}
}