import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdObjectStore;
import com.sas.metadata.remote.MdOMIUtil;
import com.sas.metadata.remote.MdOMRConnection;
import com.sas.metadata.remote.MetadataObjects;
import com.sas.metadata.remote.Root;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

Boolean debug = true;

// get attributes
String a_host = attributes["host"];
String a_port = attributes["port"];
String a_user = attributes["userid"];
String a_password = attributes["passwd"];
String a_objFQID = attributes["fqid"];
String a_objType = attributes["type"];
String a_objClassId = attributes["classid"];
String a_objOldName = attributes["oldname"];
String a_objNewName = attributes["newname"];
String a_ignoreHost = attributes["ignorehost"];

if (debug) {
   self.log("a_host: " + a_host);
   self.log("a_port: " + a_port);
   self.log("a_user: " + a_user);
   self.log("a_password: " + a_password);
   self.log("a_objFQID: " + a_objFQID);
   self.log("a_objType: " + a_objType);
   self.log("a_objClassId: " + a_objClassId);
   self.log("a_objOldName: " + a_objOldName);
   self.log("a_objNewName: " + a_objNewName);
   self.log("a_ignoreHost: " + a_ignoreHost);
}

String objectName = a_objOldName;

// get messages
String m_changed = project.getProperty("status.changeMetadataObjectName.changed.msg");
String m_unchanged = project.getProperty("status.changeMetadataObjectName.not.changed.msg");
String m_identical = project.getProperty("status.changeMetadataObjectName.identical.name.msg");
String m_update = project.getProperty("status.changeMetadataObjectName.update.failed.msg");
String m_nomatch = project.getProperty("status.changeMetadataObjectName.nomatch.msg");
String m_errmsg = project.getProperty("status.changeMetadataObjectName.unknown.msg");
String m_report = a_objType + " " + a_objFQID + " (" + a_objOldName + ") updated to " + a_objNewName + ".";
String m_hostvalues = project.getProperty("status.changeHostName.hostvalues.msg");
String m_oldhnf = project.getProperty("status.changeHostName.oldhost.notfound.msg");
String m_newhnf = project.getProperty("status.changeHostName.newhost.notfound.msg");
String m_noname = project.getProperty("status.changeMetadataObjectName.noname.msg");

if (debug) {
   self.log("m_changed: " + m_changed);
   self.log("m_unchanged: " + m_unchanged);
   self.log("m_identical: " + m_identical);
   self.log("m_update: " + m_update);
   self.log("m_nomatch: " + m_nomatch);
   self.log("m_errmsg: " + m_errmsg);
   self.log("m_report: " + m_report);
   self.log("m_hostvalues: " + m_hostvalues);
   self.log("m_oldhnf: " + m_oldhnf);
   self.log("m_newhnf: " + m_newhnf);
   self.log("m_noname: " + m_noname);
}

try {
   boolean ignoreHost = Boolean.parseBoolean(a_ignoreHost);

   // get host name property values
   if (!ignoreHost) {
      String p_oldhost = project.getProperty("deploymgr.changehost.old.host.name");
      String p_newhost = project.getProperty("deploymgr.changehost.new.host.name");
      if ((null == p_oldhost) || (null == p_newhost) || (p_oldhost.length() == 0) || (p_newhost.length() == 0)) {
         self.log(m_hostvalues);
         throw new Exception();
      }
      newHostList = new StringTokenizer(p_newhost, ",");
      oldHostList = new StringTokenizer(p_oldhost, ",");
   }
   else {
      if ((a_objNewName == null) || (a_objNewName.length() == 0)) {
         self.log(m_noname);
         throw new Exception();
      }
   }

   // Make a JOMA connection.
   def MdFactory       factory    = new MdFactoryImpl(false);
   def MdOMRConnection connection = factory.getConnection();
   def MdObjectStore   myStore    = factory.createObjectStore();
   def MdOMIUtil       omiUtil    = factory.getOMIUtil();

   connection.makeOMRConnection( a_host, a_port, a_user, a_password);
   String foundationID = omiUtil.getFoundationReposID();
   self.log("Repository ID: " + foundationID);

   // check name lengths
   String checkSelect = "<xmlselect search=\"" + a_objType + "[@Name LE '" + a_objOldName + "']\"/>";

   if (debug)
      self.log("Check name lengths: " + checkSelect);

   int checkFlags = MdOMIUtil.OMI_GET_METADATA | MdOMIUtil.OMI_XMLSELECT | MdOMIUtil.OMI_ALL_SIMPLE;
   List checkList = omiUtil.getMetadataObjectsSubset(myStore, foundationID, a_objType, checkFlags, checkSelect);

   if (debug)
      self.log("Object list: " + checkList);

   if (checkList != null && checkList.size() > 0) {
      int objMaxLength = checkList[0].getNameMaxLength();

      if (debug)
         self.log("Object Max Length: " + objMaxLength);

      if (a_objOldName.length() > objMaxLength) {
         a_objOldName = a_objOldName.substring(0, objMaxLength);
         self.log("Old name too long, truncated to " + objMaxLength + ": " + a_objOldName);
      }

      if ((a_objNewName != null) && (a_objNewName.length() > objMaxLength)) {
         a_objNewName = a_objNewName.substring(0, objMaxLength);
         self.log("New name too long, truncated to " + objMaxLength + ": " + a_objNewName);
      }
   }

   // set the xmlselect
   String objFQID = "";
   if (a_objFQID != null && a_objFQID.length() > 0)
      objFQID = " and @Id='" + a_objFQID + "'";

   String objClassId = "";
   if (a_objClassId != null && a_objClassId.length() > 0)
      objClassId = " and @ClassIdentifier='" + a_objClassId + "'";

   String xmlselect = "<xmlselect search=\"" + a_objType + "[@Name='" + a_objOldName + "'" + objFQID + objClassId + "]\"/>";
   self.log("xmlselect: " + xmlselect);

   int omiFlags = MdOMIUtil.OMI_DEPENDENCY_USES | MdOMIUtil.OMI_GET_METADATA | MdOMIUtil.OMI_XMLSELECT | MdOMIUtil.OMI_ALL_SIMPLE | MdOMIUtil.OMI_INCLUDE_SUBTYPES;
   if (debug)
      self.log("omiFlags: " + omiFlags);

   // get metadata object list
   List objectList = omiUtil.getMetadataObjectsSubset(myStore, "", a_objType, omiFlags, xmlselect);

   // no object returned
   if (objectList == null || objectList.size() == 0)
      self.log(m_nomatch);
   else {
      if (debug)
         self.log("* objectList: " + objectList);

      for (Iterator i = objectList.iterator(); i.hasNext();) {
         // Get the object name
         def Root myObject = i.next();

         objectName = myObject.name;
         if (debug)
            self.log("* objectName: " + objectName);
            
         // old and new names are equal - do nothing
         if (objectName.equalsIgnoreCase(a_objNewName)) {
            self.log(m_identical);
         }
         // match found
         else if (objectName.equalsIgnoreCase(a_objOldName)) {
            // process the host pairs
            if (!ignoreHost) {
               while ((oldHostList.hasMoreTokens()) && (newHostList.hasMoreTokens())) {
                  String oldHostItem = oldHostList.nextToken().trim();
                  String newHostItem = newHostList.nextToken().trim();

                  // make sure we have host values
                  if (null == oldHostItem || oldHostItem.length() == 0) {
                     self.log(m_oldhnf);
                     throw new Exception(m_oldhnf);
                  }
                  else if (null == newHostItem || newHostItem.length() == 0) {
                     self.log(m_newhnf);
                     throw new Exception(m_newhnf);
                  }

                  String q_oldHostItem = "(?i:" + java.util.regex.Pattern.quote(oldHostItem) + ")";
                  if (debug) {
                     self.log("* oldHostItem: " + oldHostItem);
                     self.log("* newHostItem: " + newHostItem);
                  }

                  // define the pattern to match
                  prefixPattern = "(^|[^-0-9a-zA-Z.])";
                  suffixPattern = "((?!\\.?[-0-9a-zA-Z]))";
                  Pattern matchPattern = Pattern.compile(prefixPattern + q_oldHostItem + suffixPattern);
                  if (debug) {
                     self.log("* matchPattern: " + matchPattern.toString());
                  }

                  Matcher matcher = matchPattern.matcher(objectName);
                  self.log("-Search for: " + oldHostItem + " in object name: " + objectName);

                  if (matcher.find()) {
                     self.log("--Found match.");

                     a_objNewName = objectName.replaceAll(matchPattern.toString(), "\$1" + newHostItem + "\$2");
                     if (debug) {
                        self.log("* a_objNewName: " + a_objNewName);
                     }

                     // update the metadata object
                     try {
                        myObject.setName(a_objNewName);
                        myStore.updatedMetadataAll();
                        self.log(m_changed + myObject.name);
                        g_insertDocpart("metadata", "changeHost_metadata_item.txt", "", "", m_report, "");
                     }
                     catch (Throwable t) {
                        m_errmsg = m_update;
                        self.log(m_errmsg);
                        g_issueWarning("metadata", "changeHost_metadata_warning_item.txt", m_errmsg, objectName, a_objNewName, m_errmsg, "");
                     }
                  }
               }
            }
            else {
               // update the metadata object
               try {
                  myObject.setName(a_objNewName);
                  myStore.updatedMetadataAll();
                  self.log(m_changed + myObject.name);
                  g_insertDocpart("metadata", "changeHost_metadata_item.txt", "", "", m_report, "");
               }
               catch (Throwable t) {
                  m_errmsg = m_update;
                  self.log(m_errmsg);
                  g_issueWarning("metadata", "changeHost_metadata_warning_item.txt", m_errmsg, objectName, a_objNewName, m_errmsg, "");
               }
            }
         }
         // no match on the name
         else {
            self.log(m_nomatch);
         }
      }
   }

   myStore.dispose();
   connection.closeOMRConnection();
   factory.dispose();
   return;
}
catch (Throwable t) {
   self.log(m_errmsg);
   self.log(m_unchanged + objectName);
   t.printStackTrace();
   return;
}

// issue a task warning
void g_issueWarning(task, docpart, message, oldvalue, newvalue, param1, param2) {
   self.log("Set warning status: " + message);
   statusProps = new Properties();
   statusProps.setProperty("warningsEncountered", "true");
   statusProps.setProperty("error.msg", message);
   statusFileName = project.getProperty("whatFailed.property.file");
   statusFile = new File(statusFileName);
   statusFOS = new FileOutputStream(statusFile);
   statusProps.store(statusFOS, null);
   statusFOS.close();
   g_insertDocpart(task, docpart, oldvalue, newvalue, param1, param2);
}

// insert docpart
void g_insertDocpart(name, docpart, oldvalue, newvalue, param, param2) {
   try {
      if (docpart.length() == 0)
         docpart = "changeHost_" + name + "_item.txt";

      insert = project.createTask("addDocpart");
      insert.setDynamicAttribute("name", name);
      insert.setDynamicAttribute("docpart", docpart);
      insert.setDynamicAttribute("oldhost", oldvalue);
      insert.setDynamicAttribute("newhost", newvalue);
      insert.setDynamicAttribute("param", param.toString());
      insert.setDynamicAttribute("param2", param2.toString());
      insert.perform();

      g_writeDocpartHeader(name);
   }
   catch (Throwable t) {
      self.log("Error writing docpart: " + t.toString());
   }
}

// write docpart header
void g_writeDocpartHeader(name) {
   try {
      header = project.createTask("addDocpartHeader");
      header.setDynamicAttribute("name", name);
      header.perform();
   }
   catch (Throwable t) {
      self.log("Error writing docpart header: " + t.toString());
   }
}
