import com.sas.meta.SASOMI.IOMI;
import com.sas.metadata.*;
import java.util.*;

MetadataWorkspace workspace = null;
IOMI omrConnection = null;
String fqId = null;
MdObjectStore objectStore = null;

String _host = attributes["host"];
String _port = attributes["port"];
String _user = attributes["user"];
String _password = attributes["password"];
String _repository = attributes["repository"];
String _serverDbms = attributes["serverdbms"];
String _serverName = attributes["servername"];
String _serverDbName = attributes["serverdbname"];
String _serverHost = attributes["serverhost"];
String _serverPort = attributes["serverport"];
String _serverDomain = attributes["serverdomain"];
String _serverListener = attributes["serverlistener"];
String _serverText = attributes["servertext"];
String _outputproperty = attributes["outputproperty"];
String _resultproperty = attributes["resultproperty"];

echoLevel = new org.apache.tools.ant.taskdefs.Echo.EchoLevel();
echoLevel.setValue("info");
echoTask = project.createTask("echo");
echoTask.setLevel(echoLevel);
echoTask.setMessage("Connecting to the metadata server.");  
echoTask.perform();

if (_serverText != null) {
    echoTask.setMessage("Setting _connectionName to " + _serverText + " " + _serverDomain);    echoTask.perform();
  
    echoTask.perform();
   _connectionName=_serverText + " " + _serverDomain
   } 
else {
   _connectionName=_serverDomain
   }

// connect to the server
try {
	workspace     = MetadataWorkspace.getWorkspace();
	omrConnection = workspace.makeOMRConnection(_host, _port, _user, _password);
	workspace.setCMRHandle(omrConnection);
}
catch (Exception e) {
	returnProperty("1", "Error connecting to the Metadata Server: " + e.getMessage() +
			"\n   Host: " + _host +
			"\n   Port: " + _port +
			"\n   User: " + _user, _outputproperty, _resultproperty);
	return;
}

echoTask.setMessage("Connected.");  
echoTask.perform();

// find the repository id we want
try {
	List repositories = MetadataUtil.getRepositories();
	
	for (Iterator it = repositories.iterator(); it.hasNext();) {
		Root repos = (Root)it.next();
		if (_repository.equals(repos.getName()))
			fqId = repos.getFQID();
	}
}
catch (Exception e) {
	returnProperty("1", "Error finding repository " + _repository + "\n" + e.getMessage(), _outputproperty, _resultproperty);
    workspace.closeOMRConnection();
	return;
}

if (fqId == null) {
	returnProperty("1", "Error finding repository " + _repository + ".", _outputproperty, _resultproperty);
    workspace.closeOMRConnection();
	return;    
} else {
    echoTask.setMessage(_repository + " repository found.");  
    echoTask.perform();
}

objectStore = MdObjectFactory.createObjectStore();

AuthenticationDomain authDomain = getAuthDomain(_serverDomain, objectStore, fqId);
if (authDomain == null){
    returnProperty("1", "The Authentication Domain, " + _serverDomain + ", was not found.", _outputproperty, _resultproperty);
    workspace.closeOMRConnection();
    return;
}

ServerComponent sComp = getServerComponent(_serverName, objectStore, fqId);
if (sComp == null){
    returnProperty("1", "The Server Component, " + _serverName + ", was not found.", _outputproperty, _resultproperty);
    workspace.closeOMRConnection();
    return;
}

SASClientConnection sasConn = getClientConnection(_connectionName, objectStore, fqId);
if (sasConn != null){
    echoTask.setMessage("The SAS Client Connection, " + "Connection: " + _connectionName + ", already exists, updating.");  
    echoTask.perform();
    updateSConn(sConn, _serverDbms, _serverDbName, _serverPort, _dbName, _serverListener, authDomain);
    returnProperty("0","The SAS Client Connection, " + "Connection: " + _connectionName + " was updated." + e.getMessage(),_outputproperty, _resultproperty);
    if (workspace != null) {
        // Close metadata server connection
       try {
           workspace.closeOMRConnection();
       }
       catch (Exception e1) {
         returnProperty("1","Error closing metadata server connection: " + e1.getMessage(),_outputproperty, _resultproperty);
         return;
       }
    }
    return;
}

// Create the SAS Client Connection
MdFactory factory = new MdObjectFactory();
String reposId=fqId.substring(9);
switch ( _serverDbms ) {
    case "oracle":
         _appProtocol="OracleProtocol"
        break

    case "db2":
         _serverListener="${_serverDbName}"
         _appProtocol="DB2UDBPROTOCOL"
         break

    case "mysql":
         _serverListener="${_serverDbName}"
         _serverName="${_serverHost}"
         _appProtocol="MYSQLPROTOCOL"
         break

    case "odbc":
         _appProtocol="ODBCPROTOCOL"
         break

    case "sqlsvr":
         _appProtocol="SQLSvrPROTOCOL"
         break

    case "teradata":
         _serverListener="${_serverDbName}"
         _appProtocol="TeradataProtocol"
         break
}

SASClientConnection newSasConn = makeClientConnection(_connectionName, _serverDbms, _serverName, _serverDomain, _serverPort, authDomain, _serverListener, objectStore, reposId, factory);
AssociationList connections = sComp.getSourceConnections();
connections.add(newSasConn);
sComp.updateMetadataAll();

objectStore.dispose();

if (workspace != null) {
    // Close metadata server connection
   try {
       workspace.closeOMRConnection();
   }
   catch (Exception e) {
     returnProperty("1","Error closing metadata server connection: " + e.getMessage(),_outputproperty, _resultproperty);
     return;
   }
}

returnProperty("0", "End of script.", _outputproperty, _resultproperty);

public SASClientConnection getClientConnection(String connectionName, MdObjectStore objectStore, String reposId){
    String mdType = MetadataObjects.SASCLIENTCONNECTION;
    String xmlSelect = "<XMLSELECT Search=\"@Name='" + "Connection: " + connectionName + "'\"/>";
    SASClientConnection cConn = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        cConn = (SASClientConnection)found.get(0);
    }
    return cConn;
}

public SASClientConnection makeClientConnection(String connectionName, String serverDbms, String serverName, String domain, String serverPort, AuthenticationDomain authDomain, String listener, MdObjectStore objectStore, String reposId, MdFactory mdObjFactory){
    String type = MdObjectFactory.SASCLIENTCONNECTION;
    SASClientConnection newConn = (SASClientConnection)mdObjFactory.createComplexMetadataObject(objectStore, null, "Connection: " + connectionName, type, reposId);
    Integer port = new Integer(0);
    newConn.setPort(port.intValue());
    newConn.setApplicationProtocol("${_appProtocol}");
    newConn.setAuthenticationType("user/password");
    newConn.setCommunicationProtocol("TCP");
    newConn.setDomain(authDomain);
    switch ( serverDbms ) {
      case "oracle":
         Property serverProperty = makeNewProperty("Connection.Oracle.Property.PATH.Name.xmlKey.txt", "PATH", listener, 12, objectStore, reposId, mdObjFactory);
         AssociationList properties = newConn.getProperties();
         properties.add(serverProperty);
         break
      case "db2":
         Property serverProperty = makeNewProperty("Connection.DB2U.Property.Datasrc.Name.xmlKey.txt", "Datasrc", listener, 12, objectStore, reposId, mdObjFactory);
         AssociationList properties = newConn.getProperties();
         properties.add(serverProperty);
         break
      case "mysql":
         Property serverProperty1 = makeNewProperty("Connection.MySQL.Property.DATABASE.Name.xmlKey.txt", "DATABASE", listener, 12, objectStore, reposId, mdObjFactory);
         AssociationList properties1 = newConn.getProperties();
         properties1.add(serverProperty1);
         Property serverProperty2 = makeNewProperty("Connection.MySQL.Property.SERVER.Name.xmlKey.txt", "SERVER", serverName, 12, objectStore, reposId, mdObjFactory);
         AssociationList properties2 = newConn.getProperties();
         properties2.add(serverProperty2);
         Property serverProperty3 = makeNewProperty("Connection.Property.Port.Name.xmlKey.txt", "MYSQL_PORT", serverPort, 12, objectStore, reposId, mdObjFactory);
         AssociationList properties3 = newConn.getProperties();
         properties3.add(serverProperty3);
         break
      case "odbc":
         Property serverProperty = makeNewProperty("Connection.ODBC.Property.DATASRC.Name.xmlKey.txt", "DATASRC", listener, 12, objectStore, reposId, mdObjFactory);
         AssociationList properties = newConn.getProperties();
         properties.add(serverProperty);
         break
      case "sqlsvr":
         Property serverProperty = makeNewProperty("Connection.SQL.Property.Datasrc.Name.xmlKey.txt", "Datasrc", listener, 12, objectStore, reposId, mdObjFactory);
         AssociationList properties = newConn.getProperties();
         properties.add(serverProperty);
         break
      case "teradata":
         Property serverProperty = makeNewProperty("Connection.Teradata.Property.SERVER.Name.xmlKey.txt", "SERVER", listener, 12, objectStore, reposId, mdObjFactory);
         AssociationList properties = newConn.getProperties();
         properties.add(serverProperty);
         break
   }
    return newConn;
}

public void updateSConn(SASClientConnection conn, String connectionName, String serverDbms, String serverDbName, String serverPort, String dbName, String listener, AuthenticationDomain domain) {
    Integer port = new Integer(serverPort);
    conn.setPort(port.intValue());
    AssociationList domains = conn.getDomains();
    domains.removeAllElements();
    conn.setDomain(domain);
    AssociationList properties = conn.getProperties();
    for (Iterator props = properties.iterator(); props.hasNext();) {
        Property property = (Property)props.next();
        if ("Connection.Oracle.Property.PATH.Name.xmlKey.txt".equals(property.getName())){
            property.setDefaultValue(listener);
            break
        }
        else if ("Connection.DB2U.Property.Datasrc.Name.xmlKey.txt".equals(property.getName())) {
            property.setDefaultValue(serverDbName);
            break
        }
        else if ("Connection.MySQL.Property.DATABASE.Name.xmlKey.txt".equals(property.getName())) {
            property.setDefaultValue(listener);
            break
        }
        else if ("Connection.Property.Port.Name.xmlKey.txt".equals(property.getName())) {
            property.setDefaultValue(serverPort);
            break
        }
        else if ("Connection.MySQL.Property.SERVER.Name.xmlKey.txt".equals(property.getName())) {
            property.setDefaultValue(serverName);
            break
        }
        else if ("Connection.ODBC.Property.DATASRC.Name.xmlKey.txt".equals(property.getName())) {
            property.setDefaultValue(serverDbName);
            break
        }
        else if ("Connection.SQL.Property.Datasrc.Name.xmlKey.txt".equals(property.getName())) {
            property.setDefaultValue(serverDbName);
            break
        }
        else if ("Connection.Teradata.Property.SERVER.Name.xmlKey.txt".equals(property.getName())) {
            property.setDefaultValue(serverDbName);
            break
        }
    }
}

public Property makeNewProperty(String name, String propertyName, String defaultValue, int sqlType, MdObjectStore objectStore, String reposId, MdFactory mdObjFactory){
    String type = MdObjectFactory.PROPERTY;
    Property newProperty = (Property)mdObjFactory.createComplexMetadataObject(objectStore, null, name, type, reposId);
    newProperty.setDelimiter("=");
    newProperty.setPropertyName(propertyName);
    newProperty.setDefaultValue(defaultValue);
    newProperty.setSQLType(sqlType);
    return newProperty;
}

public ServerComponent getServerComponent(String name, MdObjectStore objectStore, String reposId){
    String mdType = MetadataObjects.SERVERCOMPONENT;
    String xmlSelect = "<XMLSELECT Search=\"@Name='" + name + "'\"/>";
    ServerComponent sComp = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        sComp = (ServerComponent)found.get(0);
    }
    return sComp;
}

public AuthenticationDomain getAuthDomain(String name, MdObjectStore objectStore, String reposId){
    String mdType = MetadataObjects.AUTHENTICATIONDOMAIN;
    String xmlSelect = "<XMLSELECT Search=\"@Name='" + name + "'\"/>";
    AuthenticationDomain aDomain = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        aDomain = (AuthenticationDomain)found.get(0);
    }
    return aDomain;
}

public List findMetadataObjects(MdObjectStore mdObjectStore,
        String reposId, String mdType, String xmlSelect)
        throws MdException {
    List objectList = MetadataUtil.getMetadataObjectsSubset(mdObjectStore,
            reposId, mdType, MetadataUtil.OMI_XMLSELECT
                           | MetadataUtil.OMI_GET_METADATA,
            xmlSelect
    );
    return objectList;
}

public void returnProperty(String _rc, String _message, String _outputproperty, String _resultproperty) {
	project.setProperty(_outputproperty, _message);
	project.setProperty(_resultproperty, _rc);
	echoTask.setMessage(_message);  echoTask.perform();
	return;
}
