import com.sas.meta.SASOMI.IOMI;
import com.sas.metadata.*;
import java.util.*;

MetadataWorkspace workspace = null;
IOMI omrConnection = null;
String fqId = null;
MdObjectStore objectStore = null;

String _host = attributes["host"];
String _port = attributes["port"];
String _user = attributes["user"];
String _password = attributes["password"];
String _repository = attributes["repository"];
String _serverDbms = attributes["serverdbms"];
String _serverPort = attributes["serverport"];
String _serverName = attributes["servername"];
String _serverDomain = attributes["serverdomain"];
String _libname = attributes["libname"];
String _libref = attributes["libref"];
String _schemaObjectName = attributes["schemaobjectname"];
String _schemaName = attributes["schemaname"];
String _loginName = attributes["loginname"];
String _serverContextName = attributes["servercontext"];
String _folderSearchPath = attributes["foldersearchpath"];
String _outputproperty = attributes["outputproperty"];
String _resultproperty = attributes["resultproperty"];

echoLevel = new org.apache.tools.ant.taskdefs.Echo.EchoLevel();
echoLevel.setValue("info");
echoTask = project.createTask("echo");
echoTask.setLevel(echoLevel);
echoTask.setMessage("Connecting to the metadata server.");  
echoTask.perform();

// connect to the server
try {
	workspace     = MetadataWorkspace.getWorkspace();
	omrConnection = workspace.makeOMRConnection(_host, _port, _user, _password);
	workspace.setCMRHandle(omrConnection);
}
catch (Exception e) {
	returnProperty("1", "Error connecting to the Metadata Server: " + e.getMessage() +
			"\n   Host: " + _host +
			"\n   Port: " + _port +
			"\n   User: " + _user, _outputproperty, _resultproperty);
	return;
}

echoTask.setMessage("Connected.");  
echoTask.perform();

// find the repository id we want
try {
	List repositories = MetadataUtil.getRepositories();
	
	for (Iterator it = repositories.iterator(); it.hasNext();) {
		Root repos = (Root)it.next();
		if (_repository.equals(repos.getName()))
			fqId = repos.getFQID();
	}
}
catch (Exception e) {
	returnProperty("1", "Error finding repository " + _repository + "\n" + e.getMessage(), _outputproperty, _resultproperty);
    workspace.closeOMRConnection();
	return;
}

if (fqId == null) {
	returnProperty("1", "Error finding repository " + _repository + ".", _outputproperty, _resultproperty);
    workspace.closeOMRConnection();
	return;    
} else {
    echoTask.setMessage(_repository + " repository found.");  
    echoTask.perform();
}

objectStore = MdObjectFactory.createObjectStore();

ServerComponent sComp = getServerComponent(_serverName, objectStore, fqId);
if (sComp == null){
    returnProperty("1", "The Server Component, " + _serverName + ", does not exists.", _outputproperty, _resultproperty);
    workspace.closeOMRConnection();
    return;
}

// Create the SAS Library
MdFactory factory = new MdObjectFactory();
String reposId=fqId.substring(9);

switch ( _serverDbms ) {
   case "oracle":
      _dbms="Oracle"
      break

   case "db2":
      _dbms="DB2"
      break

   case "mysql":
      _dbms="MYSQL"
      break

   case "odbc":
      _dbms="ODBC"
      break

   case "sqlsvr":
      _dbms="SQLSVR"
      break

   case "teradata":
      _dbms="Teradata"
      break
}

Login defaultLogin = getLogin(_loginName, objectStore, fqId);
if (defaultLogin == null){
    returnProperty("1", "The Login, " + _loginName + ", was not found.", _outputproperty, _resultproperty);
    return;
}
SASClientConnection clientConn = getConnection("Connection: " + _serverDomain, objectStore, fqId);
if (clientConn == null){
    returnProperty("1", "The SAS Client Connection, " + "Connection: " + _serverDomain + ", was not found.", _outputproperty, _resultproperty);
    workspace.closeOMRConnection();
    return;
}
Tree folder = getFolder(_folderSearchPath, objectStore, fqId);
if (folder == null){
    returnProperty("1", "The Folder, " + "Libraries" + ", was not found.", _outputproperty, _resultproperty);
    workspace.closeOMRConnection();
    return;
}
ServerContext serverContext = getServerContext(_serverContextName, objectStore, fqId);
if (serverContext == null){
    returnProperty("1", "The Server Context, " + _serverContextName + ", was not found.", _outputproperty, _resultproperty);
    workspace.closeOMRConnection();
    return;
}
SASLibrary lib = getSASLibrary(_libname, objectStore, fqId);
if (lib != null){
    echoTask.setMessage("The SAS Library, " + _libname + ", already exists, updating.");  
    echoTask.perform();
    AssociationList dComponents = lib.getDeployedComponents();
    dComponents.removeAllElements();
    dComponents.add(serverContext);
    AssociationList logins = lib.getDefaultLogins();
    logins.removeAllElements();
    lib.setDefaultLogin(defaultLogin);
    returnProperty("0", "The SAS Library, " + _libname + ", was updated.", _outputproperty, _resultproperty);
    if (workspace != null) {
        // Close metadata server connection
       try {
           workspace.closeOMRConnection();
       }
       catch (Exception e2) {
         returnProperty("1","Error closing metadata server connection: " + e2.getMessage(),_outputproperty, _resultproperty);
         return;
       }
    }
    return;
}

SASLibrary sasLib = makeSASLibrary(_libname, _libref, defaultLogin, serverContext, clientConn, folder, objectStore, reposId, factory);
DatabaseSchema schema = makeSchema(_schemaObjectName, _schemaName, objectStore, reposId, factory);
AssociationList dPackages = sComp.getDataPackages();
dPackages.add(schema);
AssociationList uPackages = sasLib.getUsingPackages();
uPackages.add(schema);
sasLib.updateMetadataAll();

objectStore.dispose();

if (workspace != null) {
    // Close metadata server connection
   try {
       workspace.closeOMRConnection();
   }
   catch (Exception e) {
     returnProperty("1","Error closing metadata server connection: " + e.getMessage(),_outputproperty, _resultproperty);
     return;
   }
}

returnProperty("0", "End of script.", _outputproperty, _resultproperty);

public DatabaseSchema makeSchema(String name, String schemaName, MdObjectStore objectStore, String reposId, MdFactory mdObjFactory){
    String type = MdObjectFactory.DATABASESCHEMA;
    DatabaseSchema newSchema = (DatabaseSchema)mdObjFactory.createComplexMetadataObject(objectStore, null, name, type, reposId);
    newSchema.setSchemaName(schemaName);
    return newSchema;
}

public SASLibrary makeSASLibrary(String name, String ref, Login defaultLogin, ServerContext sContext, SASClientConnection sasClientConn, Tree folder, MdObjectStore objectStore, String reposId, MdFactory mdObjFactory){
    String type = MdObjectFactory.SASLIBRARY;
    SASLibrary newLib = (SASLibrary)mdObjFactory.createComplexMetadataObject(objectStore, null, name, type, reposId);
    newLib.setUsageVersion(1000000);
    newLib.setLibref(ref);
    newLib.setEngine("${_dbms}");
    newLib.setIsDBMSLibname(1);
    newLib.setIsPreassigned(1);
    newLib.setPublicType("Library");
    newLib.setDefaultLogin(defaultLogin);
    newLib.setLibraryConnection(sasClientConn);
    AssociationList trees = newLib.getTrees();
    trees.add(folder);
    AssociationList dComponents = newLib.getDeployedComponents();
    dComponents.add(sContext);
    return newLib;
}

public SASLibrary getSASLibrary(String name, MdObjectStore objectStore, String reposId){
    String mdType = MetadataObjects.SASLIBRARY;
    String xmlSelect = "<XMLSELECT Search=\"@Name='" + name + "'\"/>";
    SASLibrary lib = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        lib = (SASLibrary)found.get(0);
    }
    return lib;
}

public ServerContext getServerContext(String name, MdObjectStore objectStore, String reposId){
    String mdType = MetadataObjects.SERVERCONTEXT;
    String xmlSelect = "<XMLSELECT Search=\"@Name='" + name + "'\"/>";
    ServerContext context = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        context = (ServerContext)found.get(0);
    }
    return context;
}

public Tree getFolder(String searchString, MdObjectStore objectStore, String reposId){
    String mdType = MetadataObjects.TREE;
    String xmlSelect = "<XMLSELECT Search=\"" + searchString + "\"/>";
    Tree folder = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        folder = (Tree)found.get(0);
    }
    return folder;
}

public SASClientConnection getConnection(String name, MdObjectStore objectStore, String reposId){
    String mdType = MetadataObjects.SASCLIENTCONNECTION;
    String xmlSelect = "<XMLSELECT Search=\"@Name='" + name + "'\"/>";
    SASClientConnection connection = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        connection = (SASClientConnection)found.get(0);
    }
    return connection;
}

public Login getLogin(String name, MdObjectStore objectStore, String reposId){
    String mdType = MetadataObjects.LOGIN;
    String xmlSelect = "<XMLSELECT Search=\"@Name='" + name + "'\"/>";
    Login login = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        login = (Login)found.get(0);
    }
    return login;
}

public ServerComponent getServerComponent(String name, MdObjectStore objectStore, String reposId){
    String mdType = MetadataObjects.SERVERCOMPONENT;
    String xmlSelect = "<XMLSELECT Search=\"@Name='" + name + "'\"/>";
    ServerComponent sComp = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        sComp = (ServerComponent)found.get(0);
    }
    return sComp;
}

public AuthenticationDomain getAuthDomain(String name, MdObjectStore objectStore, String reposId){
    String mdType = MetadataObjects.AUTHENTICATIONDOMAIN;
    String xmlSelect = "<XMLSELECT Search=\"@Name='" + name + "'\"/>";
    AuthenticationDomain aDomain = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        aDomain = (AuthenticationDomain)found.get(0);
    }
    return aDomain;
}

public Machine getAssocMachine(String name, MdObjectStore objectStore, String reposId){
    String mdType = MetadataObjects.MACHINE;
    String xmlSelect = "<XMLSELECT Search=\"@Name='" + name + "'\"/>";
    ServerComponent machine = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        machine = (ServerComponent)found.get(0);
    }
    return machine;
}

public List findMetadataObjects(MdObjectStore mdObjectStore,
        String reposId, String mdType, String xmlSelect)
        throws MdException {
    List objectList = MetadataUtil.getMetadataObjectsSubset(mdObjectStore,
            reposId, mdType, MetadataUtil.OMI_XMLSELECT
                           | MetadataUtil.OMI_GET_METADATA,
            xmlSelect
    );
    return objectList;
}

public void returnProperty(String _rc, String _message, String _outputproperty, String _resultproperty) {
	project.setProperty(_outputproperty, _message);
	project.setProperty(_resultproperty, _rc);
	echoTask.setMessage(_message);  echoTask.perform();
	return;
}
