import com.sas.meta.SASOMI.IOMI;
import com.sas.metadata.*;
import java.util.*;

MetadataWorkspace workspace = null;
IOMI omrConnection = null;
String fqId = null;
MdObjectStore objectStore = null;

String _host = attributes["host"];
String _port = attributes["port"];
String _user = attributes["user"];
String _password = attributes["password"];
String _repository = attributes["repository"];
String _searchpath = attributes["libsearch"]
String _libpath = attributes["libpath"];
String _outputproperty = attributes["outputproperty"];
String _resultproperty = attributes["resultproperty"];

echoLevel = new org.apache.tools.ant.taskdefs.Echo.EchoLevel();
echoLevel.setValue("info");
echoTask = project.createTask("echo");
echoTask.setLevel(echoLevel);
echoTask.setMessage("Connecting to the metadata server.");
echoTask.perform();

// connect to the server
try {
    workspace     = MetadataWorkspace.getWorkspace();
    omrConnection = workspace.makeOMRConnection(_host, _port, _user, _password);
    workspace.setCMRHandle(omrConnection);
}
catch (Exception e) {
    returnProperty("1", "Error connecting to the Metadata Server: " + e.getMessage() +
            "\n   Host: " + _host +
            "\n   Port: " + _port +
            "\n   User: " + _user, _outputproperty, _resultproperty);
    return;
}

echoTask.setMessage("Connected.");
echoTask.perform();

// find the repository id we want
try {
    List repositories = MetadataUtil.getRepositories();
    
    for (Iterator it = repositories.iterator(); it.hasNext();) {
        Root repos = (Root)it.next();
        if (_repository.equals(repos.getName()))
            fqId = repos.getFQID();
    }
}
catch (Exception e) {
    returnProperty("1", "Error finding repository " + _repository + "\n" + e.getMessage(), _outputproperty, _resultproperty);
    return;
}

if (fqId == null) {
    returnProperty("1", "Error finding repository " + _repository + ".", _outputproperty, _resultproperty);
    return;
} else {
    echoTask.setMessage(_repository + " repository found.");
    echoTask.perform();
}

objectStore = MdObjectFactory.createObjectStore();

// Update Library Path

SASLibrary lib = getLibrary(_searchpath, _libpath, objectStore, fqId);
if (lib == null){
    returnProperty("1", "Error finding SAS Library.", _outputproperty, _resultproperty);
    return;
} else {
    echoTask.setMessage("SAS Library, " + lib.getName() + ", found.");
    echoTask.perform();
}

Directory pathDir = updatePath(lib, _libpath);
if (pathDir == null){
    returnProperty("2", "Error finding Directory.", _outputproperty, _resultproperty);
    return;
} else {
    echoTask.setMessage("Directory, " + pathDir.getName() + ", updated.");
    echoTask.perform();
}

objectStore.dispose();

if (workspace != null) {
    // Close metadata server connection
   try {
       workspace.closeOMRConnection();
   }
   catch (Exception e) {
     returnProperty("1","Error closing metadata server connection: " + e.getMessage(),_outputproperty, _resultproperty);
     return;
   }
}

returnProperty("0", "End of script.", _outputproperty, _resultproperty);

public void returnProperty(String _rc, String _message, String _outputproperty, String _resultproperty) {
    project.setProperty(_outputproperty, _message);
    project.setProperty(_resultproperty, _rc);
    echoTask.setMessage(_message);  echoTask.perform();
    return;
}

public List findMetadataObjects(MdObjectStore mdObjectStore,
    String reposId, String mdType, String xmlSelect)
    throws MdException {
        List objectList = MetadataUtil.getMetadataObjectsSubset(mdObjectStore,
            reposId, mdType, 
            MetadataUtil.OMI_XMLSELECT | MetadataUtil.OMI_GET_METADATA,
            xmlSelect
        );
        return objectList;
}

public SASLibrary getLibrary(String searchPath, String dirPath, MdObjectStore objectStore, String reposId) {
    String mdType = MetadataObjects.SASLIBRARY;
    String xmlSelect = "<XMLSELECT Search=\"" + searchPath + "\"/>";
    SASLibrary library = null;
    List found = findMetadataObjects(objectStore, reposId, mdType, xmlSelect);
    if (found != null && found.size() > 0) {
        library = (SASLibrary)found.get(0);
    }
    return library;
}

public Directory updatePath(SASLibrary library, String dirPath){
    AssociationList usingPackages = library.getUsingPackages();
    Directory directory = null;
    String dirName = "Path";
    if (usingPackages.size() > 0){
        for (int aPackage = 0; aPackage < usingPackages.size(); aPackage++){
            directory = (Directory)usingPackages.get(aPackage);
            if (dirName.equals(directory.getName())){
                directory.setDirectoryName(dirPath);
                library.updateMetadataAll();
                break;
            } else {
                directory = null;
            }
        }
    }
    return directory;
}

