import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdObjectStore;
import com.sas.metadata.remote.MdOMIUtil;
import com.sas.metadata.remote.MdOMRConnection;
import com.sas.metadata.remote.MetadataObjects;
import com.sas.metadata.remote.PrimaryType;

// Set debug
Boolean debug = true;

// Get the script attributes
def host = attributes.get("host");
def port = attributes.get("port");
def user = attributes.get("user");
def password = attributes.get("passwd");
def metadataObjectIdentifier = attributes.get("fqid");
def metadataObjectType = attributes.get("type");
def includetype = attributes.get("includetype");
def objectpathproperty = attributes.get("objectpathproperty");
def rcPropName = attributes.get("resultproperty");

if (debug) {
   self.log("host: " + host);
   self.log("port: " + port);
   self.log("user: " + user);
   self.log("password: " + password);
   self.log("metadataObjectIdentifier: " + metadataObjectIdentifier );
   self.log("metadataObjectType: " + metadataObjectType);
   self.log("includetype: " + includetype);
   self.log("objectpathproperty: " + objectpathproperty);
   self.log("rcPropName: " + rcPropName);
}
 
// Initialize values used later.
String objp="";
rc = 0;

try {
   // Make a JOMA connection.
   def MdFactory       factory    = new MdFactoryImpl(false);
   def MdOMRConnection connection = factory.getConnection();
   def MdObjectStore   myStore    = factory.createObjectStore();
   def MdOMIUtil       omiUtil    = factory.getOMIUtil();

   connection.makeOMRConnection( host, port, user, password);
   String foundationID = omiUtil.getFoundationReposID();
   self.log("Repository ID: " + foundationID);
   
   // Create the xmlselect statement to identify the object.
   String objectSelect = "<xmlselect search=\"" + metadataObjectType + "[@Id eq '" + metadataObjectIdentifier + "']\"/>";
   
   if (debug)
      self.log("Check object select: " + objectSelect);

	
   int checkFlags = MdOMIUtil.OMI_GET_METADATA | MdOMIUtil.OMI_XMLSELECT | MdOMIUtil.OMI_ALL_SIMPLE;
   // This call loads the appropriate object into the local object store.
   List checkList = omiUtil.getMetadataObjectsSubset(myStore, foundationID, metadataObjectType, checkFlags, objectSelect);
   if (checkList.size() == 1) 
   {
     // Obtain the object path.
     objp = omiUtil.getObjectPath(myStore,(PrimaryType) myStore.getObject(metadataObjectIdentifier), Boolean.parseBoolean(includetype));
   }
   
   if (debug)
      self.log("Object list: " + checkList);
   if (debug)
      self.log("Object path: " + objp);
	  
   // Clean Up.	  
   myStore.dispose();
   connection.closeOMRConnection();
   factory.dispose();
}
catch (Throwable t) {
   rc=8;
   self.log("ERROR: Problem obtaining object path using xmlselect: " + objectSelect );
   t.printStackTrace();
   return;
}

if (objp.size() == 0)
{
self.log("WARNING: Object path not set.");
rc=4;
}

project.setNewProperty(rcPropName, "" + rc);
project.setNewProperty(objectpathproperty,"" + objp);

