import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdObjectStore;
import com.sas.metadata.remote.MdException;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdOMRConnection;
import com.sas.metadata.remote.MdOMIUtil;
import com.sas.metadata.remote.Document;
import com.sas.metadata.remote.TextStore;

// Get the script attributes from ant project
_host = attributes.get("host");
_port = attributes.get("port");
_user = attributes.get("user");
_password = attributes.get("password");
_sitenumber = attributes.get("sitenumber");
_setinittext = attributes.get("setinittext");
_rcPropName = attributes.get("resultproperty");

rc = 1;

try {

// Connect to the metadata server
	def String		m_repositoryFQID	= "";
	def String		m_repositoryID		= "";

	def MdFactory		m_mdFactory		= null;
	def MdOMIUtil		m_mdOMIUtil		= null;
	def MdObjectStore	m_mdObjectStore		= null;
	def Document		setinitDocument		= null;
	def TextStore		setinitTextStore	= null;

	def String		m_documentName		= "SAS Setinit Location";

	m_mdFactory = new MdFactoryImpl(false);
	MdOMRConnection connection = m_mdFactory.getConnection();
	connection.makeOMRConnection(_host, _port, _user, _password);

	m_mdObjectStore = m_mdFactory.createObjectStore();
	
	MdOMIUtil m_omiUtil = m_mdFactory.getOMIUtil();

	m_repositoryFQID = m_omiUtil.getFoundationReposID();
	
	// getFoundationReposID() returns an id of the form A0000001.A5IODDX9, we need the last 8 chars of the repository fqid for createComplexMetadataObject method below
	m_repositoryID = m_repositoryFQID.substring(9);

	self.log("Foundation Repository FQID = " + m_repositoryFQID);

	
	def listofobjects = m_mdFactory.getOMIUtil().getMetadataObjectsSubset(m_mdObjectStore, 
									m_repositoryFQID,
									"Document",
									MdOMIUtil.OMI_XMLSELECT |
									MdOMIUtil.OMI_TRUSTED_CLIENT |
									MdOMIUtil.OMI_ALL_SIMPLE |
									MdOMIUtil.OMI_TEMPLATE |
									MdOMIUtil.OMI_GET_METADATA,
									"<XMLSelect search=\"Document[@Name='" + m_documentName + "' and @TextRole='setinit']\" />");

	if ( listofobjects.size() == 0 ) {
		// no document was found so one must be created
		self.log("Creating a new Document object in this repository with Name='" + m_documentName + "' and TextRole='setinit' since none was found.");

		setinitDocument = (Document) m_mdFactory.createComplexMetadataObject(m_mdObjectStore, null, m_documentName, m_mdFactory.DOCUMENT, m_repositoryID);

		// Set the TextRole attribute of the Document
		setinitDocument.setTextRole("setinit");
		setinitDocument.updateMetadataAll();
	}
	else {
		// Objects found.  Since there can only be one setinit document, and the search returns the list, get the first item on the list
		setinitDocument = (Document) listofobjects.get(0);
		self.log("Found existing SAS Setinit Location Document Name=" + setinitDocument);
	}


	//now we have the SAS Setinit Document
	//search for an assocaited TextStore objects on the document with a matching site number 


	listofobjects = m_mdFactory.getOMIUtil().getMetadataObjectsSubset(m_mdObjectStore, 
									m_repositoryFQID,
									"TextStore",
									MdOMIUtil.OMI_XMLSELECT |
									MdOMIUtil.OMI_TRUSTED_CLIENT |
									MdOMIUtil.OMI_ALL_SIMPLE |
									MdOMIUtil.OMI_TEMPLATE |
									MdOMIUtil.OMI_GET_METADATA,
									"<XMLSelect search=\"Textstore[@Name='Setinit text' and @TextRole='" + _sitenumber + "'][Objects/Document[@Name='" + m_documentName + "' and @TextRole='setinit']]\" />");

	if ( listofobjects.size() == 0 ) {

		// We didn't find an existing TextStore for this Site so we need to create one

		setinitTextStore = (TextStore) m_mdFactory.createComplexMetadataObject(m_mdObjectStore, null, "Setinit text", m_mdFactory.TEXTSTORE, m_repositoryID);

		// Set the TextRole attribute of the Textstore to the site number that was passed in
		setinitTextStore.setTextRole(_sitenumber);
	
		// associate this TextStore with the document that was either found or created above
		setinitDocument.getNotes().add(setinitTextStore);
	
		setinitTextStore.updateMetadataAll();
		self.log("Created a new SAS Setinit TextStore for site number " + _sitenumber + " since none was found.");	

	}
	else {
		// Objects found.  Since there can only be one setinit document, and the search returns the list, get the first item on the list
		setinitTextStore = (TextStore) listofobjects.get(0);
		self.log("Found existing setinit TextStore for the site number " + _sitenumber + " so the existing setinit will be updated");
	}

	//Now add the contents of the setinit to the TextStore that was either found or created

	setinitTextStore.setStoredText(_setinittext);
	setinitTextStore.updateMetadataAll();


	// Now we are done we need to clean up all the objects created if we are no longer using them.
	// Now we are done we need to clean up all the objects created if we are no longer using them.
	m_mdObjectStore.dispose();

	// Disconnect from the metadata server
	m_mdFactory.closeOMRConnection();
	rc=0;



} catch (Throwable t) {
	t.printStackTrace();
	self.log("caught exception: " + t.toString());
	rc = 1;
}

// Set the return code property
project.setNewProperty(_rcPropName, "" + rc);

