import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdObjectStore;
import com.sas.metadata.remote.MdException;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdOMRConnection;
import com.sas.metadata.remote.MdOMIUtil;

import com.sas.metadata.remote.MetadataObjects;
import com.sas.metadata.remote.SoftwareComponent;
import com.sas.metadata.remote.Document;
import com.sas.metadata.remote.TextStore;
import com.sas.metadata.remote.AssociationList;

import com.sas.tools.webappconfig.tasks.GlobalRootProperties;


// Get the script attributes from ant project

String _host 			= attributes.get("host");
String _port 			= attributes.get("port");
String _user 			= attributes.get("user");
String _password		= attributes.get("password");
String _repository		= attributes.get("repository");

String _webappsrvFQID	= attributes.get("webappsrv.fqid");
String _contextroot		= attributes.get("contextroot");

String _outputPropName	= attributes.get("outputproperty");

String _output			= "True"; //this varaible holds the value that the returned ant property will be set to, suitable for istrue condition task.  Values other than "True" and "False" are error messages


try {

	//	 Connect to the metadata server
	String			m_repositoryFQID	= "";
	String			m_repositoryID		= "";

	MdFactory		m_mdFactory			= null;
	MdOMIUtil		m_mdOMIUtil			= null;
	MdObjectStore	m_mdObjectStore		= null;


	m_mdFactory = new MdFactoryImpl(false);
	MdOMRConnection connection = m_mdFactory.getConnection();
	connection.makeOMRConnection(_host, _port, _user, _password);

	m_mdObjectStore = m_mdFactory.createObjectStore();

	MdOMIUtil m_omiUtil = m_mdFactory.getOMIUtil();

	m_repositoryFQID = m_omiUtil.getFoundationReposID();

	// getFoundationReposID() returns an id of the form A0000001.A5IODDX9, we need the last 8 chars of the repository fqid for createComplexMetadataObject method below
	m_repositoryID = m_repositoryFQID.substring(9);

	self.log("Foundation Repository FQID = " + m_repositoryFQID);


	List listofobjects = m_mdFactory.getOMIUtil().getMetadataObjectsSubset(m_mdObjectStore,
									m_repositoryFQID,
									MetadataObjects.SOFTWARECOMPONENT,
									MdOMIUtil.OMI_XMLSELECT |
									MdOMIUtil.OMI_TRUSTED_CLIENT |
									MdOMIUtil.OMI_ALL_SIMPLE |
									MdOMIUtil.OMI_SUCCINCT |
									MdOMIUtil.OMI_GET_METADATA,
									"<XMLSelect search=\"SoftwareComponent[DependsOnComponents/SoftwareComponent[@Id='" + _webappsrvFQID + "']]\"/>");

	if ( listofobjects.isEmpty() ) {
		// no SoftwareComponents found as dependents of supplied webappsrv fqid so no webapps are currently configured
		self.log("No Metadata Objects were found.");
	}
	else {

		self.log(listofobjects.size() + " metadata objects found:");
		for (Iterator it=listofobjects.iterator(); it.hasNext(); ) {
			SoftwareComponent swc = (SoftwareComponent)it.next();
			self.log("Checking for collisions in configurable object for Web Application Named: " + swc.getName());

			// now search throuh all the properties that have "contextroot" in the property name, and if the value is the passed in _contextroot, then return True
			if(propertyValueInUse(swc, "context\\.*root", _contextroot)) {

				// we have a match so set our return value for ant and quit looking
				_output = "False";
				break;
			}
		} // end for loop (search)
	}// end else


	// Now we are done we need to clean up all the objects created if we are no longer using them.
	m_mdObjectStore.dispose();

	// Disconnect from the metadata server
	m_mdFactory.closeOMRConnection();

}
catch (Throwable t) {
	t.printStackTrace();
	self.log("caught exception: " + t.toString());
	_output = "-1";
}

// Set the return code property
project.setNewProperty(_outputPropName, "" + _output);

public boolean propertyValueInUse(SoftwareComponent swc, String regex, String propertyValue) {

	Properties props = GlobalRootProperties.getPropertiesfromConfigurableObj(swc,"all");

	Properties list = new Properties();

	Pattern   p      = Pattern.compile(regex, Pattern.CASE_INSENSITIVE);

	Iterator keys   = (props.keySet()).iterator();
	boolean detected = false;
	int matches = 0;

	while(keys.hasNext()) {

		String candidate = keys.next().toString();
		Matcher m = p.matcher(candidate);

		if(m.find()) {
			String candidateValue = props.getProperty(candidate);
			if(propertyValue.equals(candidateValue)) {
				self.log("\nA duplicate value of \""+propertyValue+"\" was found in property ["+ candidate +"] owned by configurable object ["+swc.getName()+"] using FQID ["+swc.getFQID()+"]");
				detected = true;
				break;
			}
		}
	}
	return detected;
}