import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;

// Velocity bug:
// http://issues.apache.org/jira/browse/VELOCITY-78
// http://issues.apache.org/jira/browse/VELOCITY-193

templateFile = args[0]			// Velocity template
configPropertiesFile = args[1]	// Config properties
l10nPropertiesFile = args[2]	// Localization properties
outputFile = args[3]			// File to save docpart to
templatePaths = args[4] 		// list of comma seperated directories 
append = args[5] 				// true or false
docpartPropertiesFile = args[6] // Properties file to track which output files were created from which templates.
encoding=args[7]				// Template Input/Output Encoding. Ex. UTF-8, etc.
debug = args[8]					// Enable debug information. true/false.

println "templateFile=$templateFile"
println "configPropertiesFile=$configPropertiesFile"
println "l10nPropertiesFile=$l10nPropertiesFile"
println "outputFile=$outputFile"
println "templatePaths=$templatePaths"
println "append=$append"
println "docpartPropertiesFile=$docpartPropertiesFile"
println "encoding=$encoding"
println "debug=$debug"
			
debug = new Boolean(debug).booleanValue()
append = new Boolean(append).booleanValue()

// Get log directory.
File logDir = new File(outputFile).getParentFile()
println "logDir=" + logDir.getAbsolutePath()	
File log = new File(logDir, "velocity.log")	
			
// Initialize Velocity Engine
Properties velocityProperties = new Properties();    
velocityProperties.put("resource.loader","file");
velocityProperties.put("file.resource.loader.description", "Velocity File Resource Loader");
velocityProperties.put("file.resource.loader.path", templatePaths)
velocityProperties.put("file.resource.loader.class", "org.apache.velocity.runtime.resource.loader.FileResourceLoader");				
velocityProperties.put("resource.manager.logwhenfound", "true")
velocityProperties.put("directive.foreach.counter.name", "velocityCount")
velocityProperties.put("directive.foreach.iterator.name", "velocityHasNext")
velocityProperties.put("directive.foreach.counter.initial.value", "1")
velocityProperties.put("runtime.log", log.getAbsolutePath()) 
velocityProperties.put("input.encoding", encoding)
velocityProperties.put("output.encoding", encoding)
velocityProperties.put("runtime.log.invalid.references", "true")


velocityProperties.put("velocimacro.library", "macros.vm")
velocityEngine = new VelocityEngine();
velocityEngine.init(velocityProperties);

// Track output files by template.
Properties dp = new Properties()
if( ! (new File(docpartPropertiesFile)).exists() ) {
	dp.store(new FileOutputStream(docpartPropertiesFile), null);
}
// Replace periods in template names with underscore to make it velocity friendly.
dp.load(new FileInputStream(docpartPropertiesFile))
String key = templateFile.replaceAll("\\.", "_")
String value = dp.get(key)
if(value == null) {
	dp.put(key, (new File(outputFile)).getName())
}
else {
	if( ! value.contains((new File(outputFile)).getName()) ) {
		dp.put(key, value + "," + (new File(outputFile)).getName())
	}
}
dp.store(new FileOutputStream(docpartPropertiesFile), null);	

// Load Template Properties
configProperties = new Properties()
configProperties.load(new FileInputStream(configPropertiesFile))
configContext = new VelocityContext(configProperties)

docpartProperties = new Properties()
docpartProperties.load(new FileInputStream(docpartPropertiesFile))
docpartContext = new VelocityContext(docpartProperties, configContext)

l10nProperties = new Properties()
if( (new File(l10nPropertiesFile)).exists() ) {
	l10nProperties.load(new FileInputStream(l10nPropertiesFile))
}
l10nContext = new VelocityContext(l10nProperties, docpartContext)	

if(debug) {
	println "Begin: Velocity Context"
	l10nContext.getKeys().each{ vProperty ->
		println "$vProperty=" + l10nContext.get(vProperty) 
	}
    println "End: Velocity Context"	
}

// Merge and Write DocPart File
template = velocityEngine.getTemplate(templateFile, encoding);	
OutputStreamWriter fw = new OutputStreamWriter(new FileOutputStream(new File(outputFile), append),encoding)			
template.merge(l10nContext, fw);
fw.close()			
	