// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config

import java.io.File
import java.util.Map
import org.apache.velocity.VelocityContext
import org.apache.velocity.app.VelocityEngine

public class VelocityTemplateEngine implements TemplateEngine {
	private String encoding = "UTF-8"
	private List<File> templatePaths = new ArrayList<File>()
	private File logDirectory
	private String fileResourceLoaderClass = "org.apache.velocity.runtime.resource.loader.FileResourceLoader"
	
	public VelocityTemplateEngine() {}
	
	public VelocityTemplateEngine(File templatePath, File logDirectory, String encoding) {
		List<File> paths = new ArrayList<File>()
		paths.add(templatePath)
		setTemplatePaths(paths)
		setLogDirectory(logDirectory)
		setEncoding(encoding)
	}		
	
	public VelocityTemplateEngine(List<File> templatePaths, File logDirectory, String encoding) {		
		setTemplatePaths(templatePaths)
		setLogDirectory(logDirectory)
		setEncoding(encoding)
	}
	
	public VelocityTemplateEngine(File templatePath, File logDirectory) {
		List<File> paths = new ArrayList<File>()
		paths.add(templatePath)
		setTemplatePaths(paths)
		setLogDirectory(logDirectory)
	}	
	
	public VelocityTemplateEngine(List<File> templatePaths, File logDirectory) {
		setTemplatePaths(templatePaths)
		setLogDirectory(logDirectory)
	}
	
	public void setEncoding(String e) { encoding = e }
	public void setTemplatePaths(List<File> paths) { templatePaths = paths }
	public void setLogDirectory(File dir) { logDirectory = dir }
	public void setFileResourceLoaderClass(String clazz) { fileResourceLoaderClass = clazz }
	
	// TODO: Move implementation to merge(Context, templateFile, outputFile) when removing.
	@Deprecated
	public void merge(Context context, File templateFile, String encoding, File outputFile) {
		Properties velocityProperties = new Properties()
		velocityProperties.put("resource.manager.logwhenfound", "true")
		velocityProperties.put("directive.foreach.counter.name", "velocityCount")
		velocityProperties.put("directive.foreach.iterator.name", "velocityHasNext")
		velocityProperties.put("directive.foreach.counter.initial.value", "1")
		velocityProperties.put("runtime.log.invalid.references", "false") // Disabling for unix shell scripts.
		velocityProperties.put("resource.loader","file");
		velocityProperties.put("file.resource.loader.description", "Velocity File Resource Loader");
		velocityProperties.put("file.resource.loader.class", fileResourceLoaderClass);		
		velocityProperties.put("file.resource.loader.unicode", "true")
		velocityProperties.put("file.resource.loader.encoding", encoding)
		
		velocityProperties.put("input.encoding", encoding)
		velocityProperties.put("output.encoding", encoding)
		//velocityProperties.put("runtime.log.logsystem.class", "org.apache.velocity.runtime.log.AvalonLogChute")
		
		StringBuffer sbTemplatePaths = new StringBuffer()
		templatePaths.each() { path ->
			if(sbTemplatePaths.length() > 0) { sbTemplatePaths.append(", ") }
			sbTemplatePaths.append(path.getAbsolutePath())				
		}
		velocityProperties.put("file.resource.loader.path", sbTemplatePaths.toString())						
		velocityProperties.put("velocimacro.library", "macros.vm")
		velocityProperties.put("runtime.log", logDirectory.getAbsolutePath() + "/velocity.log") 		
		
		VelocityEngine velocityEngine = new VelocityEngine()
		velocityEngine.init(velocityProperties)			
		
		println ">>> Begin Context"
		println ((context != null) ? context.toString() : context)
		println "<<< End Context"
		
		VelocityContext velocityContext = new VelocityContext(context)			
		def template = velocityEngine.getTemplate(templateFile.getName(), encoding)	
		OutputStreamWriter fw = new OutputStreamWriter(new FileOutputStream(outputFile, false),encoding)			
		template.merge(velocityContext, fw)
		fw.close()					
	}
	
	public void merge(Context context, File templateFile, File outputFile) {
		merge(context, templateFile, encoding, outputFile)
	}
}
