// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config;

import java.io.InputStream
import java.io.FileInputStream
import java.io.InputStreamReader;

import org.apache.commons.collections.ExtendedProperties
import org.apache.velocity.exception.ResourceNotFoundException
import org.apache.velocity.runtime.resource.loader.FileResourceLoader

/**
 * Converts (java) escaped unicode characters (i.e. \u0041 ) in a template file to its native equivalent. 
 * Use of this class restricts a template from containing the "backslash u" sequence for non-unicode strings. 
 */
class VelocityUnicodeFileResourceLoader extends FileResourceLoader {
	private static String encoding = "UTF-8"
	
	public void init( ExtendedProperties configuration) {
		super.init(configuration)
		encoding = configuration.getString("encoding", encoding)
	}
	
	@Override
	public InputStream getResourceStream(String templateName) throws ResourceNotFoundException {
		String text = readAndConvertToNativeChars(super.getResourceStream(templateName))
		return new ByteArrayInputStream(text.getBytes(encoding))
	}
	
	// TODO: Groovy Bug in version 1.7.1 prevents this method signature from using InputStream instead of FileInputStream.
	//       Should Fix this method once SDW upgrades its version of groovy. 
	public static String readAndConvertToNativeChars(FileInputStream is) {
		StringBuilder sb = new StringBuilder()
				
		PushbackReader pis = new PushbackReader(new InputStreamReader(is, encoding))
		
		int c
		while((c=pis.read()) != -1) {			
			if(c == "\\") {
				int nextChar = pis.read()
				if(nextChar == "u" ) {
					try {
						char[] hexArray = new char[4]
						pis.read(hexArray, 0, hexArray.length)
						String hex = new String(hexArray)
						
						c = (char) Integer.parseInt(hex, 16)
					}
					catch(IOException ioe) {
						ioe.printStackTrace()
						throw new RuntimeException(ioe, "Invalid unicode escape sequence.")
					}
				}
				else {
					pis.unread(nextChar)
					sb.append((char)c)
				}
			}			
			sb.append((char)c)			
		}
		pis.close()
		return sb.toString()
	}
}
