// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
import org.w3c.dom.NodeList
//import org.apache.commons.io.FileUtils

class Application extends Resource {
    String name;
    String pathtoear;
    String servername;
    String deploymentdir;
    String explode;
    
    String resourceType = "application"
    Application(){}
    Application(String appname,String p,String s,String d,String x ){
        name = appname
        pathtoear = p
        servername = s
        deploymentdir = d
        explode = x
    }
    /*
		application.app1.appname=SASThemes_default
		application.app1.pathtoear=C:/Users/sasgww/Sas/test_config_9.2/Lev1/Web/Staging/sas.themes.ear
		application.app1.servername=SASServer1
		application.app1.deploymentdir=deploy_sas
		application.app1.explode=false
    */
    int configure(String instance, Properties scriptProperties){
        try {
    
        Logger.log('DEBUG', "application - Configuring ${instance}")
        name = scriptProperties.get("application."+instance+".appname")
        pathtoear = scriptProperties.get("application."+instance+".pathtoear")  
        servername = scriptProperties.get("application."+instance+".servername")  
        deploymentdir = scriptProperties.get("application."+instance+".deploymentdir")  
        explode = scriptProperties.get("application."+instance+".explode")
		
        def jbossInstallDir = scriptProperties.'config.jboss.install.dir'        
		def version = new JBossVersion(jbossInstallDir)
        
        if (deploymentdir == null) deploymentdir = "deploy_sas"
        
        def deploymentpath = "$jbossInstallDir/server/$servername/$deploymentdir" 
		def fileFilterMetadata
		if (version.getVersion()=="6"){
			fileFilterMetadata = "$jbossInstallDir/server/$servername/deployers/scanning-deployers-jboss-beans.xml"
		}else{
			fileFilterMetadata = "$jbossInstallDir/server/$servername/deployers/metadata-deployer-jboss-beans.xml"		
		}

		Logger.log('DEBUG', "application - appname=$name")
        Logger.log('DEBUG', "application - pathtoear=$pathtoear")
        Logger.log('DEBUG', "application - servername=$servername")
        Logger.log('DEBUG', "application - deploymentdir=$deploymentdir")
        Logger.log('DEBUG', "application - explode=$explode")
        Logger.log('DEBUG', "application - deploymentpath=$deploymentpath")
		Logger.log('DEBUG', "application - fileFilterMetadata=$fileFilterMetadata")
		
            //Ant.touchDir(deploymentpath)
            if (explode == 'true'){
            	Utils.explode(pathtoear, deploymentpath, deploymentpath+'/'+"temp")
				boolean foundMetadata=(new File(fileFilterMetadata)).exists()
				Logger.log('DEBUG', "application - Looking for fileFilterMetadata : $foundMetadata")
				
				if (foundMetadata) { //If not present, probably not JBoss 5, don't attempt optimization
				   updateVirtualFileFilter(pathtoear,deploymentpath, fileFilterMetadata)
				}
				// removed due to S0750022
				//else {
				//   Logger.log('DEBUG', "application - can't find fileFilterMetadata=$fileFilterMetadata")
				//}
            }else {
               Ant.copyFileToDir(pathtoear, deploymentpath, "true")    
            }
         }
         catch (Throwable e) {
             Logger.log('DEBUG', "Failure : " + e.toString());
             e.printStackTrace();
             return 8;
         }
         return 0;
    }
	
	
	
	/*
	<bean class="org.jboss.deployers.vfs.spi.structure.helpers.VirtualFileDeploymentUnitFilter" name="JBossCustomDeployDUFilter">
		<constructor>
		  <parameter>
			<bean class="org.jboss.deployment.FileNameVirtualFileFilter">
			  <constructor>
				<parameter>
				  <map keyClass="java.lang.String" valueClass="java.util.Set">
					
					<entry>
					  <key>jbossweb.sar</key>
					  <value>
						 <null/>
					  </value>
					</entry>

 */

	void updateVirtualFileFilter(String archive, String deploymentDir, String filterFile){
		Logger.log('DEBUG', "application - updateVirtualFileFilter archive=$archive")
		Logger.log('DEBUG', "application - updateVirtualFileFilter deploymentDir=$deploymentDir")
		Logger.log('DEBUG', "application - updateVirtualFileFilter filterFile=$filterFile")
		
		File archiveFile = new File(archive)
		def archiveName=archiveFile.getName()
		String applicationXml = "$deploymentDir/$archiveName/META-INF/application.xml"
		def applicationParser = new Parser(applicationXml)
		NodeList list = applicationParser.findNodes("//web-uri")
		if (list != null){
			def filterParser = new Parser(filterFile)
			def updateFilterFile=null
			def xpathFilterString='//bean[@name="JBossCustomDeployDUFilter"]/constructor/parameter/bean/constructor/parameter/map'
			
			def filterMapNode = filterParser.findNode(xpathFilterString)
			if (filterMapNode){
				for (node in list) {
					def textNode = node.getFirstChild();
					def currentText=null
					if (textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
					   currentText=textNode.getNodeValue()
					   if (filterMapNode) {
						   //Search and see if we need to add new entry with archive key
						   def existingKeyNode = filterParser.findTextNode(filterMapNode, 'entry/key', currentText)
						   if (existingKeyNode==null) {	
							   updateFilterFile=true	  
							   Logger.log('DEBUG', "application - updateVirtualFileFilter adding war=$currentText")					   
		                       def entryNode = filterParser.addNode("entry","",filterMapNode)
							   def keyNode = filterParser.addNode("key", "$currentText", entryNode)
							   def valueNode = filterParser.addNode("value","",entryNode)
							   def nullNode = filterParser.addNode("null","",valueNode)	
						   }else {
						       Logger.log('DEBUG', "application - updateVirtualFileFilter found war=$currentText")					   
						   }
						   									
					   }
				    }
			    }
			}
			if (updateFilterFile) filterParser.output(filterFile)
	    }
	}
}
