// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
//import org.apache.commons.io.FileUtils

class Datasource extends Resource {
	
    String resourceType = "datasource"
    String name;
    String jndiname;
    String servername;
    String classpath;
    String driver;
    String xa;
    String connectionUrl;
    String user;
    String password;
	String encoding;
    String driverOptions;
    String hostName;
    String serverPort;
    Collection<DatasourceOption> dumpedOptions
    
    Datasource() {}
    Datasource(String n,String j, String s, String c, String d, String x,String url, String u, String p, String o, String host, String port){
        name = n
        jndiname = j
        servername = s
        classpath = c
        driver = d
        xa = x
        connectionUrl = url
        user = u
        password = p
        driverOptions = o
        hostName = host
        serverPort = port

        dumpedOptions = new ArrayList<DatasourceOption>();
        
        for (option in driverOptions.tokenize(',')){

            Logger.log('TRACE', "datasource - looping over options: option = $option")
                def namevalue = option.split("=")
            Logger.log('TRACE', "datasource - looping over options: namevalue = $namevalue")
                def optionName = namevalue[0]
            Logger.log('TRACE', "datasource - looping over options: optionName = $optionName")
                def value = ""	
				
				value = option - (optionName+"=")
                
				Logger.log('DEBUG', "datasource - processing options: optionName=$optionName value=$value");
                if (
                    ! optionName.compareTo("blocking-timeout-millis") || 
                    ! optionName.compareTo("check-valid-connection-sql") || 
                    ! optionName.compareTo("exception-sorter-class-name") ||
                    ! optionName.compareTo("idle-timeout-minutes") ||
                    ! optionName.compareTo("isSameRM-override-value") ||
                    ! optionName.compareTo("max-pool-size") ||
                    ! optionName.compareTo("min-pool-size") ||
                    ! optionName.compareTo("new-connection-sql") ||
                    ! optionName.compareTo("no-tx-separate-pools") ||
                    ! optionName.compareTo("prepared-statement-cache-size") ||
                    ! optionName.compareTo("share-prepared-statements") ||
                    ! optionName.compareTo("track-connection-by-tx") ||
                    ! optionName.compareTo("track-statements") ||
                    ! optionName.compareTo("transaction-isolation") ||
                    ! optionName.compareTo("use-java-context") ||
                    ! optionName.compareTo("valid-connection-checker-class-name") ||
                    ! optionName.compareTo("xa-datasource-class")
                   ) {
                       dumpedOptions.add(new DatasourceOption(optionName,value));            
                }
                /*
                else if (! optionName.compareTo("type-mapping") ){
                    def metadata = datasourceParser.addNode("metadata","",tx_datasource)
                    datasourceParser.addNode(optionName,value,metadata)
                }
                */
                else if ( ! optionName.compareTo("security-domain")){

                    dumpedOptions.add(new DatasourceOption("security-domain",value));              
                }

                else {
                    def tx_datasource_property
                    if (xa == "true") {
                      tx_datasource_property = "xa-datasource-property"          
                    } else {
                      tx_datasource_property = "connection-property"          
                    }
                    dumpedOptions.add(new DatasourceOption(optionName, value, tx_datasource_property))
                } 
         
            }
            if (user!=null && user!="not-set"){
                dumpedOptions.add(new DatasourceOption("user-name", user))
                dumpedOptions.add(new DatasourceOption("password", "xxxxxxxx"))
            }
            
        
    }

	Collection<String> classPathList(){
		return 	classpath.tokenize(";").collect {jars -> jars as String}

	}
	
	int configure(String instance, Properties scriptProperties){
        try {
    
		Logger.log('DEBUG', "datasource - Configuring ${instance}")

		name = scriptProperties.get("datasource."+instance+".name")
		jndiname = scriptProperties.get("datasource."+instance+".jndiname")  
		servername = scriptProperties.get("datasource."+instance+".servername")
		classpath = scriptProperties.get("datasource."+instance+".classpath")
		driver = scriptProperties.get("datasource."+instance+".driver")
		xa = scriptProperties.get("datasource."+instance+".xa")
		connectionUrl = scriptProperties.get("datasource."+instance+".connectionUrl")
		user = scriptProperties.get("datasource."+instance+".user")
		password = scriptProperties.get("datasource."+instance+".password")
		encoding = scriptProperties.get("datasource."+instance+".encoding")
		driverOptions = scriptProperties.get("datasource."+instance+".options")
		hostName = scriptProperties.get("datasource."+instance+".hostName")
		serverPort = scriptProperties.get("datasource."+instance+".serverPort")
		//def jbossDatasourceOptions = scriptProperties.get("datasource."+instance+".jbossDatasourceOptions")
		def jbossInstallDir = scriptProperties.'config.jboss.install.dir'

		Logger.log('DEBUG',"datasource - datasourcename=${name}")
		Logger.log('DEBUG',"datasource - jndiname=${jndiname}")
		Logger.log('DEBUG',"datasource - servername=${servername}")
		Logger.log('DEBUG',"datasource - classpath=${classpath}")
		Logger.log('DEBUG',"datasource - driver=${driver}")
		Logger.log('DEBUG',"datasource - xa=${xa}")
		Logger.log('DEBUG',"datasource - connectionUrl=${connectionUrl}")
		Logger.log('DEBUG',"datasource - user=${user}")
		Logger.log('DEBUG',"datasource - password=xxxxxx")
		Logger.log('DEBUG',"datasource - driverOptions=${driverOptions}")
		Logger.log('DEBUG',"datasource - hostName=${hostName}")
		Logger.log('DEBUG',"datasource - serverPort=${serverPort}")
		//Logger.log('DEBUG',"datasource - jbossDatasourceOptions=${jbossDatasourceOptions}")
		Logger.log('DEBUG',"datasource - jbossInstallDir=${jbossInstallDir}")
        
		def datasourceFile = new String("${jbossInstallDir}/server/${servername}/deploy/${name}-ds.xml")
		
		//Don't have commons IO, so using ant...
		//FileUtils.touch( new File( datasourceFile ) )  
		Ant.touch(datasourceFile)
		def datasourceParser = new Parser()     
		
		def tx_datasource

	    String options=driverOptions
	    //if (jbossDatasourceOptions != "not-set")	
		//   options += "," + jbossDatasourceOptions
		
		def datasourceNode = datasourceParser.addNode("datasource","",datasourceParser.getDocument())

		if (xa == "true") {
			tx_datasource = datasourceParser.addNode("xa-datasource","",datasourceNode)
			datasourceParser.addNode("xa-datasource-class",driver,tx_datasource)
		} else {
		    tx_datasource = datasourceParser.addNode("local-tx-datasource","",datasourceNode)
		    datasourceParser.addNode("driver-class",driver,tx_datasource)
		    datasourceParser.addNode("connection-url",connectionUrl, tx_datasource)
		}
		Logger.log('DEBUG', "datasource - options = $options")
		for (option in options.tokenize(',')){
		    Logger.log('TRACE', "datasource - looping over options: option = $option")
				def namevalue = option.split("=")
		    Logger.log('TRACE', "datasource - looping over options: namevalue = $namevalue")
			    def optionName = namevalue[0]
		    Logger.log('TRACE', "datasource - looping over options: optionName = $optionName")
				def value = ""

				value = option - (optionName+"=")
				
				Logger.log('DEBUG', "datasource - processing options: optionName=$optionName value=$value");
			    if (
				    ! optionName.compareTo("blocking-timeout-millis") || 
				    ! optionName.compareTo("check-valid-connection-sql") || 
				    ! optionName.compareTo("exception-sorter-class-name") ||
				    ! optionName.compareTo("idle-timeout-minutes") ||
				    ! optionName.compareTo("isSameRM-override-value") ||
				    ! optionName.compareTo("max-pool-size") ||
				    ! optionName.compareTo("min-pool-size") ||
				    ! optionName.compareTo("new-connection-sql") ||
				    ! optionName.compareTo("no-tx-separate-pools") ||
				    ! optionName.compareTo("prepared-statement-cache-size") ||
				    ! optionName.compareTo("share-prepared-statements") ||
				    ! optionName.compareTo("track-connection-by-tx") ||
				    ! optionName.compareTo("track-statements") ||
				    ! optionName.compareTo("transaction-isolation") ||
				    ! optionName.compareTo("use-java-context") ||
				    ! optionName.compareTo("valid-connection-checker-class-name") ||
				    ! optionName.compareTo("xa-datasource-class")
				   ) {
				    datasourceParser.addNode(optionName,value,tx_datasource)         	
				}
				else if (! optionName.compareTo("type-mapping") ){
					def metadata = datasourceParser.addNode("metadata","",tx_datasource)
					datasourceParser.addNode(optionName,value,metadata)
				}
				else if ( ! optionName.compareTo("security-domain")){
				    user = null //Don't use username or password elements
				    datasourceParser.addNode(optionName,value,tx_datasource)          
				}
				else {
				    def tx_datasource_property
				    if (xa == "true") {
					tx_datasource_property = datasourceParser.addNode("xa-datasource-property",value,tx_datasource)          
				    } else {
					tx_datasource_property = datasourceParser.addNode("connection-property",value,tx_datasource)          
				    }
				    datasourceParser.addAttribute("name",optionName,tx_datasource_property)
				}        	
			}
			
		if (jndiname!=null){
		    datasourceParser.addNode("jndi-name", jndiname, tx_datasource)          
		}
			
		if (user!=null){
			    datasourceParser.addNode("user-name", user, tx_datasource)
				String prompt = "\nEnter Password \n\tDatasource=$name \n\tuser=$user \n\tPassword="
				password = com.sas.config.jboss.Utils.encodeDecodePassword(password, encoding, prompt)
				datasourceParser.addNode("password", password, tx_datasource)
		}

		datasourceParser.output(datasourceFile)
        
		if (classpath!=null){
		   def targetDir = new String("${jbossInstallDir}/server/${servername}/lib")
		   Logger.log('DEBUG',"datasource - touch $targetDir")
		   Ant.touchDir(targetDir)
		   for (sourceJar in classpath.tokenize(';')){
		      Logger.log('DEBUG', "datasource - copy $sourceJar $targetDir")
        
		      Ant.copyFileToDir(sourceJar,targetDir, "true")
        
		   }
		}

     }
     catch (Throwable e) {
             Logger.log('DEBUG', "Failure : " + e.toString());
             e.printStackTrace();
             return 8;
     }
     return 0;
	}
	
	int changeHost(String instance, Properties scriptProperties){
		try {
	
		Logger.log('DEBUG', "datasource - ChangeHost ${instance}")

		name = scriptProperties.get("datasource."+instance+".name")
		jndiname = scriptProperties.get("datasource."+instance+".jndiname")
		servername = scriptProperties.get("datasource."+instance+".servername")
		classpath = scriptProperties.get("datasource."+instance+".classpath")
		driver = scriptProperties.get("datasource."+instance+".driver")
		xa = scriptProperties.get("datasource."+instance+".xa")
		connectionUrl = scriptProperties.get("datasource."+instance+".connectionUrl")
		user = scriptProperties.get("datasource."+instance+".user")
		password = scriptProperties.get("datasource."+instance+".password")
		encoding = scriptProperties.get("datasource."+instance+".encoding")
		driverOptions = scriptProperties.get("datasource."+instance+".options")
		hostName = scriptProperties.get("datasource."+instance+".hostName")
		serverPort = scriptProperties.get("datasource."+instance+".serverPort")
		//def jbossDatasourceOptions = scriptProperties.get("datasource."+instance+".jbossDatasourceOptions")
		def jbossInstallDir = scriptProperties.'config.jboss.install.dir'

		Logger.log('DEBUG',"datasource - datasourcename=${name}")
		Logger.log('DEBUG',"datasource - jndiname=${jndiname}")
		Logger.log('DEBUG',"datasource - servername=${servername}")
		Logger.log('DEBUG',"datasource - classpath=${classpath}")
		Logger.log('DEBUG',"datasource - driver=${driver}")
		Logger.log('DEBUG',"datasource - xa=${xa}")
		Logger.log('DEBUG',"datasource - connectionUrl=${connectionUrl}")
		Logger.log('DEBUG',"datasource - user=${user}")
		Logger.log('DEBUG',"datasource - password=xxxxxx")
		Logger.log('DEBUG',"datasource - driverOptions=${driverOptions}")
		Logger.log('DEBUG',"datasource - hostName=${hostName}")
		Logger.log('DEBUG',"datasource - serverPort=${serverPort}")
		//Logger.log('DEBUG',"datasource - jbossDatasourceOptions=${jbossDatasourceOptions}")
		Logger.log('DEBUG',"datasource - jbossInstallDir=${jbossInstallDir}")
		def tx_datasource="local-tx-datasource"
		if (connectionUrl)
		{
			def datasourceFile = new String("${jbossInstallDir}/server/${servername}/deploy/${name}-ds.xml")		
			def datasourceParser = new Parser(datasourceFile)
			if (xa == "true") {
				tx_datasource = "xa-datasource"
			}
			def connectionUrlNode = datasourceParser.findNode("//datasource/$tx_datasource/connection-url")			
			if (connectionUrlNode)
			{
				Logger.log('DEBUG', "datasource - updating connection-url with $connectionUrl")
				datasourceParser.addNode("connection-url",connectionUrl, connectionUrlNode, "Sibling")
				datasourceParser.removeNode(connectionUrlNode)
				datasourceParser.output(datasourceFile)
			}
			else
			{
				Logger.log('DEBUG', "datasource - can't locate connection-url node.")
				Logger.log('DEBUG', "datasource - won't update connection-url.")
			}
		}
		else
		{
			Logger.log('DEBUG', "datasource - no connectionUrl found in datasource properties")
		}
	 }
	 catch (Throwable e) {
			 Logger.log('DEBUG', "Failure : " + e.toString());
			 e.printStackTrace();
			 return 8;
	 }
	 return 0;
	}

}

