// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
//import org.apache.commons.io.FileUtils

class Mailsession extends Resource {
    
    String resourceType = "mailsession"
    String name;
    String jndiname;
    String servername;
    String mailhost;
    String user;
    String password;
    
    Mailsession() {}
	Mailsession(mailsessionname,j,s,h){
		name = mailsessionname
		jndiname = j
		servername = s
		mailhost = h
		user=null
		password=null
	}
    Mailsession(mailsessionname,j,s,h,u,p ){
        name = mailsessionname
        jndiname = j
        servername = s
        mailhost = h
        user = u
        password = p
    }
    /*
    mailsession.ms1.name=sas-mail-service
    mailsession.ms1.jndiname=sas/mail/Session
    mailsession.ms1.mailhost=mailhost.fyi.sas.com
    mailsession.ms1.user=user@foobar.com
    mailsession.ms1.password=xxxxxx
    mailsession.ms1.servername=SASServer1
    */
    int configure(String instance, Properties scriptProperties){
        try {

        Logger.log('DEBUG', "mailsession - Configuring ${name}")

        name = scriptProperties.get("mailsession."+instance+".name")
        jndiname = scriptProperties.get("mailsession."+instance+".jndiname")  
        mailhost = scriptProperties.get("mailsession."+instance+".mailhost")
        user = scriptProperties.get("mailsession."+instance+".user")
        password = scriptProperties.get("mailsession."+instance+".password")
        servername = scriptProperties.get("mailsession."+instance+".servername")
        def jbossInstallDir = scriptProperties.'config.jboss.install.dir'

        Logger.log('DEBUG',"mailsession - jbossInstallDir=${jbossInstallDir}")
        Logger.log('DEBUG',"mailsession - servername=${servername}")
        Logger.log('DEBUG',"mailsession - mailsession name=${name}")
        
        def mailsessionFile = new String("${jbossInstallDir}/server/${servername}/deploy/${name}.xml")
		
        	
        createWrapperScript(mailsessionFile, jndiname, mailhost, user, password)
         }
         catch (Throwable e) {
             Logger.log('DEBUG', "Failure : " + e.toString());
             e.printStackTrace();
             return 8;
         }
         return 0;
    }   
	
	int changeHost(String instance, Properties scriptProperties){
		try {

		Logger.log('DEBUG', "mailsession - ChangeHost ${name}")

		name = scriptProperties.get("mailsession."+instance+".name")
		jndiname = scriptProperties.get("mailsession."+instance+".jndiname")
		mailhost = scriptProperties.get("mailsession."+instance+".mailhost")
		user = scriptProperties.get("mailsession."+instance+".user")
		password = scriptProperties.get("mailsession."+instance+".password")
		servername = scriptProperties.get("mailsession."+instance+".servername")
		def jbossInstallDir = scriptProperties.'config.jboss.install.dir'

		Logger.log('DEBUG',"mailsession - jbossInstallDir=${jbossInstallDir}")
		Logger.log('DEBUG',"mailsession - servername=${servername}")
		Logger.log('DEBUG',"mailsession - mailsession name=${name}")
		
		def mailsessionFile = new String("${jbossInstallDir}/server/${servername}/deploy/${name}.xml")
		if (mailhost)
		{
			def mailsessionParser = new Parser(mailsessionFile)
			def mailhostNode = mailsessionParser.findNode("//server/mbean/attribute[@name='Configuration']/configuration/property[@name='mail.smtp.host']")
			if (mailhostNode)
			{
				Logger.log('DEBUG', "mailsession - updating mailhost with $mailhost")
				mailsessionParser.addAttribute("value",mailhost, mailhostNode)
				mailsessionParser.output(mailsessionFile)
			}
			else
			{
				Logger.log('DEBUG', "mailsession - can't locate mailhost node.")
				Logger.log('DEBUG', "mailsession - won't update mailhost.")
			}
		}
		else
		{
			Logger.log('DEBUG', "mailsession - no mailhost found in scripting properties")
		}

			
		 }
		 catch (Throwable e) {
			 Logger.log('DEBUG', "Failure : " + e.toString());
			 e.printStackTrace();
			 return 8;
		 }
		 return 0;
	}

def createWrapperScript(
		 String filename,
         String jndiname,
         String mailhost,
         String username,
         String password
     ){  
     def wrapperScript = '''<?xml version="1.0" encoding="UTF-8"?>

<server>
  <mbean code="org.jboss.mail.MailService" name="jboss:service=mail/Session">
    <attribute name="JNDIName">$jndiname</attribute>
 ${password?'   <attribute name="User">'+username+'</attribute>':''}
 ${password?'   <attribute name="Password">'+password+'</attribute>':''}
    <attribute name="Configuration">
        <configuration>
            <property name="mail.transport.protocol" value="smtp"/>
 ${password?'           <property name="mail.smtp.auth" value="true"/>':''}
            <property name="mail.smtp.host" value="$mailhost"/>
            <property name="mail.smtp.port" value="25"/>
            <property name="mail.debug" value="false"/>

        </configuration>
    </attribute>
  </mbean>
</server>
 '''      
        def engine = new groovy.text.SimpleTemplateEngine()
        def template = engine.createTemplate(wrapperScript)
        def decodedpassword = null

        //
        //mailsession can't have an encoded password. Force decode.
        //
		if (password != null) {
		     String prompt = "\nEnter Password \n\tMail Session \n\tUserID=$user \n\tPassword="
		     decodedpassword = com.sas.config.jboss.Utils.encodeDecodePassword(password, "false", prompt)
        }
        def binding = [
            jndiname: jndiname,
            mailhost : mailhost,
            username : username,
            password : decodedpassword, 
              ]
        def scriptOut = template.make(binding)
        def outFile = new File(filename)
        outFile.write(scriptOut.toString())        
 }

}