// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved


package com.sas.config.jboss

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
import org.w3c.dom.NodeList

class Parser {
    String xmlInFile
    String xmlOutFile
    DocumentBuilderFactory builderFactory
    DocumentBuilder builder
    Document document
    TransformerFactory transformerFactory
    Transformer transformer
    
    Parser(){
        builderFactory = DocumentBuilderFactory.newInstance();
        //Hack to turn off validating/loading of dtd's.  Don't need it.
            builderFactory.setValidating(false);
            builderFactory.setAttribute("http://apache.org/xml/features/nonvalidating/load-external-dtd",false);
        builder = builderFactory.newDocumentBuilder();
        document = builder.newDocument();
        
        transformerFactory = TransformerFactory.newInstance()
        
        Logger.log('DEBUG',"TransformaerFactory.getClass.name =  " + transformerFactory.class.name);
        
        try {
           transformerFactory.setAttribute("indent-number", "4")
        } catch (IllegalArgumentException e1) {
           // Can't seem to use indent-number with this transformer impl
           Logger.log('DEBUG',"Unable to set 'indent-number', will use default value from Transformer ");
        }

        transformer = transformerFactory.newTransformer()
        
        if (transformerFactory.class.name ==  "org.apache.xalan.processor.TransformerFactoryImpl")
        {
            Logger.log('DEBUG',"Found Xalan, calling transformer.setOutputProperty indent-amount instead of relying on TransformerFactory.setAttribute");
            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4") 
        }
        
    }
    Parser(String inFile){
        builderFactory = DocumentBuilderFactory.newInstance();
        //Hack to turn off validating/loading of dtd's.  Don't need it.
            builderFactory.setValidating(false);
            builderFactory.setAttribute("http://apache.org/xml/features/nonvalidating/load-external-dtd",false);
        builder = builderFactory.newDocumentBuilder();
        transformerFactory = TransformerFactory.newInstance()

        try {
           transformerFactory.setAttribute("indent-number", "4")
        } catch (IllegalArgumentException e1) {
           // Can't seem to use indent-number with this transformer impl
           Logger.log('DEBUG',"Unable to set 'indent-number', will use default value from Transformer ");
        }

        transformer = transformerFactory.newTransformer()
        if (transformerFactory.class.name ==  "org.apache.xalan.processor.TransformerFactoryImpl")
        {
            Logger.log('DEBUG',"Found Xalan, calling transformer.setOutputProperty indent-amount instead of relying on TransformerFactory.setAttribute");
            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4") 
        }

        parse(inFile)
    }
    Parser(String inFile, String indentNumber){
        builderFactory = DocumentBuilderFactory.newInstance();
        //Hack to turn off validating/loading of dtd's.  Don't need it.
            builderFactory.setValidating(false);
            builderFactory.setAttribute("http://apache.org/xml/features/nonvalidating/load-external-dtd",false);
        builder = builderFactory.newDocumentBuilder();
        transformerFactory = TransformerFactory.newInstance()

        try {
           transformerFactory.setAttribute("indent-number", indentNumber)
        } catch (IllegalArgumentException e1) {
           // Can't seem to use indent-number with this transformer impl
           Logger.log('DEBUG',"Unable to set 'indent-number', will use default value from Transformer ");
        }

        transformer = transformerFactory.newTransformer()
        if (transformerFactory.class.name ==  "org.apache.xalan.processor.TransformerFactoryImpl")
        {
            Logger.log('DEBUG',"Found Xalan, calling transformer.setOutputProperty indent-amount instead of relying on TransformerFactory.setAttribute");
            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", indentNumber) 
        }

        parse(inFile)
    }
    
    
    void addAttribute(String attr,String value,String xpathParm){
        def doc = getDocument()      
        def element = doc.documentElement
        def xpath = XPathFactory.newInstance().newXPath()
        // Find element and update the attribute
        Logger.log('TRACE',"addAttribute: ")
        Logger.log('TRACE',"addAttribute: attr = $attr ")
        Logger.log('TRACE',"addAttribute: value = $value ")
        Logger.log('TRACE',"addAttribute: xpathParm = $xpathParm")
        Logger.log('TRACE',"addAttributee: ")
        def nodeList=null
        def node=null
        
        try {
           nodeList = xpath.evaluate(xpathParm, element, XPathConstants.NODESET)
        } 
        catch(Exception e){
          Logger.log('DEBUG',"addAttribute: can not evaluate xpath - $xpathParm")
          Logger.log('DEBUG',"addAttribute: attribute will not get updated")
        }
        if (nodeList)
        {
            node = nodeList.item(0)
            
            // If we can't find this element, no sense in attempting to do any work...
            if (node)
            {
                node.setAttribute(attr,value);
                Logger.log('DEBUG',"addAttribute: Set $attr at $xpathParm")
            } else{
                Logger.log('DEBUG',"addAttribute: Unable to locate $xpathParm")
            }
         }
   }
    void addAttribute(String attr,String value,Node node){
        def doc = getDocument()      
        def element = doc.documentElement
        // Find element and update the attribute
        Logger.log('TRACE',"addAttribute: ")
        Logger.log('TRACE',"addAttribute: attr = $attr ")
        Logger.log('TRACE',"addAttribute: value = $value ")
        Logger.log('TRACE',"addAttribute: node = $node")
        Logger.log('TRACE',"addAttributee: ")
        
        // If we can't find this element, no sense in attempting to do any work...
        if (node)
        {
            node.setAttribute(attr,value);
            Logger.log('DEBUG',"addAttribute: Set $attr")
        } else{
            Logger.log('DEBUG',"addAttribute: Unable to locate node")
        }
   }
    
    Node addComment(String comment){
        def doc = getDocument()
        def commentNode
        commentNode = doc.createComment(comment)
        return commentNode
    }
    
    Node addComment(String comment, Node sibling){
        def doc = getDocument()
        def commentNode
        commentNode = doc.createComment(comment)
        sibling.getParentNode().insertBefore(commentNode,sibling)
        return commentNode
    }
    
    Node addNode(String name, String value, String xpathParm){
        //     profileNode = profileXmlParser.addNode("value","${jboss.server.home.url}deploy_sas",'//bean[@name="BootstrapProfileFactory"]/property[@name="applicationURIs"]/list')
        def doc = getDocument()      
        def element = doc.documentElement
        def xpath = XPathFactory.newInstance().newXPath()
        // Find element and update the attribute
        Logger.log('TRACE',"addNode: ")
        Logger.log('TRACE',"addNode: name = $name ")
        Logger.log('TRACE',"addNode: value = $value ")
        Logger.log('TRACE',"addNode: xpathParm = $xpathParm")
        Logger.log('TRACE',"addNode: ")
        
        def node
        
        if (xpathParm == ""){
            node = element
        }else{
            node = xpath.evaluate(xpathParm, element, XPathConstants.NODESET).item(0)
        }
        Node newElement = null
        // If we can't find this element, no sense in attempting to do any work...
        if (node)
        {           
            newElement=doc.createElement(name);
            def whitespaceTextNode=doc.createTextNode("   ");  // Add in three spaces               
            
            if (value != ""){
                def newTextElement=doc.createTextNode(value);
                newElement.appendChild(newTextElement);
            }

            node.appendChild(newElement);
            node.insertBefore(whitespaceTextNode,newElement);
        }else {
            Logger.log('DEBUG',"Can't find node $xpathParm")
        }

        return newElement

    }

    Node addNode(String name, String value, Node node){
        
        return addNode(name, value, node, "CHILD")
        
    }

    Node addNode(String name, String value, Node node, String childOrSibling){
        //    keyNode = vfsXmlParser.addNode("key","${jboss.server.home.url}deploy_sas",entryNode)
        def doc = getDocument()      
        def element = doc.documentElement
        Node newElement = null

        // Find element and update the attribute
        Logger.log('TRACE',"addNode: ")
        Logger.log('TRACE',"addNode: name = $name ")
        Logger.log('TRACE',"addNode: value = $value ")
        Logger.log('TRACE',"addNode: parent node = $node")
        Logger.log('TRACE',"addNode: ")

        // If we can't find this element, no sense in attempting to do any work...
        if (node)
        {           
            newElement=doc.createElement(name);
            def whitespaceTextNode=doc.createTextNode("   ");  // Add in three spaces               
            
            if (value != ""){
                def newTextElement=doc.createTextNode(value);
                newElement.appendChild(newTextElement);
            }

            if (childOrSibling == "CHILD"){
                node.appendChild(newElement);
            }else{
                node.getParentNode().insertBefore(newElement,node)
            }
                
            //node.insertBefore(whitespaceTextNode,newElement);
        }else {
            Logger.log('DEBUG',"Can't find node $node")
        }
        return newElement

    }

    void addTextNode(String value,String xpathParm){
        def doc = getDocument()      
        def element = doc.documentElement
        def xpath = XPathFactory.newInstance().newXPath()
        def node
        // Find element and update the attribute
        Logger.log('TRACE',"addTextNode: ")
        Logger.log('TRACE',"addTextNode: value = $value ")
        Logger.log('TRACE',"addTextNode: xpathParm = $xpathParm")
        Logger.log('TRACE',"addTextNode: ")
        
        if (xpathParm == ""){
            node = element
        }else{
            node = xpath.evaluate(xpathParm, element, XPathConstants.NODESET).item(0)
        }
        
        // If we can't find this element, no sense in attempting to do any work...
        if (node)
        {
           def newTextNode=doc.createTextNode(value)
           Logger.log('DEBUG',"addTextNode -   newText="+newTextNode.getNodeValue())
           node.appendChild(newTextNode)
        }else {
            Logger.log('DEBUG',"Can't find node $node")
        }
        
   }

   void addTextNode(String value, Node node, String childOrSibling){
        def doc = getDocument()      
        def element = doc.documentElement

        // Find element and update the attribute
        Logger.log('TRACE',"addTextNode: ")
        Logger.log('TRACE',"addTextNode: value = $value ")
        Logger.log('TRACE',"addTextNode: node = $node ")
        Logger.log('TRACE',"addTextNode: childOrSibling = $childOrSibling ")
        Logger.log('TRACE',"addTextNode: ")
        
        // If we can't find this element, no sense in attempting to do any work...
        if (node)
        {
           def newTextNode=doc.createTextNode(value)
           Logger.log('DEBUG',"addTextNode -   newText="+newTextNode.getNodeValue())
           if (childOrSibling == "CHILD"){
               node.appendChild(newTextNode);
           }else{
               node.getParentNode().insertBefore(newTextNode,node)
           }
        }else {
            Logger.log('DEBUG',"Can't find node $node")
        }
        
   }

   void addTextNode(String value, Node node){
      addTextNode(value, node, "CHILD")
   }
   
   NodeList findNodes(String xpathParm){
        def nodeList=null
        def doc = getDocument()              
        def element = doc.documentElement
        def xpath = XPathFactory.newInstance().newXPath()
        
        nodeList = xpath.evaluate(xpathParm, element, XPathConstants.NODESET)
        
        Logger.log('TRACE',"findNodes: ")
        Logger.log('TRACE',"findNodes: xpath = $xpathParm")
        Logger.log('TRACE',"findNodes: nodeList = $nodeList")
        Logger.log('TRACE',"findNodes: nodeList count = " + nodeList.getLength())
        Logger.log('TRACE',"findNodes: nodeList = $nodeList")
        for (node in nodeList) {
                Logger.log('DEBUG',"findNodes: node = $node")
        }
        
        // If we can't find any elements, no sense in attempting to do any work...
        if (nodeList){
            Logger.log('DEBUG',"findNodes: found $xpathParm")
        } else{
            Logger.log('DEBUG',"findNodes: Unable to locate $xpathParm")
        }
        return nodeList
   }
   
   Node findTextNode(String xpathParm, String textValue){

        def foundNode=null
        def doc = getDocument()              
        def element = doc.documentElement
        
        foundNode =  findTextNode(element, xpathParm, textValue)
                
        return foundNode
   }
   
   Node findTextNode(Node parentNode, String xpathParm, String textValue){
        def nodeList=null
        def xpath = XPathFactory.newInstance().newXPath()
        def foundNode=null
        
        nodeList = xpath.evaluate(xpathParm, parentNode, XPathConstants.NODESET)
        
        Logger.log('TRACE',"findTextNode: ")
        Logger.log('TRACE',"findTextNode: xpath = $xpathParm")
        Logger.log('TRACE',"findTextNode: nodeList = $nodeList")
        Logger.log('TRACE',"findTextNode: nodeList count = " + nodeList.getLength())
        Logger.log('TRACE',"findTextNode: nodeList = $nodeList")
        for (node in nodeList) {
           Logger.log('TRACE',"findTextNode: node = $node")
           def textNode = node.getFirstChild();
           def currentText=null
           if (textNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
              currentText=textNode.getNodeValue()
              Logger.log('DEBUG',"findTextNode -   currentText="+currentText)
              if (currentText == textValue) {
                 Logger.log('DEBUG',"findTextNode - found $textValue : $node")
                 foundNode = node
              }               
           }
        }
        
        // If we can't find any elements, no sense in attempting to do any work...
        if (foundNode){
            Logger.log('DEBUG',"findTextNode: found $xpathParm with value of $textValue")
        } else{
            Logger.log('DEBUG',"findTextNode: Unable to locate $xpathParm with value of $textValue")
        }
        return foundNode
   }
   
   
   Node findNode(String xpathParm){
        Node node = null
        def doc = getDocument()      
        def element = doc.documentElement
        def xpath = XPathFactory.newInstance().newXPath()
        // Find element and update the attribute
        Logger.log('TRACE',"findNode: ")
        Logger.log('TRACE',"findNode: xpathParm = $xpathParm")
        Logger.log('TRACE',"findNode: ")
        
        node = xpath.evaluate(xpathParm, element, XPathConstants.NODESET).item(0)
        
        // If we can't find this element, no sense in attempting to do any work...
        if (node){
            Logger.log('DEBUG',"findNode: found $xpathParm")
        } else{
            Logger.log('DEBUG',"findNode: Unable to locate $xpathParm")
        }
        return node
   }

   void output(String outFile){
        xmlOutFile = outFile
        Logger.log('DEBUG',"   dumping xml to file: $outFile ");
        
        StringWriter sw = new StringWriter()
        
        transformer.setOutputProperty(OutputKeys.INDENT, "yes"); 
        transformer.transform(new DOMSource(document), new StreamResult(sw))
        new File(xmlOutFile).withWriter{ it << sw }        
        Logger.log('DEBUG',"   dumping xml to file...Finished ");
     }
     
   Document parse(String inFile){
         return parse(new File(inFile))
      }

   Document parse(File inFile){
	   document = builder.parse(new FileInputStream(inFile));
	   return document
	}

   void removeNode(Node node){
	   node.getParentNode().removeChild(node)
   }
   void setSystemOutputProperty(String value){
    	 transformer.setOutputProperty("doctype-system",value)
     }

   //
   //JBoss 5.1 has moved some configurable components into CDATA sections
   // to allow them to be injected dynamically.  Therefore, we need the ability
   // to update these sections as required.
   //   
   Node updateCDATANode(String value,Node node){
	   def doc = getDocument()
	   Node oldCDATANode=null
	   def newCDATA=""
	   def currentCDATA=""
		   
	Logger.log('TRACE',"updateCDATANode: ")
	   Logger.log('TRACE',"updateCDATANode: value = $value ")
	   Logger.log('TRACE',"updateCDATtNode: node = $node")
	   
	   // If we can't find this element, no sense in attempting to do any work...
	   if (node)
	   {
		   oldCDATANode = node.getFirstChild();
		   while (oldCDATANode.getNodeType() != org.w3c.dom.Node.CDATA_SECTION_NODE){
			   oldCDATANode = oldCDATANode.getNextSibling()
		   }
		   
		   if (oldCDATANode.getNodeType() == org.w3c.dom.Node.CDATA_SECTION_NODE)
		   {
			  currentCDATA=oldCDATANode.getNodeValue()
			  def newCDATANode=doc.createCDATASection(value)
			  Logger.log('DEBUG',"updateCDATANode -   currentCDATA="+currentCDATA)
			  Logger.log('DEBUG',"updateCDATANode -   newCDATA="+newCDATANode.getNodeValue())
			  node.removeChild(oldCDATANode)
			  node.appendChild(newCDATANode)
			  return newCDATANode
		   } else {
		      Logger.log('DEBUG',"This is not a CDATA node - $oldCDATANode")
		      Logger.log('DEBUG',"  it is a " + oldCDATANode.getNodeType())
		   }
		   
	   }else {
		   Logger.log('DEBUG',"Can't find CDATA node $node")
	   }
	   
  }

   void updateTextNode(String value,String xpathParm){
       def doc = getDocument()      
       def element = doc.documentElement
       def xpath = XPathFactory.newInstance().newXPath()
       // Find element and update the attribute
       Logger.log('TRACE',"updateTextNode: ")
       Logger.log('TRACE',"updateTextNode: value = $value ")
       Logger.log('TRACE',"updateTextNode: xpathParm = $xpathParm")
       Logger.log('TRACE',"updateTextNode: ")
       
       def node = xpath.evaluate(xpathParm, element, XPathConstants.NODESET).item(0)
       updateTextNode(value, node)
              
  }
   
   void updateTextNode(String value,Node node){
       def doc = getDocument()      
       Logger.log('TRACE',"updateTextNode: ")
       Logger.log('TRACE',"updateTextNode: value = $value ")
       Logger.log('TRACE',"updateTextNode: node = $node")
       
       // If we can't find this element, no sense in attempting to do any work...
       if (node)
       {
           def oldTextNode = node.getFirstChild();
           def newText=""
           def currentText=""
           if (oldTextNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE)
           {
              currentText=oldTextNode.getNodeValue()
              def newTextNode=doc.createTextNode(value)
              Logger.log('DEBUG',"updateXMLTextNode -   currentText="+currentText)
              Logger.log('DEBUG',"updateXMLTextNode -   newText="+newTextNode.getNodeValue())
              node.removeChild(oldTextNode)
              node.appendChild(newTextNode)
           }
       }else {
           Logger.log('DEBUG',"Can't find node $node")
       }
       
  }
   void updateTextNode(String value,String xpathParm, String compareText){
       def doc = getDocument()      
       def element = doc.documentElement
       def xpath = XPathFactory.newInstance().newXPath()
       // Find element and update the attribute
       Logger.log('TRACE',"updateTextNode: ")
       Logger.log('TRACE',"updateTextNode: value = $value ")
       Logger.log('TRACE',"updateTextNode: xpathParm = $xpathParm")
       Logger.log('TRACE',"updateTextNode: ")
       
       def node = xpath.evaluate(xpathParm, element, XPathConstants.NODESET).item(0)
       
       // If we can't find this element, no sense in attempting to do any work...
       if (node)
       {
           def oldTextNode = node.getFirstChild();
           def newText=""
           def currentText=""
           if (oldTextNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE)
           {
              currentText=oldTextNode.getNodeValue()
              if (currentText == compareText){
              def newTextNode=doc.createTextNode(value)
              Logger.log('DEBUG',"updateXMLTextNode -   currentText="+currentText)
              Logger.log('DEBUG',"updateXMLTextNode -   newText="+newTextNode.getNodeValue())
              node.removeChild(oldTextNode)
              node.appendChild(newTextNode)
              }
              else{
                  Logger.log('DEBUG',"updateXMLTextNode -   can not locate currentText="+currentText)            	  
              }
           }
       }else {
           Logger.log('DEBUG',"Can't find node $node")
       }
       
  }
}
