// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

public class ResourceProperties {
 int result = 0;
 
 ResourceProperties(){}
 
 ResourceProperties(ResourceInterface ri, HashMap<String,String> resourceMap, String propFile, String webappsrvAutoConfigure) {
	 configure( ri, resourceMap, propFile, webappsrvAutoConfigure, true, true)
 }
 
 ResourceProperties(ResourceInterface ri, HashMap<String,String> resourceMap, String propFile, String webappsrvAutoConfigure, Boolean persist) {
     configure( ri, resourceMap, propFile, webappsrvAutoConfigure, persist, true)
 }
 	 
ResourceProperties(ResourceInterface ri, HashMap<String,String> resourceMap, String propFile, String webappsrvAutoConfigure, Boolean persist, Boolean cacheCredentials) {
	 configure( ri, resourceMap, propFile, webappsrvAutoConfigure, persist, cacheCredentials)	
 }
void configure(ResourceInterface ri, HashMap<String,String> resourceMap, String propFile, String webappsrvAutoConfigure, Boolean persist, Boolean cacheCredentials) {
 
   def globalPropFile = propFile
   
   File property_file = null
   
   def passwordFile =""
   
   def inmemoryProperties = new SortedProperties()
   def persistedProperties = new SortedProperties()
   def passwordProperties = new SortedProperties()
   
   // String loginmodules
   String resources
   
   String resourceName = null   // server1, server2, datasource2, mailsession1 ...
   String resourceType = ri.resourceType //server, datasource, mailsession

   def resourceList
   def resourceID

   def name
   def count=0

   Logger.logLevel = "DEBUG"
   Logger.log('DEBUG',"resource: BEGIN configure for resource $resourceType properties")
   
   try {
	   property_file = new File(globalPropFile)
	   passwordFile = property_file.getParent() + "/jbossScriptingCredentials.properties"
	   
	   persistedProperties.load(new FileInputStream(globalPropFile))
	   passwordProperties.load(new FileInputStream(passwordFile))	   
   } catch(Exception e){
       e.printStackTrace();    
   }    
   def listseparator = persistedProperties.'listseparator'
   if (listseparator==null) listseparator = / /

   Logger.log('DEBUG',"resource: resourceType  = $resourceType")
   resources = persistedProperties."${resourceType}"
   Logger.log('DEBUG',"resource: resources  = $resources")

   count++;
   if (resources == null) {
       resourceName = resourceType+count
	   if (persist) {
            Logger.log('DEBUG',"resource: Found no resources, create new: $resourceType = $resourceName")
            persistedProperties.put(resourceType, resourceName)
	   }
   }
   else {
	  Logger.log('DEBUG',"resource: Found resources, checking count - resources=$resources resourceType = $resourceType") 
      while (resources.contains(resourceType+count)){
    	  Logger.log('DEBUG',"resource: Found resources, increment count - resources=$resources resourceType = $resourceType") 
    	  count++
      }
	  Logger.log('DEBUG',"resource: Found resources : split")
      
	  resourceList= resources.split(listseparator)
      
      Logger.log('DEBUG',"resource: Found resources : done with split")
      
      def match
      def persistedValue=null
      Logger.log('DEBUG',"resource: Found resources : resourceList = $resourceList")
      //Logger.log('DEBUG',"resource: Found resources : resourceMap = $resourceMap")
	  printResourceMap(resourceMap)
      resourceList.each{ resource ->
	      Logger.log('DEBUG',"resource: Found resources : it=$resource")
    	  Logger.log('DEBUG',"resource: Checking $resourceType.$resource")
    	  match=true
          // 
    	  // Make sure we haven't already seen this entry.  If so, re-use the resourceName
    	  // Check each key of each resource.  If the key is password or user, need to load
    	  // the value from the appropriate properties object.
    	  //
          //    Logger.log('DEBUG',"resource: Found resources : resourceMap = $resourceMap")              
    	  resourceMap.each { key, value ->
    	      if (key.contains("config.") || key.contains("options") || key.contains("service.dependency")) {
    	    	  return //dont bother comparing any properties other than resourceType properties - ignore "options" when considering replacement
    	      }else if (key.contains("password") || key.contains("user") || key.contains("encoding")){
					  persistedValue = passwordProperties."${resourceType}.${resource}.${key}"
			  }else {
				      persistedValue = persistedProperties."${resourceType}.${resource}.${key}"
			  }
              Logger.log('DEBUG',"resource:  comparing $key ")
	    	  if (value!=persistedValue) {
	    		  match=false
                  Logger.log('DEBUG',"resource:  comparing $key - $key does not match persistedValue")
	    	  }
    	  }

          Logger.log('DEBUG',"resource: comparing $resourceType.$resource : match=$match")
         
          if (match) resourceName=resource
          Logger.log('DEBUG',"resource: Found resources : done counting resources: resourceName=$resourceName")
       }

      if (resourceName == null){
	      resourceName=resourceType+count	      
		  if (persist) {
	          persistedProperties.put(resourceType, resources + listseparator + resourceName)
		  }
      }
      Logger.log('DEBUG',"resource: resourceName = $resourceName")
   }
   
   if (!persist) {
	   // Remove "$resourceName" from resourceType value, then remove all properties based on "$resourceName".
	   // get rid of leading spaces and replace any double space substrings with single space delimiter
	   // extra single space can occur at the front of the list when removing the first entry
	   // extra double space can occur in the middle of the list when removing a middle entry - we 
	   // only match (and remove) the actual matched name, not surrounding spaces, so need to strip those after.
       Logger.log('DEBUG',"resource: updating persisted key : $resourceType")
       Logger.log('DEBUG',"resource: updating persisted key : original value : $resources")
       Logger.log('DEBUG',"resource: updating persisted key : removing resourceName=$resourceName")
	   if (resources){	   
		   // only attempt removal if we have resources to fix
		   def newValue=(resources - resourceName).trim().replaceAll("  ", " ")
		   
		   //After trimming spaces, do we have any left?  If not, remove the key.
		   if (newValue.size()==0) {
			   persistedProperties.remove(resourceType)
		       Logger.log('DEBUG',"resource: no values left, removing key : $resourceType ")
		   } else {
		       persistedProperties.put(resourceType, newValue )
		       Logger.log('DEBUG',"resource: new value for key : $resourceType = $newValue")
		   }
		   
		   def propertyList = persistedProperties.propertyNames()
		   propertyList.each {key ->
		       if (key.contains(resourceType+"."+resourceName)){
		    	   Logger.log('DEBUG',"resource: removing persisted key : $key")
		    	   persistedProperties.remove(key)
		       }
		   }
		   propertyList = passwordProperties.propertyNames()
		   propertyList.each {key ->
		       if (key.contains(resourceType+"."+resourceName)){
		    	   Logger.log('DEBUG',"resource: removing persisted key : $key")
				   passwordProperties.remove(key)
		       }
		   }
	   }
   }
   
   inmemoryProperties.put(resourceType, resourceName)
   
   resourceMap.each { key, value ->
       if (key.contains("config.")) {
           persistedProperties.put(key,value)
           inmemoryProperties.put(key,value)       	   
       }else{	   
    	   if (persist)
    	   {
    	      //Lets go load up the properties with the new values
    	      // if we are caching credentials (storing passwords with sas encoding into a file),
    	      // make sure the passwords are encoded.  If we are not caching, store all passwords
    	      // with the value "not-cached".
    	      //
		      Logger.log('DEBUG',"resource: checking key for password -> key=$key")
		      if (key.contains("password")){
		          Logger.log('DEBUG',"resource: found password key=$key")
				  if (cacheCredentials){
					 def encodedpw=com.sas.util.SasPasswordString.encode(com.sas.util.SasPasswordString.decode(value))
                     passwordProperties.put(resourceType+"."+resourceName+"."+key,encodedpw)
                     //Logger.log('DEBUG',"resource: found password key=$key, storing '$encodedpw'")
				  }else{
				     passwordProperties.put(resourceType+"."+resourceName+"."+key,"not-cached")
                     Logger.log('DEBUG',"resource: found password key=$key, storing 'not-cached'")
				  }				  
		      }else if (key.contains("user") || key.contains("encoding") ){
			         Logger.log('DEBUG',"resource: found user key=$key")
			         passwordProperties.put(resourceType+"."+resourceName+"."+key,value)
    	      }else{        	   
		             Logger.log('DEBUG',"resource: found NON-password key=$key")
				     persistedProperties.put(resourceType+"."+resourceName+"."+key,value)
		      }
		   }        	   
	       inmemoryProperties.put(resourceType+"."+resourceName+"."+key,value)   
       }
   }


   try { 
       String comments = new String("SAS Scripting for JBoss")
       FileOutputStream globalOutfile = new FileOutputStream(globalPropFile)
       FileOutputStream passOutfile = new FileOutputStream(passwordFile)
	   
	   assert persistedProperties instanceof java.util.Properties
	   assert globalOutfile instanceof FileOutputStream
	
	   Logger.log('DEBUG',"resource: storing properties for $resourceName to $globalPropFile")
	   Logger.log('TRACE',"$persistedProperties")
	   persistedProperties.store(globalOutfile,comments)
	   
	   Logger.log('DEBUG',"resource: storing credentials for $resourceName to $passwordFile")
	   Logger.log('TRACE',"$passwordProperties")

	   passwordProperties.store(passOutfile,comments)
	   
	   } catch (Exception e){
	       e.printStackTrace()
	       throw(e)
	   }  

   
   Logger.log('DEBUG'," **** webappsrv.auto_configure = $webappsrvAutoConfigure ")
   Logger.log('DEBUG'," **** persist = $persist ")

//   if (persist && webappsrvAutoConfigure == "true" )  Value could be mixed case...Defect S0717752
      if (persist && webappsrvAutoConfigure.equalsIgnoreCase("true")) {
          result = ri.configure(resourceName, inmemoryProperties)
   }
   Logger.log('DEBUG',"resource: END configure for resource $resourceType properties")

 }
 void printResourceMap(HashMap<String,String> resourceMap){
	Logger.log('DEBUG',"print resources map: ")
	resourceMap.each { key, value ->
		 if(key!=null){
		 if(key.toLowerCase().contains("password")||key.toLowerCase().contains("pwd")){
			 Logger.log('DEBUG',"$key=XXXXXXXXXXXXXXXX")
		 }else{
			 Logger.log('DEBUG',"$key=$value")
		 }
		 }
		}
	
	}

}