// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss

import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.parsers.DocumentBuilder
import javax.xml.transform.dom.DOMSource
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.TransformerFactory
import javax.xml.transform.Transformer
import javax.xml.transform.OutputKeys
import javax.xml.xpath.XPathFactory
import javax.xml.xpath.XPathConstants
import org.w3c.dom.Document
import org.w3c.dom.Node
import org.w3c.dom.NodeList
import com.sas.config.util.Chmod

//import org.apache.commons.io.FileUtils

class Server  extends Resource
{

    String resourceType = "server"

    String name;
    String httpport;
    Boolean jmssecurity;
    String jmssecurityuser;
    String jmssecuritypw;
	String encoding;
    String jvm_option_list;
    String wrapperdir;
    String portIncrement;
    String serverSource;
    
    Collection<String> options;
    Collection<Loginmodule> loginmodules;
    Collection<Policy> policies;
    Collection<Connectionfactory> connectionfactories;
    Collection<Queue> queues;
    Collection<Topic> topics;
    Collection<Datasource> datasources;
	Collection<JDBCProvider> jdbcproviders;
    Collection<Mailsession> mailsessions;
    Collection<Application> applications;
    Collection<Externalcontext> externalcontexts;
    Server() {}
    
   
    /* 
     server.server1.name=SASServer1 
     server.server1.portIncrement=0
     server.server1.options=-Xms768m,-Xmx768m,-DSpecialOptionWithSpaces="This option has spaces",-Xss128k,-Xmso32k,-Xgcpolicy:optavgpause,-Djavax.management.builder.initial=,-Dcom.sun.management.jmxremote,-Dsun.rmi.dgc.client.gcInterval=3600000,-Dsun.rmi.dgc.server.gcInterval=3600000,-Djava.awt.headless=true,-Dsas.container.identifier=websphere,-Dsas.auto.publish.port=9080,-Dcom.sas.services.logging.disableRemoteList=true,-Dcom.sas.services.logging.disableRemoteLogging=true,-Djava.net.preferIPv4Stack=true,-Djava.net.preferIPv6Addresses=false,-Dmulticast_udp_ip_ttl=32,-Dmulticast.address=239.12.11.244,-Dmulticast.port=8561,-Dcom.sas.log.config.url=file:///D:/config/Lev1/Web/Common/LogConfig/
     server.server1.source=[all,default]
     server.server1.transaction=[JTS,] (not used for "default")
     server.server1.jmssecurity=[true,false,]
     server.server1.jmssecurity.user=[sasJmsUser,,]
     server.server1.jmssecurity.password=[sasJmsPW,,]
     
	 config.jboss.install.dir=C:\\jboss\\jboss-eap-5.0\\jboss-as
	 config.jdk.install.dir=C:\\Java\\jdk1.6.0_18
	 config.jboss.bind.host=d71067.na.sas.com
     */
    int configure(String instance, Properties scriptProperties){
        try {

        Logger.log('DEBUG', "server - Configuring ${instance}")

        name = scriptProperties.get("server."+instance+".name")  
        jvm_option_list = scriptProperties.get("server."+instance+".options") 
        portIncrement = scriptProperties.get("server."+instance+".portIncrement")
        
        def jbossBindHost = scriptProperties.get("config.jboss.bind.host")
        def jbossInstallDir = scriptProperties.get("config.jboss.install.dir")
        def jdkInstallDir = scriptProperties.get("config.jdk.install.dir")
        wrapperdir = scriptProperties.get("config.tanuki.wrapper.dir")
        def configHostType = scriptProperties.get("config.host.type")
        
        serverSource = scriptProperties.get("server."+instance+".source")
        def transaction = scriptProperties.get("server."+instance+".transaction")
        def jmsSecurity = scriptProperties.get("server."+instance+".jmssecurity")
        def jmsSecurityUser = scriptProperties.get("server."+instance+".jmssecurity.user")
        def jmsSecurityPW = scriptProperties.get("server."+instance+".jmssecurity.password")
		def encoding = scriptProperties.get("server."+instance+".jmssecurity.encoding")
        def allPolicyFile = scriptProperties.get("server."+instance+".all.policy.file")
        def restrictivePolicyFile = scriptProperties.get("server."+instance+".restrictive.policy.file")
        def jbossNamingPort
//        def messagingDir = scriptProperties.get("server."+instance+".messagingDir")
        def jbossServiceDependency = scriptProperties.get("server."+instance+".service.dependency")
        
        def multicastAutoToken = ""        
//        def version = new JBossVersion(jbossInstallDir+"/bin/run.jar") 
		def version = new JBossVersion(jbossInstallDir)
		def messagingDir = version.getMessagingDir()
		
        Logger.log('DEBUG',"server - jbossInstallDir=${jbossInstallDir}")
        Logger.log('DEBUG',"server - jdkInstallDir=${jdkInstallDir}")
        Logger.log('DEBUG',"server - wrapperDir=${wrapperdir}")
        Logger.log('DEBUG',"server - servername=${name}")
        Logger.log('DEBUG',"server - portIncrement=${portIncrement}")
        Logger.log('DEBUG',"server - options=${jvm_option_list}")
        Logger.log('DEBUG',"server - source=${serverSource}")
        Logger.log('DEBUG',"server - transaction=${transaction}")
        
        if (version.getVersion()=="4"){
            Logger.log('INFO', "server - JBoss version is " + version.getFullVersion())
            
            //JBoss version 4 has a bug in run.bat.  Need to get rid of 
            // "-Xms128m -Xmx512m" from the line 
            // containing "set JAVA_OPTS=%JAVA_OPTS% -Xms128m -Xmx512m"
            def runBat = jbossInstallDir+"/bin/run.bat"
            Map<String,String> replaceMap = new HashMap<String,String>()
            replaceMap.put("set JAVA_OPTS=%JAVA_OPTS% -Xms128m -Xmx512m", "set JAVA_OPTS=%JAVA_OPTS%")
            Utils.replace (new File(runBat), replaceMap) 
        } else {
            Logger.log('INFO', "server - JBoss version is " + version.getFullVersion())
        }

        if (serverSource == null){
        	serverSource = "default"
        }
        //Copy default configuration as a baseline - Use Ant since we don't have apache common io   
        def targetDir=jbossInstallDir+"/server/"+name
        def sourceDir=jbossInstallDir+"/server/"+serverSource
        Ant.copyDir(sourceDir,targetDir,"log/ tmp/ work/", "false")
        
        if ((serverSource == 'all' || serverSource == 'standard') && transaction == 'JTS' && version.getVersion()!="4"){
        	
            //Update jbossts-properties.xml
            def tsSourceFile=jbossInstallDir+"/docs/examples/transactions/jbossts-properties.xml"
            def tsTargetDir=jbossInstallDir+"/server/"+name+"/conf"
            def jbosstsPropertyFile="${tsTargetDir}/jbossts-properties.xml"
            def xpathString='//properties[@name="recoverymanager"]/property[@value="com.arjuna.ats.internal.jta.recovery.arjunacore.XARecoveryModule"]'
            def jbosstsParser = new Parser()
            Logger.log('DEBUG',"server - copy ${tsSourceFile} ${tsTargetDir}")
        	Ant.copyFileToDir(tsSourceFile,tsTargetDir,"true")
            jbosstsParser.parse(jbosstsPropertyFile)            
            def arjunacoreXARecoveryNode=jbosstsParser.findNode(xpathString)
            if (arjunacoreXARecoveryNode!=null){
            	Logger.log('DEBUG',"server - Removing ${xpathString} from jbossts-properties.xml")
            	jbosstsParser.removeNode(arjunacoreXARecoveryNode)
            }
            jbosstsParser.output(jbosstsPropertyFile)
        	
            //Update transaction-jboss-beans.xml
            def trBeansFile=jbossInstallDir+"/server/"+name+"/deploy/transaction-jboss-beans.xml"
            def trBeansParser = new Parser()
            Logger.log('DEBUG',"server - update ${trBeansFile}")
            trBeansParser.parse(trBeansFile)            
            
            def trManagerNode = trBeansParser.findNode('//bean[@name="TransactionManager"]')
            
            Logger.log('DEBUG',"server - replace JTA with JTS in TransactionManager bean")
            trBeansParser.addAttribute("class","com.arjuna.ats.jbossatx.jts.TransactionManagerService",trManagerNode)
            
            Logger.log('DEBUG',"server - replace JTA with JTS in TransactionManager/annotation bean")
            def newTextNode='@org.jboss.aop.microcontainer.aspects.jmx.JMX(name="jboss:service=TransactionManager", exposedInterface=com.arjuna.ats.jbossatx.jts.TransactionManagerServiceMBean.class, registerDirectly=true)'
            def oldTextNode='@org.jboss.aop.microcontainer.aspects.jmx.JMX(name="jboss:service=TransactionManager", exposedInterface=com.arjuna.ats.jbossatx.jta.TransactionManagerServiceMBean.class, registerDirectly=true)'
            xpathString='//bean[@name="TransactionManager"]/annotation'
            trBeansParser.updateTextNode(newTextNode,xpathString,oldTextNode)

//        <start>
//           <parameter>
//                 <inject bean="jboss:service=CorbaORB" property="ORB"/>
//             </parameter>
//        </start>

            Logger.log('DEBUG',"server - add corba start to TransactionManager")
            trBeansParser.addTextNode("    ",trManagerNode)
            def trStartNode = trBeansParser.addNode("start","",trManagerNode)
               def trParameterNode = trBeansParser.addNode("parameter","",trStartNode)
                  def trInjectNode = trBeansParser.addNode("inject","",trParameterNode)
                      def trBeanAttr = trBeansParser.addAttribute("bean","jboss:service=CorbaORB",trInjectNode)
                      def trPropertyAttr = trBeansParser.addAttribute("property","ORB",trInjectNode)
            trBeansParser.output(trBeansFile)
            
            //Update conf/jacorb.properties using regular expression matching
            Logger.log('DEBUG',"server - Set jacorb.properties jacorb.poa.thread_pool_max=32")
            def jacorbPropertiesFile = jbossInstallDir+"/server/"+name+"/conf/jacorb.properties"
            Map<String,String> replaceMap = new HashMap<String,String>()
            replaceMap.put("jacorb.poa.thread_pool_max=8", "jacorb.poa.thread_pool_max=32")
            Utils.replace (new File(jacorbPropertiesFile), replaceMap) 
            
            //Update deploy/iiop-service.xml
            Logger.log('DEBUG',"server - modify deploy/iiop-service.xml")
            def iiopServiceFile=jbossInstallDir+"/server/"+name+"/deploy/iiop-service.xml"
            def iiopServiceParser = new Parser(iiopServiceFile,"2")
//            iiopServiceParser.parse(iiopServiceFile)

            def iiopInitNode = iiopServiceParser.findNode('//interceptor-initializers/initializer[text()="org.jboss.tm.iiop.TxServerClientInterceptorInitializer"]')        
            Logger.log('DEBUG',"server - iiop - found interceptor node: "+ iiopInitNode)            
            iiopServiceParser.addComment(" <initializer>org.jboss.tm.iiop.TxServerClientInterceptorInitializer</initializer> ", iiopInitNode)
            iiopServiceParser.removeNode(iiopInitNode)
            
            iiopInitNode = iiopServiceParser.findNode('//interceptor-initializers/initializer[text()="org.jboss.tm.iiop.TxServerInterceptorInitializer"]')            
            Logger.log('DEBUG',"server - iiop - found interceptor node: "+ iiopInitNode)            
            iiopServiceParser.addComment(" <initializer>org.jboss.tm.iiop.TxServerInterceptorInitializer</initializer> ", iiopInitNode)
            iiopServiceParser.removeNode(iiopInitNode)
            
            def corbaInitializerNode = iiopServiceParser.findNode('//mbean[@code="org.jboss.iiop.CorbaORBService"]/attribute[@name="PortableInterceptorInitializers"]/interceptor-initializers')        
            iiopServiceParser.addTextNode("  ",corbaInitializerNode)
            iiopInitNode = iiopServiceParser.addNode("initializer","com.arjuna.ats.jts.orbspecific.jacorb.interceptors.interposition.InterpositionORBInitializerImpl",corbaInitializerNode)
            iiopServiceParser.addComment(" JBoss TS interceptor ", iiopInitNode)
            iiopServiceParser.addTextNode("\n        ",iiopInitNode,"SIBLING")
            iiopInitNode = iiopServiceParser.addNode("initializer","com.arjuna.ats.jbossatx.jts.InboundTransactionCurrentInitializer",corbaInitializerNode)
            iiopServiceParser.addComment("RMI/IIOP tx context interceptor", iiopInitNode)

            iiopServiceParser.output(iiopServiceFile)
            
        }
        
        //Configure new port bindings for server and modify based on properties
        Logger.log('DEBUG', "server - calling JBossPorts($instance,$name,$portIncrement,"+version.getVersion()+", scriptProperties)")
        JBossPorts ports = new JBossPorts(instance,name,portIncrement,version.getVersion(), scriptProperties)
    	jbossNamingPort = ports.jndiPort
    	
        if (jbossBindHost == null){
        	jbossBindHost = "0.0.0.0"
        }
		
		//If this is Version 5 but is NOT EAP, remove some JMX options that don't work. 
		//They seem to function in EAP...		
		if (version.getVersion()=="5" && !version.isEAP()){
			// -Dcom.sun.management.jmxremote
            // -Djboss.platform.mbeanserver
            // -Djavax.management.builder.initial=org.jboss.system.server.jmx.MBeanServerBuilderImpl
			Logger.log('DEBUG',"server : This is Community JBoss 5, need to clean up some JMX options")
			
			Logger.log('DEBUG',"server : remove -Dcom.sun.management.jmxremote")
            jvm_option_list = jvm_option_list.replaceAll("-Dcom.sun.management.jmxremote" , " ")
			
			Logger.log('DEBUG',"server : remove -Djboss.platform.mbeanserver")
			jvm_option_list = jvm_option_list.replaceAll("-Djboss.platform.mbeanserver" , " ")
			
			Logger.log('DEBUG',"server : remove -Djavax.management.builder.initial=org.jboss.system.server.jmx.MBeanServerBuilderImpl")
			jvm_option_list = jvm_option_list.replaceAll("-Djavax.management.builder.initial=org.jboss.system.server.jmx.MBeanServerBuilderImpl" , " ")
		}
		
		//Save off original list of options for wrapper
		def wrapper_option_list = jvm_option_list
        
		//Create Custom JBoss startup script ie. SASServer1.[bat,sh]
		
		//clean up -Dhttp.nonProxyHosts="...|...|..." option.  If using "eval" invocation, need to escape the 
		//use of quotes and pipes in options.
		if (configHostType.equalsIgnoreCase("unx")){
			if (jvm_option_list.contains("-Dhttp.nonProxyHosts")){
				def dollar = /[$]/
				Boolean found_eval=false
				def pattern = /eval \\"($dollar)JAVA/
				def runScript = jbossInstallDir+"/bin/run.sh"
				File runScriptFile = new File(runScript)
				runScriptFile.eachLine {
					if (it =~ pattern) {
						println("grep found "+ it)
						found_eval=true
					}
				}
				if (found_eval){
					//escape quotes in options
					jvm_option_list = jvm_option_list.replaceAll(/["]/, "\\\\\"")
				} else {
				    //escape pipes in options
				    jvm_option_list = jvm_option_list.replaceAll(/[|]/, "\\\\|")
				}
			}
	    }
		
		
        createBatchScript(name,version,jbossInstallDir,jdkInstallDir,jvm_option_list,"", jbossBindHost, configHostType, jbossNamingPort)
        
		//modify existing run.conf.bat if it exists, with required JAVA_OPTS.
		if (configHostType == "win" && version.getVersion()!="4"){
			//JBoss version 5 has a bug in run.conf.bat where it fails to evaluate jvm options that contain quoted spaces.
			//  To manage this, we will pass jvm options indirectly to run.bat.conf using SAS_JAVA_OPTS environment
			//  variable, and then have run.bat.conf evaluate that variable after it's done processing JAVA_OPTS.
			//  This requires us to modify run.conf.bat, so we'll do so once, keeping a backup for future reference.

			def runBat = jbossInstallDir+"/bin/run.conf.bat"
			def backupRunBat = runBat+".backup"
			File backupFile = new File(backupRunBat)
			if ( ! backupFile.exists()) {
			   Ant.copyFile(runBat, backupRunBat, "false")
			   Map<String,String> replaceMap = new HashMap<String,String>()
			   replaceMap.put(":JAVA_OPTS_SET", ":JAVA_OPTS_SET\n\nset JAVA_OPTS=%JAVA_OPTS% %SAS_JAVA_OPTS%")
			   Utils.replace (new File(runBat), replaceMap)
			}
		}
		
        //Create Tanuki Wrapper and copy Tanuki binaries
        if (configHostType == "win" && wrapperdir != null){
			//always add endorsed to options list
			wrapper_option_list = wrapper_option_list + ";-Djava.endorsed.dirs=\"$jbossInstallDir\\lib\\endorsed\""
			
			//always add -Xrs to options list
			wrapper_option_list = wrapper_option_list + ";-Xrs"
			
            Logger.log('DEBUG',"server - configure services ")
            createWrapperScript(name,version,jbossInstallDir,jdkInstallDir,wrapper_option_list,jbossServiceDependency,jbossBindHost)
            Logger.log('DEBUG',"server - create service directory")
            def serviceDirPath = jbossInstallDir+"/service"
            //def serviceDir = new File(serviceDirPath)
            //serviceDir.mkdir()
            Ant.mkdir(serviceDirPath)
            Logger.log('DEBUG',"server - grab wrapper parts")
			
			//Don't copy if we already copied earlier...
			File wrapperExe = new File(serviceDirPath,"wrapper.exe");
			if ( wrapperExe.exists() ) {
				Logger.log('DEBUG',"server - found wrapper parts, no need to copy")		
			} else {
				Logger.log('DEBUG',"server - grab wrapper parts")
				['wrapper.exe','wrapper.dll','wrapper.jar'].each{Ant.copyFile(wrapperdir+"/"+it, serviceDirPath+"/"+it, "true")}
			}

            Logger.log('DEBUG',"server - remove service")
            Logger.log('DEBUG',"server - remove service: Ant.exec(jbossInstallDir, 'cmd', '/c start \"JBoss Application Server: '+name+'\" /wait /B \"'+jbossInstallDir+'\\bin\\'+name+'.bat\" -remove')")
            Ant.exec(jbossInstallDir, 'cmd', '/c start "JBoss Application Server: '+name+'" /wait /B "'+jbossInstallDir+'\\bin\\'+name+'.bat" -remove')

            Logger.log('DEBUG',"server - install service")
            Logger.log('DEBUG',"server - install service: Ant.exec(jbossInstallDir, 'cmd', '/c start \"JBoss Application Server: '+name+'\" /wait /B \"'+jbossInstallDir+'\\bin\\'+name+'.bat\" -install')")
            Ant.exec(jbossInstallDir, 'cmd', '/c start "JBoss Application Server: '+name+'" /wait /B "'+jbossInstallDir+'\\bin\\'+name+'.bat" -install')        
        }
        
        //Make ear clasloaders isolated check for version 4 first, then look for v5
         if (version.getVersion()=="4"){
             def earDeployerFile="${jbossInstallDir}/server/${name}/deploy/ear-deployer.xml"
             def earDeployerParser = new Parser()
             earDeployerParser.parse(earDeployerFile)
             earDeployerParser.updateTextNode("true",'//mbean[@name="jboss.j2ee:service=EARDeployer"]/attribute[@name="Isolated"]')
             earDeployerParser.output(earDeployerFile)
         
         }else {
             def earDeployerFile="${jbossInstallDir}/server/${name}/deployers/ear-deployer-jboss-beans.xml"
             def earDeployerParser = new Parser()
             earDeployerParser.parse(earDeployerFile)
             earDeployerParser.updateTextNode("true",'//bean[@name="EARClassLoaderDeployer"]/property[@name="isolated"]')
             earDeployerParser.output(earDeployerFile)
         }
     
        // TODO: Turn off RMI server class downloads for V4.  already default at v5
        
        // Update tomcat to handle encoding 
         def serverXmlFile = "${jbossInstallDir}/server/${name}/deploy/jbossweb.sar/server.xml"
         if (version.getVersion()=='4'){
             serverXmlFile = "${jbossInstallDir}/server/${name}/deploy/jboss-web.deployer/server.xml"
         }   
         def serverXmlParser = new Parser()
         serverXmlParser.parse(serverXmlFile)
         serverXmlParser.addAttribute("useBodyEncodingForURI","true",'//Connector[@port="8080"]')
         serverXmlParser.output(serverXmlFile)
        
        def webDeployerFile
        
        // Set jsp "development" mode
         if (version.getVersion()=="4"){
             webDeployerFile="${jbossInstallDir}/server/${name}/deploy/jboss-web.deployer/conf/web.xml"        
         }else if (version.getVersion()=="5"){
             webDeployerFile="${jbossInstallDir}/server/${name}/deployers/jbossweb.deployer/web.xml"
         }else {
             webDeployerFile="${jbossInstallDir}/server/${name}/deploy/jbossweb.sar/web.xml"
         }
         def webDeployerParser = new Parser()
         webDeployerParser.parse(webDeployerFile)
         //
         //Go find /web-app/servlet/servlet-name == 'jsp' locate last <init-param> and add
         //   <init-param>
         //      <param-name>development</param-name>
         //      <param-value>false</param-value>
         //   </init-param>
         // Make sure the 'development' parameter isn't already defined, in which case we just need to update it.
         
         def jspNode = webDeployerParser.findTextNode('//servlet/servlet-name', 'jsp')
         if (jspNode) {
             def developmentNode = webDeployerParser.findTextNode(jspNode.getParentNode(), 'init-param/param-name', 'development')  
             if (developmentNode) {
                //development node found, reset it to false
                def init = developmentNode.getParentNode()
                def nodeList = init.getChildNodes()
                def servlet = init.getParentNode()
                Logger.log('DEBUG',"server: found existing development node in servlet: " + servlet)
                for (node in nodeList) {
                   if (node.getNodeName() == "param-value") {
                      Logger.log('DEBUG',"server: found existing param-value text node")
                      webDeployerParser.updateTextNode("false", node)
                   }
                }               
             } else {
                 Logger.log('DEBUG',"server: can't find development node, lets add one ")
                 def servletNode = jspNode.getParentNode()
                 def nodeList = servletNode.getChildNodes()
                 def lastNode = servletNode.getLastChild()
                 def lastInitParam=null
                 def newInitParam=null
                 
                 //we need to find the last init-param node and add ours after it.
                 for (node in nodeList) {
                    if (node.getNodeName() == "init-param") {
                       lastInitParam = node
                    }
                 }

                 if (lastInitParam) {
                    //Need the node <after> lastInitParam since we use parent.insertBefore(siblingNode, node) to get 
                    //indentation without having to work for it.
                    lastInitParam = lastInitParam.getNextSibling()
                    Logger.log('DEBUG',"server: add jsp development mode after last init-param ")
                    newInitParam = webDeployerParser.addNode("init-param", "", lastInitParam, "SIBLING")
                 } else {
                    //For some reason, we couldn't find the last Init Param, we'll assume we can place it
                    //as next to last.
                    Logger.log('DEBUG',"server: add jsp development mode before last node ")
                    newInitParam = webDeployerParser.addNode("init-param", "", lastNode.getPreviousSibling(), "SIBLING")
                 }
                 webDeployerParser.addNode("param-name","development", newInitParam)
                 webDeployerParser.addNode("param-value","false", newInitParam)
             } 
			 updateCompilerVersion(webDeployerParser,jspNode)
             webDeployerParser.output(webDeployerFile)
         } else {
             Logger.log('DEBUG',"server: can not locate //servlet/servlet-name 'jsp' text node in $webDeployerFile")
         }
         
        // Add deploy_sas deployment directory      
         def deployDirName = new String("deploy_sas")
         def deployDirPath = new String("$jbossInstallDir/server/$name/$deployDirName")
         def deployDir = new File(deployDirPath)
         deployDir.mkdir()
         
         if (version.getVersion()=='4'){
             Logger.log('DEBUG', "server - processing jboss-service; DeploymentScanner")
             def xpathnode="//mbean[@name='jboss.deployment:type=DeploymentScanner,flavor=URL']/attribute[@name='URLs']"

             def serviceXmlFile = new String("$jbossInstallDir/server/$name/conf/jboss-service.xml")
             Parser serviceXmlParser = new Parser()
             serviceXmlParser.parse(serviceXmlFile)
             def oldTextNode = serviceXmlParser.findNode(xpathnode)
             oldTextNode = oldTextNode.getFirstChild(); 
             
             Logger.log('DEBUG', "server - DeploymentScanner URLs node = " + oldTextNode)
             def newText=""
             Logger.log('DEBUG', "server - DeploymentScanner oldTextNode.getNodeType() = " + oldTextNode.getNodeType() )
             Logger.log('DEBUG', "server - DeploymentScanner org.w3c.dom.Node.TEXT_NODE = " + org.w3c.dom.Node.TEXT_NODE )
             if (oldTextNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE)
             {
                def currentText = oldTextNode.getNodeValue()
                Logger.log('DEBUG', "server - DeploymentScanner currentText = "+ currentText)
                
                // We now have a lines of text, strip whitespace, append to "newText"...
                for (line in currentText.split("\n"))
                {
                   if (!line) continue

				   (line =~ /\S/).each{match -> newText += match}
                }
                
                def newURLs=newText
                
                // If we already have this directory, don't add it again
                if ( ! newText.contains("$deployDirName/"))
                   {
                	newURLs += ","+deployDirName+"/"
                   }
                Logger.log('DEBUG', "server - DeploymentScanner newURLs = "+ newURLs)
                
                serviceXmlParser.updateTextNode(newURLs,xpathnode) 
                serviceXmlParser.output(serviceXmlFile)
             }
             
         }
         else {
             // Update profile.xml with appropriate element containing new directory
             //   See Section "1.5.2. Adding a custom deploy folder" in the JBoss 5.0.0 EAP "Getting Started Guide"
             def profileXmlFile = new String("$jbossInstallDir/server/$name/conf/bootstrap/profile.xml")
             Parser profileXmlParser = new Parser()
             profileXmlParser.parse(profileXmlFile)
             def profileNode = profileXmlParser.addNode("value","\${jboss.server.home.url}$deployDirName",'//bean[@name="BootstrapProfileFactory"]/property[@name="applicationURIs"]/list')
             profileXmlParser.output(profileXmlFile)
             
			 if (version.getVersion()=='5'){
	             // Update vfs.xml with appropriate element containing new directory - this is 
	             //   to enable appropriate performance of cache directory containing jar files
	             //   See Section "1.5.2. Adding a custom deploy folder" in the JBoss 5.0.0 EAP "Getting Started Guide"
	             def vfsXmlFile = new String("$jbossInstallDir/server/$name/conf/bootstrap/vfs.xml")
	             def vfsXmlParser = new Parser()
	             vfsXmlParser.parse(vfsXmlFile)
	             def entryNode = vfsXmlParser.addNode("entry","",'//bean[@name="VFSCache"]/property[@name="permanentRoots"]/map')
	                 def keyNode = vfsXmlParser.addNode("key","\${jboss.server.home.url}$deployDirName",entryNode)
	                 def valueNode = vfsXmlParser.addNode("value","",entryNode)
	                     def injectNode = vfsXmlParser.addNode("inject","",valueNode)
	                         vfsXmlParser.addAttribute("bean","VfsNamesExceptionHandler",injectNode)
	             vfsXmlParser.output(vfsXmlFile)
			 }
         }
                
         // Tune log4j categories
		 
		 def jbossLog4jXmlFile
		 if (version.getVersion() == "6"){
			 jbossLog4jXmlFile = new String("$jbossInstallDir/server/$name/deploy/jboss-logging.xml")
		 }else{
            jbossLog4jXmlFile = new String("$jbossInstallDir/server/$name/conf/jboss-log4j.xml")
		 }
		 def jbossLog4jXmlParser = new Parser()     
         jbossLog4jXmlParser.parse(jbossLog4jXmlFile)

		 if (version.getVersion() == "6"){
			 //Update org.jgroups to ERROR as WARN dumps out passwords...
			 def catJgroups = jbossLog4jXmlParser.findNode('//logger[@category="org.jgroups"]/level')
			                  jbossLog4jXmlParser.addAttribute("name","ERROR",catJgroups)
			 
			 //Handle the rest in a closure list, update in place if already exists.
			 def catList = ["com.arjuna",
						"httpclient",
						"org.springframework",
						"org.jboss.mq",
						"org.jboss.jms",
						"org.jboss.messaging",
						"org.jboss.remoting",
						"org.jboss.system.ServiceController"].each{
				 def categoryNode = jbossLog4jXmlParser.findNode("//logger[@category=\"$it\"]/level")
				 if (categoryNode != null){
					 jbossLog4jXmlParser.addAttribute("name","WARN",categoryNode)
				 }else{
					 def sibling = jbossLog4jXmlParser.findNode('//logger[@category="org.jgroups"]').getNextSibling()
					 if (sibling != null){
						 def loggerNode = jbossLog4jXmlParser.addNode("logger","",sibling,"SIBLING")
						 jbossLog4jXmlParser.addAttribute("category",it,loggerNode)
						 def levelNode = jbossLog4jXmlParser.addNode("level","",loggerNode)
						 jbossLog4jXmlParser.addAttribute("name","WARN",levelNode)
						 jbossLog4jXmlParser.addTextNode("\n\n   ",loggerNode,"SIBLING")
					 }else{
						 Logger.log('DEBUG', "server - Can't locate " + '//logger[@category="org.jgroups"]')
						 Logger.log('DEBUG', "server - org.jgroups category required in jboss-logging.xml.")
					 }
					 
				 }
			 }
			 jbossLog4jXmlParser.output(jbossLog4jXmlFile)

		 }
		 else if (version.getVersion() == "4" || version.getVersion() == "5"){
			 //Update Append to "false" so we can detect app startup failure
			 def appendNode = jbossLog4jXmlParser.findNode('//appender[@name="FILE"]/param[@name="Append"]')
			 if (appendNode){
				 jbossLog4jXmlParser.addAttribute("value","false",appendNode)
			 }
	
	         //Update org.jgroups to ERROR as WARN dumps out passwords...
	         def catJgroups = jbossLog4jXmlParser.findNode('//category[@name="org.jgroups"]/priority')
	         jbossLog4jXmlParser.addAttribute("value","ERROR",catJgroups)
	         
	         //Handle the rest in a closure list, update in place if already exists.
	         def catList = ["com.arjuna",
	                    "httpclient",
	                    "org.springframework",
	                    "org.jboss.mq",
	                    "org.jboss.jms",
						"org.jboss.messaging",
	                    "org.jboss.remoting",
	                    "org.jboss.system.ServiceController"].each{ 
	             def categoryNode = jbossLog4jXmlParser.findNode("//category[@name=\"$it\"]/priority")
	             if (categoryNode != null){
	                 jbossLog4jXmlParser.addAttribute("value","WARN",categoryNode)           
	             }else{
	                 def sibling = jbossLog4jXmlParser.findNode('//category[@name="org.jgroups"]').getNextSibling()
	                 if (sibling != null){
	                     categoryNode = jbossLog4jXmlParser.addNode("category","",sibling,"SIBLING")
	                     jbossLog4jXmlParser.addAttribute("name",it,categoryNode)
	                     def priorityNode = jbossLog4jXmlParser.addNode("priority","",categoryNode)
	                     jbossLog4jXmlParser.addAttribute("value","WARN",priorityNode)
	                     jbossLog4jXmlParser.addTextNode("\n\n   ",categoryNode,"SIBLING")
	                 }else{
	                     Logger.log('DEBUG', "server - Can't locate " + '//category[@name="org.jgroups"]')
	                     Logger.log('DEBUG', "server - org.jgroups category required in jboss-log4j.xml.")
	                 }
	                 
	             }
	         }
	
	         //log4j needs a DTD set for SYSTEM    
	         jbossLog4jXmlParser.setSystemOutputProperty("log4j.dtd")
	         jbossLog4jXmlParser.output(jbossLog4jXmlFile)
		 }
         // Tune jms security if needed
         if (jmsSecurity == "true" && (version.getVersion()=='4' || version.getVersion()=='5') ){
			     def jmsServiceFile
			     def jmsXPath
			     def applicationPolicyXPath
                 def messagingUsersFile = new String("$jbossInstallDir/server/$name/conf/props/sas-jms-users.properties")
                 def messagingRolesFile = new String("$jbossInstallDir/server/$name/conf/props/sas-jms-roles.properties") 
				 
				 if (messagingDir.equalsIgnoreCase("jms")) { 
					 jmsServiceFile = "$jbossInstallDir/server/$name/deploy/$messagingDir/jbossmq-service.xml"
					 jmsXPath = '//mbean[@code="org.jboss.mq.security.SecurityManager"]/attribute/security/role'
  
				 } else {
                    jmsServiceFile = "$jbossInstallDir/server/$name/deploy/$messagingDir/messaging-service.xml"
                    jmsXPath='//mbean[@code="org.jboss.jms.server.ServerPeer"]/attribute[@name="DefaultSecurityConfig"]/security/role'
  				    if (version.getVersion()=='5') {
					   jmsServiceFile = "$jbossInstallDir/server/$name/deploy/$messagingDir/messaging-jboss-beans.xml"
					   jmsXPath='//mbean[@code="org.jboss.jms.server.ServerPeer"]/attribute[@name="DefaultSecurityConfig"]/security/role'
				    } 

				 }
				 
				 applicationPolicyXPath='//application-policy[@name="messaging"]'
				 
			     def jmsParser = new Parser();
    
                 Logger.log('DEBUG', "server - jmsSecurity - messagingUsersFile="+messagingUsersFile)
                 Logger.log('DEBUG', "server - jmsSecurity - messagingRolesFile="+messagingRolesFile)
                 Logger.log('DEBUG', "server - jmsSecurity - jmsServiceFile="+jmsServiceFile)
                 Logger.log('DEBUG', "server - jmsSecurity - jmsXPath="+jmsXPath)
                 
				 String prompt = "\nEnter Password \n\tJMS Security \n\tJMS User=$jmsSecurityUser \n\tPassword="
				 jmsSecurityPW = com.sas.config.jboss.Utils.encodeDecodePassword(jmsSecurityPW, encoding, prompt)

                 //Update user/role files with appropriate values - creates new files
                 createUserScript(messagingUsersFile,jmsSecurityUser,jmsSecurityPW)
                 createRoleScript(messagingRolesFile,jmsSecurityUser)
                 
                 //Update jms service file with new Role name value
                 jmsParser.parse(jmsServiceFile)                          
                 jmsParser.addAttribute("name","JMSUser",jmsXPath)
				 
				 //Clean up application policy and change default guest permissions to JMSUser...
				 def appPolicy = jmsParser.findNode(applicationPolicyXPath)
				 if (appPolicy){
					 
					 //JBoss placed the "messaging" app policy into the jms file rather than the 
					 // standard login-config.xml.  Not sure exactly why, but we'll comment this one
					 // out and then add our own, which will use our own user/password already configured
					 // rather than the default for "guest".
					 //
					 def comment = jmsParser.addComment(" Comment out application-policy $appPolicy", appPolicy)
					 jmsParser.removeNode(appPolicy)
				     
				     //Now swap "guest" with "JMSUser" so we control the role.  Since this is in a CDATA
				     // we need to take care to get the spacing right.  
				     //
				     /*    
					    <bean class="org.jboss.jms.server.jbosssx.JBossASSecurityMetadataStore" name="SecurityStore">
					      <!-- default security configuration -->
					      <property name="defaultSecurityConfig">
					         <![CDATA[
					            <security>
					               <role name="guest" read="true" write="true" create="true"/>
					            </security>
					         ]]>
					      </property>
					  */
					 def defaultSecurityConfigNode = jmsParser.findNode('//bean[@name="SecurityStore"]/property[@name="defaultSecurityConfig"]')
					 def newData = '\n\t<security>\n\t\t<role name="JMSUser" read="true" write="true" create="true"/>\n\t</security>'
					 if (defaultSecurityConfigNode){
					    def cdataNode = jmsParser.updateCDATANode(newData,defaultSecurityConfigNode)
						jmsParser.addTextNode("\n      ", defaultSecurityConfigNode)
					 } else {
					    Logger.log('DEBUG', "server - jmsSecurity - defaultSecurityConfigNode not found")
					 }
				 }else{
				     Logger.log('DEBUG', "server - jmsSecurity - can't locate applicationPolicyXPath="+applicationPolicyXPath)
				 }
			 
                 jmsParser.output(jmsServiceFile)
                 
				 //
				 //Remove resource recovery process when security is on, as we don't need it, and it requires a 
				 // plain text password to allow it to function when JMS security is on - something we don't want.
				 //Turns out the file containing the node is named different between JBoss 4 and JBoss 5.  Also, the node
				 // doesn't actually exist in Jboss 5.0 and JBoss 5.0.1, only JBoss 5.1.  However, we can check regardless
				 // of JBoss 5 release, and update when found.
				 //
                 def jtaPropertiesFile=null
				 if (version.getVersion()=='4') {
                    jtaPropertiesFile = new String("$jbossInstallDir/server/$name/conf/jbossjta-properties.xml")
                 } else {
				    jtaPropertiesFile = new String("$jbossInstallDir/server/$name/conf/jbossts-properties.xml")
                 }
                
				 def jtaXPath = '//properties/property[@name="com.arjuna.ats.jta.recovery.XAResourceRecovery.JBMESSAGING1"]'
                 def jtaPropertiesParser = new Parser();
                 jtaPropertiesParser.parse(jtaPropertiesFile)
                 def resourceRecoveryNode = jtaPropertiesParser.findNode(jtaXPath)
                 if (resourceRecoveryNode) {
                   def parent = resourceRecoveryNode.getParentNode()
                   parent.removeChild(resourceRecoveryNode)
                   jtaPropertiesParser.output(jtaPropertiesFile)
                 }

         }
        
         //copy policy files.  The jvm options may use one or the other, place both in conf for safe keeping
         if (allPolicyFile != null)
             Ant.copyFileToDir(allPolicyFile,"$jbossInstallDir/server/$name/conf","true")
         //If we can't find the specific restrictive policy file, don't worry about it. (4th parm is ignore fail)
         if (restrictivePolicyFile != null)
             Ant.copyFileToDir(restrictivePolicyFile,"$jbossInstallDir/server/$name/conf","true","false")
             
         //Tune annotation scanning performance in JBoss 5 
         //We will take the simple approach, adding updates to the metadata-deployer-jboss-beans.xml file
         //using a list of war file names from sdw.
         
         /* -> moved this functionality to Application so we get new wars
         def wars = scriptProperties.get("server."+instance+".warlist")       
         def warList= wars.split(',')
         
         def jbossVersion = version.getVersion()
         if (jbossVersion != "4" && jbossVersion != "4.3") {
             
             def metadataDeployerFile="$jbossInstallDir/server/$name/deployers/metadata-deployer-jboss-beans.xml"
             def metadataDeployerParser = new Parser()
             metadataDeployerParser.parse(metadataDeployerFile)
             def customDeployNode = '//deployment/bean[@name="JBossCustomDeployDUFilter"]/constructor/parameter/bean/constructor/parameter/map'
             def entryNode
             def lastChild
             def valueNode
             warList.each {
                lastChild = metadataDeployerParser.findNode(customDeployNode).getLastChild()
                entryNode = metadataDeployerParser.addNode("entry","", lastChild, "SIBLING" )
                metadataDeployerParser.addNode("key", it, entryNode)
                valueNode = metadataDeployerParser.addNode("value","", entryNode)
                metadataDeployerParser.addNode("null","",valueNode)
             }
             metadataDeployerParser.output(metadataDeployerFile)
          }
          */ 
	      
	      /* Turn off deployment scanner in JBoss 5.1 */
	      if (version.getVersion()!='4') {
			  Ant.deleteFile("$jbossInstallDir/server/$name/deploy/hdscanner-jboss-beans.xml");
		  }

		  /* Set SessionCookie path to use root as default instead of /context so Portal can function. 
		   * Only for Jboss 5 or higher, as jboss changed the default. see S0782643 */
		  if (version.getVersion()!='4') {
			  def contextXmlFile = new String("$jbossInstallDir/server/$name/deploy/jbossweb.sar/context.xml")
			  def contextXmlParser = new Parser()
			  contextXmlParser.parse(contextXmlFile)
 			  def sessionNode = null
			  sessionNode = contextXmlParser.findNode('//Context/SessionCookie')
              if (!sessionNode) {            	  
			      sessionNode = contextXmlParser.addNode("SessionCookie","",'//Context')
					 	        contextXmlParser.addAttribute("path","/",sessionNode)
			      contextXmlParser.output(contextXmlFile)
              }
		  }
         }
         catch (Throwable e) {
             Logger.log('DEBUG', "Failure : " + e.toString());
             e.printStackTrace();
             return 8;
         }
         return 0;
    }   

def createUserScript(
        String filename,
        String username,
        String password
    ){  
    def wrapperScript = '''#
# user=password
#
$username=$password
    '''      
       def engine = new groovy.text.SimpleTemplateEngine()
       def template = engine.createTemplate(wrapperScript)
       def decodedpassword = null
       if (password!= null)
            decodedpassword = com.sas.util.SasPasswordString.decode(password)
       def binding = [
           username : username,
           password : decodedpassword, 
             ]
       def scriptOut = template.make(binding)
       def outFile = new File(filename)
       outFile.write(scriptOut.toString())        
}
def createRoleScript(
        String filename,
        String username
    ){  
    def wrapperScript = '''#
# user=role1,role2,...
#
$username=JMSUser
    '''      
       def engine = new groovy.text.SimpleTemplateEngine()
       def template = engine.createTemplate(wrapperScript)
       def binding = [
           username : username,
             ]
       def scriptOut = template.make(binding)
       def outFile = new File(filename)
       outFile.write(scriptOut.toString())        
}
    
    
//
//createBatchScript()
//TODO : implement the multicastAuthToken, not currently in the script code...
//TODO : implement the createShellScript() method for unix...
//
void createBatchScript(
  String servername,
  JBossVersion version,
  String jbossInstallDir,
  String jdkInstallDir,
  String jvmOptionsList,
  String multicastAuthToken,
  String jbossBindHost,
  String configHostType,
  String jbossNamingPort
){  
  def runBatScript='''
@echo off
rem -------------------------------------------------------------------------
rem JBoss Bootstrap Script for Win32
rem -------------------------------------------------------------------------

setlocal


set SERVICENAME=JBoss - $servername
set JBOSS_BIN_DIR=$jbossInstallDir\\\\bin
set JBOSS_SERVER_DIR=$jbossInstallDir\\\\server\\\\$servername
set WRAPPER_DIR=$jbossInstallDir\\\\service

if x%1 EQU x goto start_as_script
if %1 EQU install goto install
if %1 EQU remove goto remove
if %1 EQU start goto start
if %1 EQU stop goto stop
if %1 EQU restart goto restart
if %1 EQU pause goto pause
if %1 EQU resume goto resume
if %1 EQU status goto status
if %1 EQU -install goto install
if %1 EQU -remove goto remove
if %1 EQU -start goto start
if %1 EQU -stop goto stop
if %1 EQU -restart goto restart
if %1 EQU -pause goto pause
if %1 EQU -resume goto resume
if %1 EQU -status goto status
goto usage


:install
 start /b /wait "JBoss - $servername" "%WRAPPER_DIR%\\\\Wrapper.exe" -i "%JBOSS_SERVER_DIR%\\\\wrapper.conf"
 if %ERRORLEVEL% == 0 echo JBoss server installed as a service
 goto end
:remove
 start /b /wait "JBoss - $servername" "%WRAPPER_DIR%\\\\Wrapper.exe" -r "%JBOSS_SERVER_DIR%\\\\wrapper.conf"
 if %ERRORLEVEL% == 0 echo JBoss server removed as a service
 goto end
:start
 net start "%SERVICENAME%"
 if %ERRORLEVEL% == 0 echo JBoss server started as a service
 goto end
:stop
 net stop "%SERVICENAME%"
 if %ERRORLEVEL% == 0 echo JBoss server stopped as a service
 goto end
:restart
 net stop "%SERVICENAME%"
 if %ERRORLEVEL% == 0 echo JBoss server stopped as a service
 net start "%SERVICENAME%"
 if %ERRORLEVEL% == 0 echo JBoss server started as a service
 goto end
:pause
 net pause "%SERVICENAME%"
 if %ERRORLEVEL% == 0 echo JBoss server paused as a service
 goto end
:resume
 net continue "%SERVICENAME%"
 if %ERRORLEVEL% == 0 echo JBoss server resumed as a service
 goto end
:status
 sc interrogate "%SERVICENAME%"
 goto end
:usage
 echo USAGE:  %0 {install ^| start ^| stop ^| restart ^| pause ^| resume ^| remove ^| status}
 goto end
 
:start_as_script
set JAVA_HOME=$jdkInstallDir
set $jvmOptionsVar=$jvmOptions

$multicastAuthToken

"$jbossInstallDir\\\\bin\\\\run.bat" -c $servername -b $jbossBindHost %*

:end
endlocal
if [%2] EQU [exit] exit %ERRORLEVEL%
'''
  def runShScript='''#!/bin/sh -p
### ====================================================================== ###
##                                                                          ##
##  JBoss Bootstrap Script                                                  ##
##                                                                          ##
### ====================================================================== ###

JAVA_HOME=\"$jdkInstallDir\"
export JAVA_HOME

JAVA_OPTS=\"$jvmOptions\"
export JAVA_OPTS

$multicastAuthToken

mkdir -p \"$jbossInstallDir/server/$servername/log\"

# Get argument
if [ "\\$1" = "-start" ]; then
   arg=start ; shift
elif [ "\\$1" = "-stop" ]; then
   arg=stop ; shift
elif [ "\\$1" = "-status\" ]; then
   arg=status ; shift
elif [ "\\$1" = "-restart" ]; then
   arg=restart ; shift
elif [ "\\$1" = "-kill" ]; then
   arg=kill ; shift
elif [ "\\$1" = "start" ]; then
   arg=start ; shift
elif [ "\\$1" = "stop" ]; then
   arg=stop ; shift
elif [ "\\$1" = "status" ]; then
   arg=status ; shift
elif [ "\\$1" = "restart" ]; then
   arg=restart ; shift
elif [ "\\$1" = "kill" ]; then
   arg=kill ; shift
elif [ "x\\$1" = "x" ]; then
   arg=startForeground
fi

case "\\$arg" in
   start)
      LAUNCH_JBOSS_IN_BACKGROUND=true
      export LAUNCH_JBOSS_IN_BACKGROUND      
      nohup "$jbossInstallDir/bin/run.sh" -c $servername -b $jbossBindHost \\$@ >"$jbossInstallDir/server/$servername/log/stdout.log" 2>&1 &
      pid=\\$!
      echo \\$pid > "$jbossInstallDir/server/$servername/log/jboss.pid"
      echo "JBoss startup has begun (pid \\$pid)..."
   ;;
   
   stop)
#        add args for username/password as in "${servername}.sh stop -u username -p password"
      JAVA_OPTS="-d64 -Xms512M -Xmx512M"
      "$jbossInstallDir/bin/shutdown.sh" -s localhost:$jbossNamingPort \\$* -S
   ;;
   
   kill)
         if [ -f $jbossInstallDir/server/$servername/log/jboss.pid ]; then
            pid=`cat $jbossInstallDir/server/$servername/log/jboss.pid`
            kill \\$* \\$pid
            if [ \\$? -ne 0 ]; then
               echo "pid: \\$pid"
            fi
         else
             echo "JBoss not running." 
             exit 1
         fi
   ;;
   
   status)
          if [ -f $jbossInstallDir/server/$servername/log/jboss.pid ]; then
            pid=`cat $jbossInstallDir/server/$servername/log/jboss.pid`
            kill -0 \\$pid > /dev/null 2>&1
            if [ \\$? -eq 0 ]; then
               echo "JBoss running (pid \\$pid)"
            else
               echo "JBoss is not running"
            fi
         else
            echo "JBoss is not running"
         fi 
   ;;
   
   restart)
         \\$0 -kill -9
         if [ \\$? -eq 0 ]; then 
            sleep 10
            \\$0 -start
         fi
   
   ;;
   
   startForeground)
        "$jbossInstallDir/bin/run.sh" -c $servername -b $jbossBindHost \\$@
   ;;
   
   *)
         echo "USAGE:  \\$0 { start | stop | restart | status | kill | help}"
         echo "     :  start   - launch JBoss server in the background"
         echo "     :  stop    - issue [JBOSS-HOME]/bin/shutdown.sh to stop JBoss server"
         echo "     :  restart - kill JBoss server process, re-launch server in the background"
         echo "     :  status  - return JBoss server process status (via kill -0 <pid>)"
         echo "     :  help    - display these command options"
         echo "     :  [no arguments passed to the command results in launch of JBoss server in the foreground]"
         echo " "
         
   ;;
esac

exit 0

exit 0
'''

 def jvmOptions = jvmOptionsList.replaceAll(/;/) {' '}
 def engine = new groovy.text.SimpleTemplateEngine()
 def template 
 def subscript
 def jvmOptionsVar="JAVA_OPTS"
 if (configHostType == "win") {
    template = engine.createTemplate(runBatScript)
    subscript = ".bat"
 } else {
    template = engine.createTemplate(runShScript)
    subscript = ".sh"
	//need to clean up pipes in -Dhttp.nonProxyHosts option
	if (jvmOptions.contains("-Dhttp.nonProxyHosts")){
		
	}
 }
 if (version.getVersion() != "4") {
	 jvmOptionsVar = "SAS_JAVA_OPTS"
 }
 def binding = [
     servername: servername,
     jbossInstallDir : jbossInstallDir,
     jdkInstallDir : jdkInstallDir,
     jvmOptions : jvmOptions, 
     multicastAuthToken : multicastAuthToken,
     jbossBindHost : jbossBindHost,
     jbossNamingPort : jbossNamingPort,
	 jvmOptionsVar: jvmOptionsVar
       ]
 def scriptOut = template.make(binding)
 def filename = jbossInstallDir+"/bin/"+servername+subscript
 def outFile = new File(filename)
 def rc = outFile.write(scriptOut.toString())
 Logger.log('DEBUG', "createBatchScript - writing $filename returned $rc")
 
 Chmod.file("u+rx", outFile)
 
}


//TODO :  Implement jbossServiceDependency code...not currently in the wrapper...
//#wrapper.ntservice.dependency.1=${services.rmi.service.name}
void createWrapperScript(
      String servername,
      JBossVersion version,
      String jbossInstallDir,
      String jdkInstallDir,
      String jvmOptions,
      String jbossServiceDependency,
      String jbossBindHost
  ){  
  def wrapperScript = '''
#********************************************************************
# Wrapper Properties
#********************************************************************
# Java Application

wrapper.java.command=$jdkInstallDir\\\\bin\\\\java

# Java Main class.  This class must implement the WrapperListener interface
#  or guarantee that the WrapperManager class is initialized.  Helper
#  classes are provided to do this for you.  See the Integration section
#  of the documentation for details.
wrapper.java.mainclass=org.tanukisoftware.wrapper.WrapperSimpleApp

# Java Classpath (include wrapper.jar)  Add class path elements as
#  needed starting from 1
wrapper.java.classpath.1=$jbossInstallDir/service/wrapper.jar
wrapper.java.classpath.2=$jdkInstallDir/lib/tools.jar
wrapper.java.classpath.3=$jbossInstallDir/bin/run.jar

# Java Library Path (location of Wrapper.DLL or libwrapper.so)
wrapper.java.library.path.1=$jbossInstallDir/service

# Java Additional Parameters
<% count=0
 jvmOptionsList.each { count++
%>wrapper.java.additional.$count=$it
<%}%>

# Java Additional Parameters End

# Initial Java Heap Size (in MB)
#wrapper.java.initmemory=768

# Maximum Java Heap Size (in MB)
#wrapper.java.maxmemory=768

# Application parameters.  Add parameters as needed starting from 1
wrapper.app.parameter.1=org.jboss.Main
wrapper.app.parameter.2=-c 
wrapper.app.parameter.3=$servername 
wrapper.app.parameter.4=-b 
wrapper.app.parameter.5=$jbossBindHost



#********************************************************************
# Wrapper Logging Properties
#********************************************************************
# Format of output for the console.  (See docs for formats)
wrapper.console.format=PM

# Log Level for console output.  (See docs for log levels)
wrapper.console.loglevel=WARN

# Log file to use for wrapper output logging.
wrapper.logfile=../logs/wrapper.log

# Format of output for the log file.  (See docs for formats)
wrapper.logfile.format=LPTM

# Log Level for log file output.  (See docs for log levels)
wrapper.logfile.loglevel=WARN

# Maximum size that the log file will be allowed to grow to before
#  the log is rolled. Size is specified in bytes.  The default value
#  of 0, disables log rolling.  May abbreviate with the 'k' (kb) or
#  'm' (mb) suffix.  For example: 10m = 10 megabytes.
wrapper.logfile.maxsize=0

# Maximum number of rolled log files which will be allowed before old
#  files are deleted.  The default value of 0 implies no limit.
wrapper.logfile.maxfiles=0

# Log Level for sys/event log output.  (See docs for log levels)
wrapper.syslog.loglevel=ERROR

#********************************************************************
# Wrapper Windows Properties
#********************************************************************
# Title to use when running as a console
wrapper.console.title="JBoss - $servername"

#********************************************************************
# Wrapper Windows NT/2000/XP Service Properties
#********************************************************************
# WARNING - Do not modify any of these properties when an application
#  using this configuration file has been installed as a service.
#  Please uninstall the service before modifying this section.  The
#  service can then be reinstalled.

# Name of the service
wrapper.ntservice.name=JBoss - $servername

# Display name of the service
wrapper.ntservice.displayname=JBoss - $servername

# Description of the service
wrapper.ntservice.description=JBoss - $servername Version: $version

# Service dependencies.  Add dependencies as needed starting from 1
$jbossServiceDependency

# Mode in which the service is installed.  AUTO_START or DEMAND_START
wrapper.ntservice.starttype=DEMAND_START

# Allow the service to interact with the desktop.
wrapper.ntservice.interactive=false

# Wait 5 minutes for jvm to shut itself down before terminating
wrapper.shutdown.timeout=300
wrapper.jvm_exit.timeout=300
'''
     def jvmOptionsList = jvmOptions.tokenize(";").collect {it as String}
     
     def engine = new groovy.text.SimpleTemplateEngine()
     def template = engine.createTemplate(wrapperScript)
     def binding = [
         servername: servername,
         version: version.getFullVersion(),
         jbossInstallDir : jbossInstallDir,
         jdkInstallDir : jdkInstallDir,
         jvmOptionsList : jvmOptionsList, 
         jbossServiceDependency : jbossServiceDependency,
         jbossBindHost : jbossBindHost
           ]
     def scriptOut = template.make(binding)
     def filename = jbossInstallDir+"/server/"+servername+"/wrapper.conf"
     def outFile = new File(filename)
     outFile.write(scriptOut.toString())    
}
  public void updateCompilerVersion(webDeployerParser,jspNode){
	  Logger.log('DEBUG',"server: begin update jsp compiler version ")
	  def compilerSourceVMNode =  webDeployerParser.findTextNode('//servlet/init-param/param-name','compilerSourceVM');
	  def compilerTargetVMNode =  webDeployerParser.findTextNode('//servlet/init-param/param-name','compilerTargetVM');
	  if(compilerSourceVMNode){
		  def compilerSourceVMInitNode = compilerSourceVMNode.getParentNode();
		  def nodeList = compilerSourceVMInitNode.getChildNodes();
		  for (node in nodeList) {
			  if (node.getNodeName() == "param-value") {
				  //Logger.log('DEBUG',"server: found existing param-value text node")
				  //System.out.println(node.getTextContent());

				  webDeployerParser.updateTextNode("1.6", node)
			  }
		  }
	  }else{
		  def init = webDeployerParser.addNode("init-param", "", jspNode.getParentNode())
		  def paraName = webDeployerParser.addNode("param-name", "compilerSourceVM", init)
		  compilerSourceVMNode = paraName
		  def paravalue = webDeployerParser.addNode("param-value", "1.6", init)
	  }

	  if(compilerTargetVMNode){
		  def compilerTargetVMInitNode = compilerTargetVMNode.getParentNode();
		  def nodeList = compilerTargetVMInitNode.getChildNodes();
		  for (node in nodeList) {
			  if (node.getNodeName() == "param-value") {
				  // Logger.log('DEBUG',"server: found existing param-value text node")
				  webDeployerParser.updateTextNode("1.6", node)
			  }
		  }
	  }else{
		  Logger.log('DEBUG',"compilerSourceVMNode is "+compilerSourceVMNode)
		  def init = webDeployerParser.addNode("init-param", "", compilerSourceVMNode.getParentNode(),"SIBLING")
		  def paraName = webDeployerParser.addNode("param-name", "compilerTargetVM", init)
		  compilerTargetVMNode = paraName
		  def paravalue = webDeployerParser.addNode("param-value", "1.6", init)
	  }
	  Logger.log('DEBUG',"server: end update jsp compiler version ")
  }

}