// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss


import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import com.sas.config.jboss.Logger

//Velocity bug:
//http://issues.apache.org/jira/browse/VELOCITY-78
//http://issues.apache.org/jira/browse/VELOCITY-193

templateFile = args[0]          // Velocity template
templatePaths = args[1]
configPropertiesFile = args[2]  // Config properties
outputFile = args[3]            // File to save docpart to
encoding = args[4]              // Input/output encoding for Velocity templates
debug = args[5]                 // Enable debug information. true/false.

println "templateFile=$templateFile"
println "configPropertiesFile=$configPropertiesFile"
println "outputFile=$outputFile"
println "encoding=$encoding"
println "debug=$debug"
         
debug = new Boolean(debug).booleanValue()

// Load Scripting Properties
scriptProperties = new Properties()
scriptProperties.load(new FileInputStream(configPropertiesFile))

servers = scriptProperties.'server'
connectionfactories = scriptProperties.'connectionfactory'
queues = scriptProperties.'queue'
topics = scriptProperties.'topic'
datasources = scriptProperties.'datasource'
mailsessions = scriptProperties.'mailsession'
loginmodules = scriptProperties.'loginmodule'
applications = scriptProperties.'application'
externalcontexts = scriptProperties.'externalcontext'
listseparator = scriptProperties.'listseparator'

def logLevel = scriptProperties.'config.loglevel'
if (logLevel)
   Logger.logLevel = new String(logLevel)
else
   Logger.logLevel = new String("DEBUG")

if (listseparator==null) listseparator = / /

Logger.log('DEBUG', "servers = " + servers)
Logger.log('DEBUG', "connectionfactories = " + connectionfactories)
Logger.log('DEBUG', "queues = " + queues)
Logger.log('DEBUG', "topics = " + topics)
Logger.log('DEBUG', "datasources = " + datasources)
Logger.log('DEBUG', "mailsessions = " + mailsessions)
Logger.log('DEBUG', "loginmodules = " + loginmodules)
Logger.log('DEBUG', "applications = " + applications)
Logger.log('DEBUG', "externalcontext = " + externalcontexts)
Logger.log('DEBUG', "listseparator = " + listseparator)

def serverList
def connectionfactoryList
def queueList
def topicList
def datasourceList
def mailsessionList
def loginmoduleList
def policyList
def applicationList
def externalcontextList


if (servers!=null) serverList = servers.split(listseparator)
if (connectionfactories!=null) connectionfactoryList= connectionfactories.split(listseparator)
if (queues!=null) queueList= queues.split(listseparator)
if (topics!=null) topicList= topics.split(listseparator)
if (datasources!=null) datasourceList= datasources.split(listseparator)
if (mailsessions!=null) mailsessionList= mailsessions.split(listseparator)
if (loginmodules!=null) loginmoduleList= loginmodules.split(listseparator)
if (applications!=null) applicationList= applications.split(listseparator)
if (externalcontexts!=null) externalcontextList= externalcontexts.split(listseparator)

//Go start configuring all the pieces/parts
jbossInstallDir = scriptProperties.'config.jboss.install.dir'
Logger.log('DEBUG', "jbossInstallDir = " + jbossInstallDir)

webappsrv = new Webappsrv();
webappsrv.auto = (scriptProperties.get("config.type") == "auto")
webappsrv.wrapper = false
webappsrv.vendor = "JBoss"
webappsrv.name = "JBoss"
webappsrv.version = scriptProperties.get("config.appserver.version")
if (webappsrv.version) webappsrv.version = webappsrv.version[0]
webappsrv.fullversion = scriptProperties.get("config.appserver.version")
webappsrv.installdir = jbossInstallDir
webappsrv.jdkversion = scriptProperties.get("config.java.version")
webappsrv.host = "LocalHost" 
webappsrv.servers = new ArrayList<Server>();
webappsrv.hosttype = scriptProperties.get("config.host.type")
webappsrv.configdir = scriptProperties.get("config.lev.dir")

//Here are the main loops for each component
serverList.each { server ->
    Logger.log('DEBUG',"jbossScripting: processing server $server")
    Logger.log('DEBUG',"jbossScripting: Made it to line 90")
    def serverConfig = new Server()
    
    Logger.log('DEBUG',"jbossScripting: Made it to line 92")
    def name = scriptProperties.get("server."+server+".name") 
    def httpport = scriptProperties.("server."+server+".port.webserverHttp")
    def portIncrement = scriptProperties.get("server."+server+".portIncrement")
    def jmssecurity = scriptProperties.("server."+server+".jmssecurity")
    def jmssecurityuser = scriptProperties.("server."+server+".jmssecurity.user")
    def jmssecuritypw = scriptProperties.("server."+server+".jmssecurity.pw")
    def jvmOptions = scriptProperties.("server."+server+".options")
    def wrapperDir = scriptProperties.get("config.tanuki.wrapper.dir")
    
    if (wrapperDir) {
    	webappsrv.wrapper = true
    }
    	
    def inc = Integer.valueOf(portIncrement)
    
    if (httpport == null){
    	httpport = 8080 + inc
    }
    
    serverConfig.name = name
    serverConfig.httpport = httpport
    serverConfig.jmssecurity = jmssecurity
    serverConfig.jmssecurityuser = jmssecurityuser
    serverConfig.jmssecuritypw = jmssecuritypw
    serverConfig.wrapperdir = wrapperDir
    serverConfig.portIncrement = portIncrement
    
    serverConfig.options = jvmOptions.tokenize(";").collect {options -> options as String}

        
    if (loginmoduleList) serverConfig.policies = new ArrayList();
    loginmoduleList.each { loginmodule ->  

       def policyname = scriptProperties.get("loginmodule."+loginmodule+".policy")
       def classname = scriptProperties.get("loginmodule."+loginmodule+".classname")  
       def flag = scriptProperties.get("loginmodule."+loginmodule+".flag") 
       def servername = scriptProperties.get("loginmodule."+loginmodule+".servername")  
       def options = scriptProperties.get("loginmodule."+loginmodule+".options") 
       def deleted = scriptProperties.get("loginmodule."+loginmodule+".deleted")
       
       if (servername == name)
       {
           def loginmoduleConfig = new Loginmodule(policyname,classname,flag,servername,options,deleted)
           def foundPolicy = null
           if (serverConfig.policies){
             serverConfig.policies.each { policy ->
                if (policy.policyName == policyname) {
                   foundPolicy = policy
                }
             }
            }
            if (foundPolicy == null) {
              foundPolicy = new Policy()
              foundPolicy.policyName = policyname
              foundPolicy.loginmodules = new ArrayList()
              foundPolicy.loginmodules.add(loginmoduleConfig)
              serverConfig.policies.add(foundPolicy)
           }
           else {
              foundPolicy.loginmodules.add(loginmoduleConfig)
           }
       }
    }

    if (connectionfactoryList) serverConfig.connectionfactories = new ArrayList();
    connectionfactoryList.each { connectionfactory ->

        def cfname = scriptProperties.get("connectionfactory."+connectionfactory+".name")
        def jndiname = scriptProperties.get("connectionfactory."+connectionfactory+".jndiname")  
        def servername = scriptProperties.get("connectionfactory."+connectionfactory+".servername") 

        if (servername == name) {
            def cfConfig = new Connectionfactory(cfname,jndiname,servername)
            
            serverConfig.connectionfactories.add(cfConfig)
        }                
    }
    if (queueList) serverConfig.queues = new ArrayList();
    queueList.each { queue ->

        def qname = scriptProperties.get("queue."+queue+".name")
        def jndiname = scriptProperties.get("queue."+queue+".jndiname")  
        def servername = scriptProperties.get("queue."+queue+".servername") 

        if (servername == name) {
            def queueConfig = new Queue(qname,jndiname,servername)
            serverConfig.queues.add(queueConfig)
        }                
            
    }
    if (topicList) serverConfig.topics = new ArrayList();
    topicList.each { topic ->

        def tname = scriptProperties.get("topic."+topic+".name")
        def jndiname = scriptProperties.get("topic."+topic+".jndiname")  
        def servername = scriptProperties.get("topic."+topic+".servername") 

        if (servername == name) {
            def topicConfig = new Topic(tname,jndiname,servername)
            serverConfig.topics.add(topicConfig)
        }                
            
    }
    if (datasourceList) serverConfig.datasources = new ArrayList();
    datasourceList.each { datasource ->

        def datasourcename = scriptProperties.get("datasource."+datasource+".name")
        def jndiname = scriptProperties.get("datasource."+datasource+".jndiname")  
        def servername = scriptProperties.get("datasource."+datasource+".servername")
        def classpath = scriptProperties.get("datasource."+datasource+".classpath")
        def driver = scriptProperties.get("datasource."+datasource+".driver")
        def xa = scriptProperties.get("datasource."+datasource+".xa")
        def connectionUrl = scriptProperties.get("datasource."+datasource+".connectionUrl")
        def user = scriptProperties.get("datasource."+datasource+".user")
        def password = scriptProperties.get("datasource."+datasource+".password")
        def driverOptions = scriptProperties.get("datasource."+datasource+".options")
        def hostName = scriptProperties.get("datasource."+datasource+".hostName")
        def serverPort = scriptProperties.get("datasource."+datasource+".serverPort")

    Logger.log('DEBUG',"jbossScripting: processing datasource $datasource")
    Logger.log('DEBUG',"jbossScripting: servername == ${servername}")

        if (servername == name) {
            def dsConfig = new Datasource(datasourcename,jndiname, servername, classpath, driver, xa,connectionUrl, user, password, driverOptions, hostName, serverPort)           
            serverConfig.datasources.add(dsConfig)
        }                            
    }
       
    if (mailsessionList) serverConfig.mailsessions = new ArrayList();
    mailsessionList.each { mailsession ->

        def mailsessionname = scriptProperties.get("mailsession."+mailsession+".name")
        def jndiname = scriptProperties.get("mailsession."+mailsession+".jndiname")  
        def servername = scriptProperties.get("mailsession."+mailsession+".servername")
        def mailhost = scriptProperties.get("mailsession."+mailsession+".mailhost")
        def user = scriptProperties.get("mailsession."+mailsession+".user")
        def password = scriptProperties.get("mailsession."+mailsession+".password")

        if (servername == name) {
            def msConfig = new Mailsession(mailsessionname,jndiname,servername,mailhost,user,password )            
            serverConfig.mailsessions.add(msConfig)
        }                
            
    }
    if (applicationList)serverConfig.applications = new ArrayList();
    applicationList.each { application ->

        def appname = scriptProperties.get("application."+application+".appname")
        def pathtoear = scriptProperties.get("application."+application+".pathtoear")  
        def servername = scriptProperties.get("application."+application+".servername")  
        def deploymentdir = scriptProperties.get("application."+application+".deploymentdir")  
        def explode = scriptProperties.get("application."+application+".explode")

        if (servername == name) {
            def apConfig = new Application(appname,pathtoear,servername,deploymentdir,explode )            
            serverConfig.applications.add(apConfig)
        }                
            
    }

    Logger.log('DEBUG',"jbossScripting: processing externalcontextList : "+externalcontextList)
    if (externalcontextList)serverConfig.externalcontexts = new ArrayList();
    externalcontextList.each { externalcontext ->

        def local = scriptProperties.get("externalcontext."+externalcontext+".local")
        def remote = scriptProperties.get("externalcontext."+externalcontext+".remote")  
        def url = scriptProperties.get("externalcontext."+externalcontext+".url") 
        def initialcontextfactory = scriptProperties.get("externalcontext."+externalcontext+".initialcontextfactory")  
        def servername = scriptProperties.get("externalcontext."+externalcontext+".servername")  
        Logger.log('DEBUG',"jbossScripting: processing externalcontext $externalcontext")
        if (servername == name) {
            Logger.log('DEBUG',"jbossScripting: creating externalcontext $externalcontext in velocity context")
            def ecConfig = new Externalcontext(local,remote,url,initialcontextfactory,servername )            
            serverConfig.externalcontexts.add(ecConfig)
        }                
            
    }

    webappsrv.servers.add(serverConfig)
}

/* Gentlemen, Start your Engines */
// Initialize Velocity Engine
Properties velocityProperties = new Properties();    
velocityProperties.put("resource.loader","file");
velocityProperties.put("file.resource.loader.description", "Velocity File Resource Loader");
velocityProperties.put("file.resource.loader.path", templatePaths)
velocityProperties.put("file.resource.loader.class", "org.apache.velocity.runtime.resource.loader.FileResourceLoader");             
velocityProperties.put("resource.manager.logwhenfound", "true")
velocityProperties.put("directive.foreach.counter.name", "velocityCount")
velocityProperties.put("directive.foreach.iterator.name", "velocityHasNext")
velocityProperties.put("directive.foreach.counter.initial.value", "1")
//velocityProperties.put("runtime.log", log.getAbsolutePath()) 
velocityProperties.put("input.encoding", encoding)
velocityProperties.put("output.encoding", encoding)
velocityProperties.put("runtime.log.invalid.references", "true")


velocityProperties.put("velocimacro.library", "macros.vm")
velocityEngine = new VelocityEngine();
velocityEngine.init(velocityProperties);


/* Grab a template so we know what work needs to be done */
Template template = velocityEngine.getTemplate(templateFile, encoding);


/* update the context with our webappsrv object */
VelocityContext context = new VelocityContext();
context.put("webappsrv", webappsrv);
context.put("fileseparator",File.separator);
context.put("pathseparator",File.pathSeparator);

/* put it all together and dump to our file */
OutputStreamWriter outputStream = new OutputStreamWriter(new FileOutputStream(new File(outputFile)),encoding)         
template.merge(context, outputStream);
outputStream.close()

    

/*
 * 
 * END OF MAIN 
 * 
 */
 


