// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved


package com.sas.config.jboss
import com.sas.config.jboss.*
import com.sas.config.util.PasswordField

public class Utils {
    static void replace(File file, Map<String, String> replaceMap) { 
        def text = file.text
        replaceMap.keySet().each{ regEx ->
           text = text.replaceAll(regEx, replaceMap.get(regEx))
        }
        file.write(text) 
    }   

    static void explode(String archive, String destination, String tempdir){

    	/**
    	 * unzips ear files
    	 */
         Logger.log('DEBUG', "Ant.explode - archive=$archive")
         Logger.log('DEBUG', "Ant.explode - destination=$destination")
         Logger.log('DEBUG', "Ant.explode - tempdir=$tempdir")
            
         Ant.mkdir(tempdir)
         
         Logger.log('DEBUG', "Ant.explode - finished mkdir")

         Ant.mkdir(destination)
            
         explodeCompressedFile(archive,tempdir)
         File archiveFile = new File(archive)
    	
    	 def name=archiveFile.getName()
    	 
         Logger.log('DEBUG', "Ant.explode - explodeInputFile($tempdir,$destination/$name)")
    	 explodeInputFile(tempdir,destination+"/"+name)
         
    	 Ant.deleteDir(tempdir)

       
    }
    static void explodeInputFile(input, output){
        def filePattern = ~/.*\.(war|ear)$/
        File inputFile = new File(input)
        if(inputFile.isDirectory()){
            inputFile.eachFile{
                def name = it.getName()
                 if(!it.isDirectory()) {
                	 Logger.log('DEBUG',"Ant.explodeInputFile - Found $name , should attempt to explode if .war")
                     if (name =~ filePattern) {
                    	 Logger.log('DEBUG',"Ant.explodeInputFile - explodeCompressedFile($input, $output/$name)")
                         explodeCompressedFile(input+"/"+name, output+"/"+name)
                     } else {
                    	 Logger.log('DEBUG',"Ant.explodeInputFile - copyFileToDir($input/$name,$output, true)")
                         Ant.copyFileToDir(input+"/"+name,output, "true")
                     }
                 } else{
                	 Logger.log('DEBUG',"Ant.explodeInputFile - copyDir($input,$output,true)")
                     Ant.copyDir(input+"/"+name,output+"/"+name, "true")
                 }
            }
        }else{
        	def name = inputFile.getName()
            if(name =~ filePattern){
            	Logger.log('DEBUG',"Ant.explodeInputFile - explodeCompressedFile($input, $output/$name)")
                explodeCompressedFile(input, output+"/"+name)
            }else{
            	Logger.log('DEBUG',"Ant.explodeInputFile - copyFileToDir($input,$output, true)")
                Ant.copyFileToDir(input,output, "true")
            }
        }
    }

   static void explodeCompressedFile(input, output){
        try{
            Ant.mkdir(output)
            Ant.unzip(input,output,"true")
        }catch(Throwable t){
		    println "\nFailed to unzip $input: ${t.toString()}"
// Catching this can cause silent failures.  Be sure and throw it back up the chain.			
// 		    t.printStackTrace();
//		    throw (new java.lang.RuntimeException("Failed to unzip $input", t))
        }
    }
   
   static String promptForPassword(java.io.InputStream input, String prompt){
	   String password = null;
	   try {
		  java.io.Console console = System.console()
		  if (console){
			  Logger.log('DEBUG',"util - found java.io.Console, using for prompting")
			  password = String.valueOf(console.readPassword(prompt));
		  } else {
			  Logger.log('DEBUG',"util - no java.io.Console, using InputStream for prompting")
		      password = PasswordField.getPassword(input, prompt);
		  }
	   } catch(IOException ioe) {
		  ioe.printStackTrace();
		  throw (new java.lang.RuntimeException("Prompt for password failed", ioe))
	   }
	   return password
   }
   
   static String promptForPassword( String prompt){
	   return promptForPassword(System.in, prompt)
   }
   
   static String encodeDecodePassword(String password, String encoding, String prompt){
	   if (password.equalsIgnoreCase("not-cached")) {
		   //String prompt = "\nEnter Password \n\tDatasource=$name \n\tuser=$user \n\tPassword="
		   password = com.sas.config.jboss.Utils.promptForPassword(prompt)
		   if (encoding.equalsIgnoreCase("true")){
			   password = com.sas.util.SasPasswordString.encode(password)
		   }
	   }else{
		   if (encoding.equalsIgnoreCase("false")){
			   password = com.sas.util.SasPasswordString.decode(password)
		   }
	   }
       return password
   }
}
