// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.jboss.docparts


import java.io.File;
import java.util.List;
import java.util.HashMap;
import java.util.regex.Pattern;

import com.sas.config.Context
import com.sas.config.ContextBuilder
import com.sas.config.ResourceLoader
import com.sas.config.DirectoryResourceLoader
import com.sas.config.PropertyContextBuilder
import com.sas.config.Resource
import com.sas.config.jboss.docparts.JBossResource
import com.sas.config.jboss.*

class CompareServerNames implements Comparator<Resource> {
    CompareServerNames() {}
    int compare(Object o1, Object o2) {
        Resource r1 = (Resource) o1
        Resource r2 = (Resource) o2
        String s1 = r1["servername"]
        String s2 = r2["servername"]
        if (s1 == null) 
            s1 = ""
        if (s2 == null)
            s2 = ""
        return s1.compareTo(s2)
    }
}

class JBossContextBuilder extends PropertyContextBuilder {

	String configPropertiesFile;
	
    public JBossContextBuilder(String target) {
        // resArrays = new HashMap<String, Object>()
		println ">>> " + getClass().getName() + "JBossContextBuilder(String target)"
		println "<<< " + getClass().getName() + "JBossContextBuilder(String target)"
		
    }

    public JBossContextBuilder(String target, String propertyFile) {
        // resArrays = new HashMap<String, Object>()
		println ">>> " + getClass().getName() + "JBossContextBuilder(String target, String propertyFile)"
		setConfigPropertiesFile(propertyFile);
		println "<<< " + getClass().getName() + "JBossContextBuilder(String target, String propertyFile)"
    }

	public void setConfigPropertiesFile(String propertyFile)
	{
		configPropertiesFile = propertyFile;
	}
	
    @Override
    protected Context buildInternal(File templateFile, Context context) {   
        super.buildInternal(templateFile, context)
		println "<<< " + getClass().getName() + ".buildInternal()"
		println "<<< " + getClass().getName() + "templateFile="+templateFile.toString()
		println "<<< " + getClass().getName() + "configPropertiesFile="+configPropertiesFile
		
		Webappsrv webappsrv = loadResources(configPropertiesFile);
		
        context.setProperty("webappsrv", webappsrv)
		context.setProperty("fileseparator",File.separator);
		context.setProperty("pathseparator",File.pathSeparator);
		
		return context
    }
  
    protected Webappsrv loadResources(String  configPropertiesFile) {
/*		
		templateFile = args[0]          // Velocity template
		templatePaths = args[1]
		configPropertiesFile = args[2]  // Config properties
		outputFile = args[3]            // File to save docpart to
		encoding = args[4]              // Input/output encoding for Velocity templates
		debug = args[5]                 // Enable debug information. true/false.

		
		println "templateFile=$templateFile"

		println "outputFile=$outputFile"
		println "encoding=$encoding"
		println "debug=$debug"
*/	

		println ">>> " + getClass().getName() + "loadResources()"
		println "    " + getClass().getName() + "configPropertiesFile="+configPropertiesFile
		
		Webappsrv webappsrv;
		
		Boolean debug = true;
		
		// Load Scripting Properties
		Properties scriptProperties = new Properties()
		scriptProperties.load(new FileInputStream(configPropertiesFile))
		try {
			// Load Users from password Scripting Properties
			String passwordFile = (new File(configPropertiesFile)).getParent() + "/jbossScriptingCredentials.properties"
			scriptProperties.load(new FileInputStream(passwordFile))
		} catch (Exception e) {
		    // Ignore password file if doesn't exist or problems with it
		}
		
		String servers = scriptProperties.'server'
		String connectionfactories = scriptProperties.'connectionfactory'
		String queues = scriptProperties.'queue'
		String topics = scriptProperties.'topic'
		String datasources = scriptProperties.'datasource'
		String jdbcproviders = scriptProperties.'jdbcprovider'
		String mailsessions = scriptProperties.'mailsession'
		String loginmodules = scriptProperties.'loginmodule'
		String applications = scriptProperties.'application'
		String externalcontexts = scriptProperties.'externalcontext'
		String listseparator = scriptProperties.'listseparator'
		
		def logLevel = scriptProperties.'config.loglevel'
		if (logLevel)
		   Logger.logLevel = new String(logLevel)
		else
		   Logger.logLevel = new String("DEBUG")
		
		if (listseparator==null) listseparator = / /
		
		Logger.log('DEBUG', "servers = " + servers)
		Logger.log('DEBUG', "connectionfactories = " + connectionfactories)
		Logger.log('DEBUG', "queues = " + queues)
		Logger.log('DEBUG', "topics = " + topics)
		Logger.log('DEBUG', "datasources = " + datasources)
		Logger.log('DEBUG', "jdbcproviders = " + jdbcproviders)
		Logger.log('DEBUG', "mailsessions = " + mailsessions)
		Logger.log('DEBUG', "loginmodules = " + loginmodules)
		Logger.log('DEBUG', "applications = " + applications)
		Logger.log('DEBUG', "externalcontext = " + externalcontexts)
		Logger.log('DEBUG', "listseparator = " + listseparator)
		
		String [] serverList
		String []  connectionfactoryList
		String []  queueList
		String []  topicList
		String []  datasourceList
		String []  jdbcproviderList
		String []  mailsessionList
		String []  loginmoduleList
		String []  policyList
		String []  applicationList
		String []  externalcontextList
		
		
		if (servers!=null) serverList = servers.split(listseparator)
		if (connectionfactories!=null) connectionfactoryList= connectionfactories.split(listseparator)
		if (queues!=null) queueList= queues.split(listseparator)
		if (topics!=null) topicList= topics.split(listseparator)
		if (datasources!=null) datasourceList= datasources.split(listseparator)
		if (jdbcproviders!=null) jdbcproviderList= jdbcproviders.split(listseparator)
		if (mailsessions!=null) mailsessionList= mailsessions.split(listseparator)
		if (loginmodules!=null) loginmoduleList= loginmodules.split(listseparator)
		if (applications!=null) applicationList= applications.split(listseparator)
		if (externalcontexts!=null) externalcontextList= externalcontexts.split(listseparator)
		
		//Go start configuring all the pieces/parts
		String jbossInstallDir = scriptProperties.'config.jboss.install.dir'
		Logger.log('DEBUG', "jbossInstallDir = " + jbossInstallDir)
		
		webappsrv = new Webappsrv();
		webappsrv.auto = (scriptProperties.get("config.type") == "auto")
		webappsrv.wrapper = false
		webappsrv.vendor = "JBoss"
		webappsrv.name = "JBoss"
		webappsrv.version = scriptProperties.get("config.appserver.version")
		if (webappsrv.version) webappsrv.version = webappsrv.version[0]
		webappsrv.fullversion = scriptProperties.get("config.appserver.version")
		webappsrv.installdir = jbossInstallDir
		webappsrv.jdkversion = scriptProperties.get("config.java.version")
		webappsrv.jmsfolder = scriptProperties.get("config.appserver.jmsfolder")
		webappsrv.host = "LocalHost"
		webappsrv.servers = new ArrayList<Server>();
		webappsrv.hosttype = scriptProperties.get("config.host.type")
		webappsrv.configdir = scriptProperties.get("config.lev.dir")
		
		// config.lev.dir is currently set to "Lev1/Web/Common/JBoss". Need to back up to get Scripts/JBoss
		def scriptPath = scriptProperties.get("config.lev.dir")+"/../../Scripts/JBoss"
		def scriptFile = new File(scriptPath)
		webappsrv.scriptdir = scriptFile.canonicalPath
		
		//Here are the main loops for each component
		serverList.each { server ->
			Logger.log('DEBUG',"jbossScripting: processing server $server")
			Logger.log('DEBUG',"jbossScripting: Made it to line 90")
			Server serverConfig = new Server()
			
			Logger.log('DEBUG',"jbossScripting: Made it to line 92")
			String name = scriptProperties.get("server."+server+".name")
			String httpport = scriptProperties.("server."+server+".port.webserverHttp")
			String portIncrement = scriptProperties.get("server."+server+".portIncrement")
	        String serverSource = scriptProperties.get("server."+server+".source")
			String jmssecurity = scriptProperties.("server."+server+".jmssecurity")
			String jmssecurityuser = scriptProperties.("server."+server+".jmssecurity.user")
			String jmssecuritypw = scriptProperties.("server."+server+".jmssecurity.password")
			String jvmOptions = scriptProperties.("server."+server+".options")
			String wrapperDir = scriptProperties.get("config.tanuki.wrapper.dir")
			
			if (wrapperDir) {
				webappsrv.wrapper = true
			}
				
			def inc = Integer.valueOf(portIncrement)
			
			if (httpport == null){
				httpport = 8080 + inc
			}
			
			serverConfig.name = name
			serverConfig.httpport = httpport
			serverConfig.jmssecurity = Boolean.valueOf(jmssecurity)
			serverConfig.jmssecurityuser = jmssecurityuser
			serverConfig.jmssecuritypw = jmssecuritypw
			serverConfig.wrapperdir = wrapperDir
			serverConfig.portIncrement = portIncrement
			serverConfig.serverSource = serverSource
			
			serverConfig.options = jvmOptions.tokenize(";").collect {options -> options as String}
		
				
			if (loginmoduleList) serverConfig.policies = new ArrayList();
			loginmoduleList.each { loginmodule ->
		
			   def policyname = scriptProperties.get("loginmodule."+loginmodule+".policy")
			   def classname = scriptProperties.get("loginmodule."+loginmodule+".classname")
			   def flag = scriptProperties.get("loginmodule."+loginmodule+".flag")
			   def servername = scriptProperties.get("loginmodule."+loginmodule+".servername")
			   def options = scriptProperties.get("loginmodule."+loginmodule+".options")
			   def deleted = scriptProperties.get("loginmodule."+loginmodule+".deleted")
			   def user = scriptProperties.get("loginmodule."+loginmodule+".user")
			   def password = scriptProperties.get("loginmodule."+loginmodule+".password")

			   if (servername == name)
			   {
				   def loginmoduleConfig = new Loginmodule(policyname,classname,flag,servername,options,user,password,deleted)
				   def foundPolicy = null
				   if (serverConfig.policies){
					 serverConfig.policies.each { policy ->
						if (policy.policyName == policyname) {
						   foundPolicy = policy
						}
					 }
					}
					if (foundPolicy == null) {
					  foundPolicy = new Policy()
					  foundPolicy.policyName = policyname
					  foundPolicy.loginmodules = new ArrayList()
					  foundPolicy.loginmodules.add(loginmoduleConfig)
					  serverConfig.policies.add(foundPolicy)
				   }
				   else {
					  foundPolicy.loginmodules.add(loginmoduleConfig)
				   }
			   }
			}
		
			if (connectionfactoryList) serverConfig.connectionfactories = new ArrayList();
			connectionfactoryList.each { connectionfactory ->
		
				def cfname = scriptProperties.get("connectionfactory."+connectionfactory+".name")
				def jndiname = scriptProperties.get("connectionfactory."+connectionfactory+".jndiname")
				def servername = scriptProperties.get("connectionfactory."+connectionfactory+".servername")
				def useNamingAlias = scriptProperties.get("connectionfactory."+connectionfactory+".useNamingAlias")
				def maxConnections = scriptProperties.get("connectionfactory."+connectionfactory+".maxConnections")
				
				if (servername == name) {
					def cfConfig = new Connectionfactory(cfname,jndiname,servername, useNamingAlias, maxConnections)
					
					serverConfig.connectionfactories.add(cfConfig)
				}
			}
			if (queueList) serverConfig.queues = new ArrayList();
			queueList.each { queue ->
		
				def qname = scriptProperties.get("queue."+queue+".name")
				def jndiname = scriptProperties.get("queue."+queue+".jndiname")
				def servername = scriptProperties.get("queue."+queue+".servername")
		
				if (servername == name) {
					def queueConfig = new Queue(qname,jndiname,servername)
					serverConfig.queues.add(queueConfig)
				}
					
			}
			if (topicList) serverConfig.topics = new ArrayList();
			topicList.each { topic ->
		
				def tname = scriptProperties.get("topic."+topic+".name")
				def jndiname = scriptProperties.get("topic."+topic+".jndiname")
				def servername = scriptProperties.get("topic."+topic+".servername")
		
				if (servername == name) {
					def topicConfig = new Topic(tname,jndiname,servername)
					serverConfig.topics.add(topicConfig)
				}
					
			}
			if (datasourceList) serverConfig.datasources = new ArrayList();
			datasourceList.each { datasource ->
		
				def datasourcename = scriptProperties.get("datasource."+datasource+".name")
				def jndiname = scriptProperties.get("datasource."+datasource+".jndiname")
				def servername = scriptProperties.get("datasource."+datasource+".servername")
				def classpath = scriptProperties.get("datasource."+datasource+".classpath")
				def driver = scriptProperties.get("datasource."+datasource+".driver")
				def xa = scriptProperties.get("datasource."+datasource+".xa")
				def connectionUrl = scriptProperties.get("datasource."+datasource+".connectionUrl")
				def user = scriptProperties.get("datasource."+datasource+".user")
				def password = scriptProperties.get("datasource."+datasource+".password")
				def driverOptions = scriptProperties.get("datasource."+datasource+".options")
				def hostName = scriptProperties.get("datasource."+datasource+".hostName")
				def serverPort = scriptProperties.get("datasource."+datasource+".serverPort")
		
			Logger.log('DEBUG',"jbossScripting: processing datasource $datasource")
			Logger.log('DEBUG',"jbossScripting: servername == ${servername}")
		
				if (servername == name) {
					def dsConfig = new Datasource(datasourcename,jndiname, servername, classpath, driver, xa,connectionUrl, user, password, driverOptions, hostName, serverPort)
					serverConfig.datasources.add(dsConfig)
				}
			}
			
			if (jdbcproviderList) serverConfig.jdbcproviders = new ArrayList();
			jdbcproviderList.each { jdbcprovider ->
		
				def jdbcprovidername = scriptProperties.get("jdbcprovider."+jdbcprovider+".name")
				def servername = scriptProperties.get("jdbcprovider."+jdbcprovider+".servername")
				def classpath = scriptProperties.get("jdbcprovider."+jdbcprovider+".classpath")
	  
			    Logger.log('DEBUG',"jbossScripting: processing jdbcprovider $jdbcprovider")
			    Logger.log('DEBUG',"jbossScripting: servername == ${servername}")
		
				if (servername == name) {
					def providerConfig = new JDBCProvider(jdbcprovidername,servername, classpath)
					serverConfig.jdbcproviders.add(providerConfig)
				}
			}

			if (mailsessionList) serverConfig.mailsessions = new ArrayList();
			mailsessionList.each { mailsession ->
		
				def mailsessionname = scriptProperties.get("mailsession."+mailsession+".name")
				def jndiname = scriptProperties.get("mailsession."+mailsession+".jndiname")
				def servername = scriptProperties.get("mailsession."+mailsession+".servername")
				def mailhost = scriptProperties.get("mailsession."+mailsession+".mailhost")
				def user = scriptProperties.get("mailsession."+mailsession+".user")
				def password = scriptProperties.get("mailsession."+mailsession+".password")
		
				if (servername == name) {
					def msConfig = new Mailsession(mailsessionname,jndiname,servername,mailhost,user,password )
					serverConfig.mailsessions.add(msConfig)
				}
					
			}
			if (applicationList)serverConfig.applications = new ArrayList();
			applicationList.each { application ->
		
				def appname = scriptProperties.get("application."+application+".appname")
				def pathtoear = scriptProperties.get("application."+application+".pathtoear")
				def servername = scriptProperties.get("application."+application+".servername")
				def deploymentdir = scriptProperties.get("application."+application+".deploymentdir")
				def explode = scriptProperties.get("application."+application+".explode")
		
				if (servername == name) {
					def apConfig = new Application(appname,pathtoear,servername,deploymentdir,explode )
					serverConfig.applications.add(apConfig)
				}
					
			}
		
			Logger.log('DEBUG',"jbossScripting: processing externalcontextList : "+externalcontextList)
			if (externalcontextList)serverConfig.externalcontexts = new ArrayList();
			externalcontextList.each { externalcontext ->
		
				def local = scriptProperties.get("externalcontext."+externalcontext+".local")
				def remote = scriptProperties.get("externalcontext."+externalcontext+".remote")
				def url = scriptProperties.get("externalcontext."+externalcontext+".url")
				def initialcontextfactory = scriptProperties.get("externalcontext."+externalcontext+".initialcontextfactory")
				def servername = scriptProperties.get("externalcontext."+externalcontext+".servername")
				Logger.log('DEBUG',"jbossScripting: processing externalcontext $externalcontext")
				if (servername == name) {
					Logger.log('DEBUG',"jbossScripting: creating externalcontext $externalcontext in velocity context")
					def ecConfig = new Externalcontext(local,remote,url,initialcontextfactory,servername )
					serverConfig.externalcontexts.add(ecConfig)
				}
					
			}
		
			webappsrv.servers.add(serverConfig)
		}
		println "<<< " + getClass().getName() + "loadResources()"
		
		return webappsrv;
    }		

}
