// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.sdw;

import java.io.File
import java.io.FileInputStream
import org.apache.commons.io.FileUtils
import org.apache.commons.io.filefilter.FileFilterUtils

import com.sas.config.Docpart
import com.sas.config.sdw.SDW
import com.sas.config.TemplateEngine
import com.sas.config.VelocityTemplateEngine
import com.sas.config.VelocityUnicodeFileResourceLoader

class SDWDocpart extends Docpart {
	private String encoding = "UTF-8" 	// Default encoding
	private SDW sdw
	
	
	public SDWDocpart(SDW sdw) {
		this.sdw = sdw
		String install_currprodconfig_dir = sdw.getProperty("install.currprod.config.dir")
		String os_localhost_selected_locale = sdw.getProperty("os.localhost.selected.locale")
		String temp_dir = sdw.getProperty("temp.dir")	// Log Directory.
		String file_encoding = sdw.getProperty("file.encoding")
		String onZosHosts = sdw.getProperty("onZosHosts")
		
		println "install_currprodconfig_dir = $install_currprodconfig_dir"
		println "os_localhost_selected_locale = $os_localhost_selected_locale"
		println "temp_dir = $temp_dir"
		println "file_encoding = $file_encoding"
		println "onZosHosts = $onZosHosts"
		
		// Determine template path for l10n.
		List<File> templatePaths = new ArrayList<File>()
		if(os_localhost_selected_locale != "en") {
			templatePaths.add(new File(install_currprodconfig_dir + "/" + os_localhost_selected_locale))
		}
		templatePaths.add(new File(install_currprodconfig_dir + "/en"))
		println "templatePaths = $templatePaths"
		
		// Determine encoding.
		if(onZosHosts != null && onZosHosts.length() > 0) {
			encoding = file_encoding
		}
		println "encoding = $encoding"				
		File tempDir = new File(temp_dir)
		
		VelocityTemplateEngine templateEngine = new VelocityTemplateEngine()
		templateEngine.setLogDirectory(tempDir)
		templateEngine.setEncoding(encoding)

		/*
		 * Defect: S0697745
		 * The code below does not work in SDW because Velocity is loaded in a higher classloader than the 
 		 * com.sas.config.jar. Therefore Velocity is unable to find the VelocityUnicodeFileResourceLoader class.
		 * Using workaround below to pre-process velocity templates.
		 */
		// templateEngine.setFileResourceLoaderClass("com.sas.config.VelocityUnicodeFileResourceLoader")
		// templateEngine.setTemplatePaths(templatePaths)

		// Begin Workaround 
		File tempTemplateDir = new File(tempDir, "vtemplates")
		if(tempTemplateDir.exists()) {
			FileUtils.deleteDirectory(tempTemplateDir)
		}
		FileUtils.forceMkdir(tempTemplateDir)
		List<File> tempTemplatePaths = new ArrayList<File>()		
		templatePaths.each() { path ->
			if(path.exists()) {
				File templatePath = new File(tempTemplateDir, path.getName())
				tempTemplatePaths.add(templatePath)
				FileUtils.copyDirectory(path, templatePath,  FileFilterUtils.suffixFileFilter(".vm"))
			}
		}
				
		tempTemplatePaths.each() { dir ->
			dir.eachFile() { file ->
				InputStream is = new FileInputStream(file)
				String text = VelocityUnicodeFileResourceLoader.readAndConvertToNativeChars(is)
				file.write(text,encoding)
			}
		}
		templateEngine.setTemplatePaths(tempTemplatePaths)
		// End Workaround
		
		setTemplateEngine(templateEngine)
	}	
	
	public void insert(File sdwFile, String tokenToReplace, File contentFile) {
		SDWHelper sdwHelper = new SDWHelper(sdw)		
		sdwHelper.insertDocpartWithReplace(sdwFile.getAbsolutePath(), tokenToReplace, contentFile.getText(encoding))		
	}
}
