// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.sdw.ant

import org.apache.tools.ant.Task
import org.apache.tools.ant.Project

import com.sas.config.sdw.SDW

public class AntSDW implements SDW {	
	private Project project;
	
	public AntSDW(Project p) { project = p }
	
	public String getProperty(String name) { return project.getProperties().get(name) }
	public void setProperty(String name, String value) { project.getProperties().put(name, value) }
	public Map getProperties() { return project.getProperties() }
	
	public void printProperties() {
		Map properties = project.getProperties()
		for( String propertyName : properties.keySet() ) {
			String propertyValue = properties.get(propertyName)
			println("$propertyName=$propertyValue")	
		}	
	}
	
	/**
	 * Execute Ant Target or Macro.
	 */
	public void execute(String name, Map parameters) {
		// Ant targets do not support parameters, force macro call.
		if(parameters != null && parameters.keySet().size() > 0) {
			executeMacro(name, parameters)			
		}
		else {	
			// TODO: This check is insufficient when an Ant macro and an Ant target are defined with the same name.
			//       One example of this is insertDocpartPrepend. 
			if(project.getTargets().keySet().contains(name)) {
				executeTarget(name)
			}
			else {
				executeMacro(name, parameters)
			}
		}
	}
	
	/**
	 * Execute Ant Target or Macro.
	 */	
	public void execute(String name) {
		if(project.getTargets().keySet().contains(name)) {
			executeTarget(name)
		}
		else {
			executeMacro(name, null)
		}		
	}
	
	private void executeTarget(String targetName) {
		project.executeTarget(targetName)
	}
	
	private void executeMacro(String macroName, Map attributes) {	
		def macro = project.createTask(macroName)    
		if(attributes != null && !attributes.isEmpty()) {
			for( String propertyName : attributes.keySet() ) {
				String propertyValue = attributes.get(propertyName)
				macro.setDynamicAttribute(propertyName.toLowerCase(), propertyValue)
			}	
		}
		macro.perform()
	}	
}