// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.util

/**
* Original java source obtained from http://java.sun.com/developer/technicalArticles/Security/pwordmask/
* Code sample
* License
*
* Copyright  2008, 2010 Oracle and/or its affiliates. All rights reserved. Use is subject to license terms.
* Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
*
*
* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
* Neither the name of Oracle Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
*
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*
* You acknowledge that this software is not designed, licensed or intended for use in the design, construction, operation or maintenance of any nuclear facility.
*
*/

import java.io.*;

/**
 * This class attempts to erase characters echoed to the console.
 */

class MaskingThread extends Thread {
   private volatile boolean stop;
   private char echochar = '*';
   private PrintStream out=System.out;
   
  public void setOut(PrintStream outputStream){
	  out = outputStream;
  }
  /**
   *@param prompt The prompt displayed to the user
   */
   public MaskingThread(String prompt) {
	  out.print(prompt);
   }

  /**
   * Begin masking until asked to stop.
   */
   public void run() {

	  int priority = Thread.currentThread().getPriority();
	  Thread.currentThread().setPriority(Thread.MAX_PRIORITY);

	  try {
		 stop = true;
		 while(stop) {
		   out.print("\010" + echochar);
		   try {
			  // attempt masking at this rate
			  Thread.currentThread().sleep(1);
		   }catch (InterruptedException iex) {
			  Thread.currentThread().interrupt();
			  return;
		   }
		 }
	  } finally { // restore the original priority
		 Thread.currentThread().setPriority(priority);
	  }
   }

  /**
   * Instruct the thread to stop masking.
   */
   public void stopMasking() {
	  this.stop = false;
   }
}
