// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.util

/**
 * Original java source obtained from http://java.sun.com/developer/technicalArticles/Security/pwordmask/
 * Code sample 
 * License
 *       
 * Copyright  2008, 2010 Oracle and/or its affiliates. All rights reserved. Use is subject to license terms. 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: 
 *  
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of Oracle Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *  
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 *  
 * 
 * You acknowledge that this software is not designed, licensed or intended for use in the design, construction, operation or maintenance of any nuclear facility.  
 * 
 */

import java.io.InputStream;
import java.io.PushbackInputStream;
import java.util.*;

/**
 * This class prompts the user for a password and attempts to mask input with "*"
 */

public class PasswordField {

  /**
   *@param input stream to be used (e.g. System.in)
   *@param prompt The prompt to display to the user.
   *@return The password as entered by the user.
   */

   public static final char[] getPassword(java.io.InputStream input, String prompt) throws IOException {
	  MaskingThread maskingthread = new MaskingThread(prompt);
	  Thread thread = new Thread(maskingthread);
	  thread.start();
	  
	  char space=(char)' ';
	  char newline=(char)'\n';
	  char cr=(char)'\r';
	  
	  char[] lineBuffer;
	  char[] buf;
	  int i;

	  buf = lineBuffer = new char[128];

	  int room = buf.length;
	  int offset = 0;
	  char c;
	  
	  loop:   while (true) {
		 switch (c = input.read()) {
			case -1:
			case newline:
			   break loop;

			case cr:
			   char c2 = input.read();
			   if ((c2 != newline) && (c2 != -1)) {
				  if (!(input instanceof PushbackInputStream)) {
					 input = new PushbackInputStream(input);
				  }
				  ((PushbackInputStream)input).unread(c2);
				} else {
				  break loop;
				}

				default:
				   if (--room < 0) {
					  buf = new char[offset + 128];
					  room = buf.length - offset - 1;
					  System.arraycopy(lineBuffer, 0, buf, 0, offset);
					  Arrays.fill(lineBuffer, space);
					  lineBuffer = buf;
				   }
				   buf[offset++] = (char) c;
				   break;
		 }
	  }
	  maskingthread.stopMasking();
	  if (offset == 0) {
		 return null;
	  }
	  char[] ret = new char[offset];
	  System.arraycopy(buf, 0, ret, 0, offset);
	  Arrays.fill(buf, space);
	  return ret;
   }
}
