// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.weblogic

import java.io.File
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import com.sas.config.VelocityTemplateEngine;
import com.sas.config.weblogic.WebLogicContextBuilder

public abstract class GroovyScriptInvoker {
	File m_globalPropertiesFile;
	File m_resourcePropertiesFile;
	Properties m_globalProps = null;
	Properties m_resourceProps = null;
	File m_resourceDirectory; // location of properties files
	
	public GroovyScriptInvoker(domainPropertiesFileName, resourcePropertiesFileName) {
		m_globalPropertiesFile = new File(domainPropertiesFileName);
		m_resourcePropertiesFile = new File(resourcePropertiesFileName);
		m_globalProps = getGlobalProperties();
		m_resourceDirectory = m_globalPropertiesFile.getParentFile();
	}
	
	public Properties getGlobalProperties() {
		   if ( m_globalProps != null ) return m_globalProps;
	       m_globalProps = new Properties();
	       try {
	    	   FileInputStream fh = new FileInputStream(m_globalPropertiesFile);
		       m_globalProps.load(fh);
		       fh.close();
	       } catch (FileNotFoundException e) {
	    	   e.printStackTrace();
	       } catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
	       }
	       return m_globalProps;
	}

	public Properties getResourceProperties() {
		   if ( m_resourceProps != null ) return m_resourceProps;
	       m_resourceProps = new Properties();
	       try {
	    	   FileInputStream fh = new FileInputStream(m_resourcePropertiesFile);
	    	   m_resourceProps.load(fh);
		       fh.close();
	       } catch (FileNotFoundException e) {
	    	   e.printStackTrace();
	       } catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
	       }
	       return m_resourceProps;
	}

	public Properties writeResourceProperties() {
		   if ( m_resourceProps == null ) return null;
	       try {
	    	   FileOutputStream fh = new FileOutputStream(m_resourcePropertiesFile);
	    	   m_resourceProps.store(fh, "");
		       fh.close();
	       } catch (FileNotFoundException e) {
	    	   e.printStackTrace();
	       } catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
	       }
	       return m_resourceProps;
	}

	Properties m_returnCodeProps = null;

	public Properties getReturnCodes() {
		if (m_returnCodeProps != null) return m_returnCodeProps;
		File returnCodeFile = new File(getGlobalProperties().getProperty("returnCodeProperties"));
		println "returnCodeFile=$returnCodeFile";
		m_returnCodeProps = new Properties();
	    try {
	    	FileInputStream fh = new FileInputStream(returnCodeFile);
		    m_returnCodeProps.load(fh);
		    fh.close();
	    } catch (FileNotFoundException e) {
	    	e.printStackTrace();
	    } catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
	    }
		return m_returnCodeProps;
	}

	public writeRC(String rc)
	{
		String rcvalue = getReturnCodes().getProperty(rc);
		Properties returnProps = new Properties();
		returnProps.setProperty("wlst.rc", rcvalue)
		String rcFileName = getGlobalProperties().getProperty("rcFileName");
		FileOutputStream fd = new FileOutputStream(new File(rcFileName));
		returnProps.store(fd, "Groovy return properties");
		fd.close();
	}

	public String readRC()
	{
		String rcFileName = getGlobalProperties().getProperty("rcFileName");
		Properties returnProps = new Properties();
		FileInputStream fd = new FileInputStream(new File(rcFileName));
		returnProps.load(fd);
		fd.close();
		return returnProps.getProperty("wlst.rc");
	}

}
