// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.weblogic;

import java.io.File;

import com.sas.config.Context
import com.sas.config.ContextBuilder
import com.sas.config.ResourceLoader
import com.sas.config.ResourceLoaderFilter
import com.sas.config.DirectoryResourceLoader
import com.sas.config.PropertyContextBuilder
import com.sas.config.Resource
import com.sas.config.weblogic.WebLogicResource

class WebLogicContextBuilder extends PropertyContextBuilder {
	private WebLogicResource domain = null
	private GlobalResource global = null
	private List<ManagedServer> servers = new ArrayList<ManagedServer>()
	private List<MailSession> mailSessions = new ArrayList<MailSession>()
	private List<Datasource> datasources = new ArrayList<Datasource>()
	private List<Datasource> jdbcproviders = new ArrayList<Datasource>()
	private List<WebLogicResource> jmsServers = new ArrayList<WebLogicResource>()
	private List<WebLogicResource> jmsConnectionFactories = new ArrayList<WebLogicResource>()
	private List<WebLogicResource> jmsQueues = new ArrayList<WebLogicResource>()
	private List<WebLogicResource> jmsTopics = new ArrayList<WebLogicResource>()
	private List<WebLogicResource> foreignJndiLinks = new ArrayList<WebLogicResource>()
	private List<WebLogicResource> loginModules = new ArrayList<WebLogicResource>()
	private List<WebLogicResource> loginPolicies = new ArrayList<WebLogicResource>()
	private Set<ApplicationResource> applications = new TreeSet<ApplicationResource>()	
	private List<WebLogicResource> users = new ArrayList<WebLogicResource>()	
	private List<WebLogicResource> policies = new ArrayList<WebLogicResource>()	
	private Credentials credentials = null
	private PropertyDocResource globalDoc = new PropertyDocResource()
	private PropertyDocResource credentialDoc = new PropertyDocResource()
	private PropertyDocResource commonDoc = new PropertyDocResource()
	private Set<PropertyDocResource> scriptingDocs = new TreeSet<WebLogicResource>()
	
	@Override
	protected Context buildInternal(File templateFile, Context context) {
		super.buildInternal(templateFile, context)
		context.setProperty("domain", domain)
		context.setProperty("global", global)
		context.setProperty("servers", (servers.isEmpty() ? null : servers))
		context.setProperty("mailSessions", (mailSessions.isEmpty() ?  null : mailSessions))
		context.setProperty("datasources", (datasources.isEmpty() ? null : datasources))	
		context.setProperty("jdbcproviders", (jdbcproviders.isEmpty() ? null : jdbcproviders))	
		context.setProperty("jmsServers", (jmsServers.isEmpty() ? null : jmsServers))
		context.setProperty("jmsConnectionFactories", (jmsConnectionFactories.isEmpty() ? null : jmsConnectionFactories))
		context.setProperty("jmsQueues", (jmsQueues.isEmpty() ? null : jmsQueues))
		context.setProperty("jmsTopics", (jmsTopics.isEmpty() ? null : jmsTopics))

		List<WebLogicResource> jmsModules = buildJmsModules()
		context.setProperty("jmsModules", (jmsModules.isEmpty()) ? null : jmsModules)

		List<WebLogicResource> jmsSubDeployments = buildJmsSubdeployments()
		context.setProperty("jmsSubdeployments", (jmsSubDeployments.isEmpty()) ? null : jmsSubDeployments)
		
		
		context.setProperty("foreignJndiLinks", (foreignJndiLinks.isEmpty() ? null : foreignJndiLinks))		
		List<WebLogicResource> foreignJndiProviders = buildForeignJNDIResources()
		context.setProperty("foreignJndiProviders", (foreignJndiProviders.isEmpty()) ? null : foreignJndiProviders)
		context.setProperty("loginModules", (loginModules.isEmpty() ? null : loginModules))	
		context.setProperty("loginPolicies", (loginPolicies.isEmpty() ? null : loginPolicies))	
		context.setProperty("applications", (applications.isEmpty() ? null : applications))	
		context.setProperty("users", (users.isEmpty() ? null : users))	
		context.setProperty("policies", (policies.isEmpty() ? null : policies))	
		context.setProperty("scriptingDocs", (scriptingDocs.isEmpty() ? null : scriptingDocs))	
		context.setProperty("globalDoc", globalDoc)	
		context.setProperty("credentialDoc", credentialDoc)	
		context.setProperty("commonDoc", commonDoc)	
		context.setProperty("credentials", credentials)
		
		// Miscellaneous Properties
		context.setProperty("fileSeparator", ((global.getProperty("osType") == "win") ? "\\" : "/"))
		context.setProperty("notSet", "not-set")
		context.setProperty("scriptExtension", ((global.getProperty("osType") == "win") ? "cmd" : "sh"))
		return context
	}
	
	public void setDomain(WebLogicResource resource) { domain = resource }
	public void setGlobal(GlobalResource resource) { global = resource }
	public void addServer(ManagedServer resource) { servers.add(resource) }
	public void addMailSession(MailSession mailSession) { mailSessions.add(mailSession) }
	public void addDatasource(Datasource datasource) { datasources.add(datasource) }
	public void addJdbcProvider(Datasource datasource) { jdbcproviders.add(datasource) }
	public void addJmsServer(WebLogicResource resource) { jmsServers.add(resource) }	
	public void addJmsConnectionFactory(WebLogicResource resource) { jmsConnectionFactories.add(resource) }
	public void addJmsQueue(WebLogicResource resource) { jmsQueues.add(resource) }
	public void addJmsTopic(WebLogicResource resource) { jmsTopics.add(resource) }
	public void addForeignJndiProvider(WebLogicResource resource) { foreignJndiLinks.add(resource) }
	public void addLoginModule(WebLogicResource resource) { loginModules.add(resource) }
	public void addLoginPolicy(WebLogicResource resource) { loginPolicies.add(resource) }
	public void addApplication(ApplicationResource resource) { applications.add(resource) }	
	public void addUser(WebLogicResource resource) { users.add(resource) }	
	public void addPolicy(WebLogicResource resource) { policies.add(resource) }	
	public void setCredentials(Credentials resource) { credentials = resource}
	public void addScriptingDoc(PropertyDocResource resource) { scriptingDocs.add(resource) }	
	public void setGlobalDoc(PropertyDocResource resource) { globalDoc = resource }	
	public void setCredentialDoc(PropertyDocResource resource) { credentialDoc = resource }	
	public void setCommonDoc(PropertyDocResource resource) { commonDoc = resource }
	
	public void loadResources(File resourceDirectory) {
		DirectoryResourceLoader directoryResourceLoader = new DirectoryResourceLoader()
		directoryResourceLoader.setDirectory(resourceDirectory)
		loadResources(directoryResourceLoader)
	}
	
	public void loadResources(DirectoryResourceLoader resourceLoader) {		
		resourceLoader.setFilenamePattern(~/^global.properties/)
		List<Resource> globalResource = resourceLoader.loadResources()
		if(globalResource.size() > 0) {
			setGlobal(new GlobalResource(globalResource[0]))
		}

		resourceLoader.setFilenamePattern(~/^domain\./)
		List<Resource> domainResource = resourceLoader.loadResources()
		if(domainResource.size() > 0) {
			setDomain(new WebLogicResource(domainResource[0]))
		}
				
		resourceLoader.setFilenamePattern(~/^server\./)
		resourceLoader.loadResources().each() { resource ->
			addServer(new ManagedServer(resource))
		}
		
		resourceLoader.setFilenamePattern(~/^mailsession\./)
		resourceLoader.loadResources().each() { resource ->
			addMailSession(new MailSession(resource))
		}		
		
		resourceLoader.setFilenamePattern(~/^datasource\./)
		resourceLoader.loadResources().each() { resource ->
			addDatasource(new Datasource(resource))
		}						
		
		resourceLoader.setFilenamePattern(~/^jdbcprovider\./)
		resourceLoader.loadResources().each() { resource ->
			addJdbcProvider(new Datasource(resource))
		}						
		
		resourceLoader.setFilenamePattern(~/^jmsserver\./)
		resourceLoader.loadResources().each() { resource ->
			addJmsServer(new WebLogicResource(resource))
		}			
		
		resourceLoader.setFilenamePattern(~/^connectionfactory\./)
		resourceLoader.loadResources().each() { resource ->
			addJmsConnectionFactory(new WebLogicResource(resource))
		}			
	
		resourceLoader.setFilenamePattern(~/^queue\./)
		resourceLoader.loadResources().each() { resource ->
			addJmsQueue(new WebLogicResource(resource))
		}	
		
		resourceLoader.setFilenamePattern(~/^topic\./)
		resourceLoader.loadResources().each() { resource ->
			addJmsTopic(new WebLogicResource(resource))
		}				
											 
		resourceLoader.setFilenamePattern(~/^foreignjndiprovider\./)
		resourceLoader.loadResources().each() { resource ->
			addForeignJndiProvider(new WebLogicResource(resource))
		}							
		
		resourceLoader.setFilenamePattern(~/^loginmodule\./)
		resourceLoader.loadResources().each() { resource ->
			addLoginModule(new WebLogicResource(resource))
		}	
		
		resourceLoader.setFilenamePattern(~/^loginpolicy\./)
		resourceLoader.loadResources().each() { resource ->
			addLoginPolicy(new WebLogicResource(resource))
		}		
		
		resourceLoader.setFilenamePattern(~/^application\./)
		resourceLoader.loadResources().each() { resource ->
			addApplication(new ApplicationResource(resource))
		}				
		
		resourceLoader.setFilenamePattern(~/^user\./)
		resourceLoader.loadResources().each() { resource ->
			addUser(new WebLogicResource(resource))
		}			
		
		resourceLoader.setFilenamePattern(~/^policy\./)
		resourceLoader.loadResources().each() { resource ->
			addPolicy(new WebLogicResource(resource))
		}		

		resourceLoader.setFilenamePattern(~/^doc\./)
		resourceLoader.loadResources().each() { resource ->
			addScriptingDoc(new PropertyDocResource(resource))
		}		

		resourceLoader.setFilenamePattern(~/^credentials.properties/)
		List<Resource> credentialsResource = resourceLoader.loadResources()
		if(credentialsResource.size() > 0) {
			setCredentials(new Credentials(credentialsResource[0]))
		}
		
		resourceLoader.setFilenamePattern(~/^doc.global.properties/)
		List<Resource> globalDocResource = resourceLoader.loadResources()
		if(globalDocResource.size() > 0) {
			setGlobalDoc(new PropertyDocResource(globalDocResource[0]))
		}

		resourceLoader.setFilenamePattern(~/^doc.credentials.properties/)
		List<Resource> credentialDocResource = resourceLoader.loadResources()
		if(credentialDocResource.size() > 0) {
			setCredentialDoc(new PropertyDocResource(credentialDocResource[0]))
		}

		resourceLoader.setFilenamePattern(~/^doc.common.properties/)
		List<Resource> commonDocResource = resourceLoader.loadResources()
		if(commonDocResource.size() > 0) {
			setCommonDoc(new PropertyDocResource(commonDocResource[0]))
		}
	}
	
	public List<WebLogicResource> buildJmsModules() {		
		List<WebLogicResource> jmsResources = getJmsResources()
		
		Map<String,WebLogicResource> jmsModuleMap = new HashMap<String,WebLogicResource>()

		jmsResources.each() { jmsResource ->
			String jmsModuleName = jmsResource.getProperty("jmsModuleName")
			WebLogicResource jmsModule = jmsModuleMap.get(jmsModuleName)
			if(jmsModule == null) {
				jmsModule = new WebLogicResource()
				jmsModule.put("jmsModuleName", jmsModuleName)
			}			
			
			Set targetSet = jmsModule.getProperty("targets")
			if(targetSet == null) {
				targetSet = new HashSet()
			}				
					
			String targets = jmsResource.getProperty("targets")
			if(targets != null) {
				targets.split(",").each() { target ->
					targetSet.add(target)	
				}
				jmsModule.put("targets", new ArrayList(targetSet))
			}

			jmsModuleMap.put(jmsModuleName, jmsModule)				
		}
		
		List<WebLogicResource> jmsModules = new ArrayList()		
		jmsModuleMap.keySet().each() { jmsModuleName ->			
			jmsModules.add(jmsModuleMap.get(jmsModuleName))
		}
		
		return jmsModules
	}
	
	public List<WebLogicResource> buildJmsSubdeployments() {
		List<WebLogicResource> jmsResources = getJmsResources()
		List<JmsSubdeployment> jmsSubdeployments = new ArrayList<JmsSubdeployment>()	
		
		jmsResources.each() { jmsResource ->
			String jmsSubdeploymentName = jmsResource.getProperty("subdeploymentName")
			String jmsModuleName = jmsResource.getProperty("jmsModuleName")

			JmsSubdeployment jmsSubdeployment = null
			for(JmsSubdeployment subdeployment in jmsSubdeployments) {
				if(subdeployment.getSubdeploymentName() == jmsSubdeploymentName &&
				   subdeployment.getModuleName() == jmsModuleName) {
					jmsSubdeployment = subdeployment
					break
				}				
			}
			
			// If not found, create subdeployment.
			boolean found = true
			if(jmsSubdeployment == null) {
				found = false
				jmsSubdeployment = new JmsSubdeployment()
				jmsSubdeployment.setSubdeploymentName(jmsSubdeploymentName)
				jmsSubdeployment.setModuleName(jmsModuleName)
			}			
			
			Set<String> targets = jmsSubdeployment.getTargets()
			if(targets == null) { targets = new HashSet<String>() }
			
			String jmsServerName = jmsResource.getProperty("jmsServerName")
			targets.add(jmsServerName)
			jmsSubdeployment.setTargets(targets)

			if(!found) {
				jmsSubdeployments.add(jmsSubdeployment)
			}				
		}
		return jmsSubdeployments		
	}
	
	public List<WebLogicResource> getJmsResources() {
		List<WebLogicResource> jmsResources = new ArrayList()
		jmsConnectionFactories.each() { cf -> jmsResources.add(cf) }		
		jmsQueues.each() { q -> jmsResources.add(q)	}		
		jmsTopics.each() { t ->	jmsResources.add(t) }
		return jmsResources
	}	
	
	public List<WebLogicResource> buildForeignJNDIResources() {
		Map<String,WebLogicResource> foreignJndiProviderMap = new HashMap<String,WebLogicResource>()

		foreignJndiLinks.each() { foreignLink ->
			String foreignLinkName = foreignLink.getProperty("foreignProviderName")
			WebLogicResource foreignJndiProvider = foreignJndiProviderMap.get(foreignLinkName)
			if(foreignJndiProvider == null) {
				foreignJndiProvider = new WebLogicResource()
				foreignJndiProvider.put("foreignProviderName", foreignLinkName)
			}
			
			String foreignProviderURL = foreignLink.getProperty("foreignProviderURL")
			foreignJndiProvider.put("foreignProviderURL", foreignProviderURL)

			String initialContextFactory = foreignLink.getProperty("initialContextFactory")
			foreignJndiProvider.put("initialContextFactory", initialContextFactory)

			foreignJndiProviderMap.put(foreignLinkName, foreignJndiProvider)
		}
		
		List<WebLogicResource> foreignJndiProviders = new ArrayList()
		foreignJndiProviderMap.keySet().each() { fjResourceName ->
			foreignJndiProviders.add(foreignJndiProviderMap.get(fjResourceName))
		}
		return foreignJndiProviders
	}

}
