// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.weblogic
import java.io.File;

import com.sas.config.util.CredentialsDialogPrep;
import com.sas.config.util.CredentialsDialog;

class WebLogicCredentialsPrep extends CredentialsDialogPrep {
	static def tasksRequiringCredentials = [
		"datasource.create":"*",                /* datasourceUsername */
		"domain.createloginmodule":"SASTrust",  /* sasTrusUsername */
		"mailsession.create":"SASMailSession",  /* wlstMailSessionUser */
		"user.create":"*",                      /* newUsername */
		]
	
	static def tasksRequireOtherCredentials = [ : ]
	
	WebLogicCredentialsPrep(String credentialsFile, Boolean debugFlag, String _promptMode, Boolean needAdminCredentials) {
		super(tasksRequiringCredentials, credentialsFile, debugFlag, _promptMode);
		if ( needAdminCredentials) {
			tasksRequiringCredentials.putAt("domain.create", "*")
			tasksRequireOtherCredentials.putAt("nodemanager.stop", "domain.create")
			tasksRequireOtherCredentials.putAt("server.create", "domain.create")
			tasksRequireOtherCredentials.putAt("servers.list", "domain.create")
		}
		else {
			tasksRequiringCredentials.remove("domain.create")
			tasksRequireOtherCredentials.remove("nodemanager.stop")
			tasksRequireOtherCredentials.remove("server.create")
			tasksRequireOtherCredentials.remove("servers.list")
		}
		setHasScope(false)
	}

	public int invokeDialog(def contents) {
		def resultsMap = [:]
		int rc;
		CredentialsDialog dialog = new CredentialsDialog(contents, resultsMap, promptMode)
		if ( dialog == null) 
			return -1
		rc = processDialogResults(resultsMap)
		return rc;
	}
	
	//**********************************************************************************************************************
	// Method:  scanCommandsForCredentials
	// Purpose: Scan the command line or script file for a list of commands that require credentials.
	// Input:   execType - Execution type RUN or FILE
	//          singleTaskLine - if execType is RUN, string containing command line
	//          taskFile - if execType is FILE, the file
	// Output:  List of resources that may need credentials for which prompting will be necessary
	//**********************************************************************************************************************
	public def scanCommandsForCredentials(String execType, String singleTaskLine, File taskFile) {
		def tasksNeedingCredentials = taskRsrcNameTable // To eliminate a Groovy bug(?) Had to use a local method variable and copy from the class variable, otherwise HashMap datatype gets lost
		def listTasksNeedingCredentials = []    // List of resources that may need credentials for which prompting will be necessary
		if (execType == "RUN") {    // Single-line task run mode
			scanCommandForCredential(singleTaskLine, listTasksNeedingCredentials, tasksRequireOtherCredentials)
		} else {    // execType="FILE", task file provided
			taskFile.eachLine { line ->
				if (!line.startsWith("#"))  {       // Process non-comment task lines
					scanCommandForCredential(line, listTasksNeedingCredentials, tasksRequireOtherCredentials)
				}  
			}   // END eachLine
		}
		listTasksNeedingCredentials = listTasksNeedingCredentials.unique()  // Make the list unique
		logger.debug("WebLogicCredentialsPrep-scanCommandsForCredentials: listTasksNeedingCredentials = ${listTasksNeedingCredentials}")
		return listTasksNeedingCredentials;
	}

}
