// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: DataSource
// Purpose: Encapsulate the attributes and methods used to manage and configure a DataSource and related 
//			JDBCProvider 
// Methods:
//		create 		- create a DataSource
//		delete 		- delete a DataSource
// ***************************************************************************************************************

class DataSource extends Resource {
	
	// Class variables available to all methods:
	CommandProcess cmdProc
	def globalPropsFileName
	def resourcePropsFileName
	def	servername
	def nodename
	def scope
	def scope_string
	def dataSourceIdentifier
	def pDsClassName 
	def pDsClassPath
    def pDsConnectionUrl
    def pDsDatabase
    def pDsDataSourceName
    def pDsHelperClass
    def pDsHost
    def pDsJaasAliasName
    def pDsJdbcProviderName
    def pDsJdbcProviderReuse
    def pDsJdbcProviderType
    def pDsJndiName
    def pDsOptions
    def pDsPort
    def pDsStatementCacheSize
    def pDsXADataSource
    def pDsXADataSourceString
    def pCpMinConnections
    def pCpMaxConnections
    def pCpConnectionTimeout
    def pCpAgedTimeout
	def pCpReapTime
    def pCpUnusedTimeout
    def pCpPurgePolicy
    def pCpTestConnection
    def pCpTestConnectionInterval
    def pDsPropPreTestSQLString
    def pDsPropValidateNewConnection
    def pDsPropValidateNewConnectionRetryCount
    def pDsPropValidateNewConnectionRetryInterval
    def pDsPropEnableMultithreadedAccessDetection
	def pDsRemoveJdbcProvider
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command 
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution	

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	public DataSource(GlobalProperties parmGblProps) {
		super(parmGblProps)
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands			
	}	// END CTOR
	
	// *************************************************************************************
	// Method to create a DataSource 
	// General flow:
	//	1. Create a DataSource using the createDataSource.jy script
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName = resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 							= resourceProps.getProperty("servername")
		scope 								= resourceProps.getProperty("scope")
		scope_string						= resourceProps.getProperty("scope_string")
		dataSourceIdentifier				= resourceProps.getProperty("dataSourceIdentifier")
		pDsClassName 						= resourceProps.getProperty("pDsClassName")
		pDsClassPath						= resourceProps.getProperty("pDsClassPath")
		pDsConnectionUrl					= resourceProps.getProperty("pDsConnectionUrl")
		pDsDatabase							= resourceProps.getProperty("pDsDatabase")
		pDsDataSourceName					= resourceProps.getProperty("pDsDataSourceName")
		pDsHelperClass						= resourceProps.getProperty("pDsHelperClass")
		pDsHost								= resourceProps.getProperty("pDsHost")
		pDsJaasAliasName					= resourceProps.getProperty("pDsJaasAliasName")
		pDsJdbcProviderName					= resourceProps.getProperty("pDsJdbcProviderName")
		pDsJdbcProviderReuse				= resourceProps.getProperty("pDsJdbcProviderReuse")
		pDsJdbcProviderType					= resourceProps.getProperty("pDsJdbcProviderType")
		pDsJndiName							= resourceProps.getProperty("pDsJndiName")
		pDsOptions							= resourceProps.getProperty("pDsOptions")
		pDsPort								= resourceProps.getProperty("pDsPort")
		pDsStatementCacheSize				= resourceProps.getProperty("pDsStatementCacheSize")
		pDsXADataSource						= resourceProps.getProperty("pDsXADataSource")
		pDsXADataSourceString				= resourceProps.getProperty("pDsXADataSourceString")
		pCpMinConnections					= resourceProps.getProperty("pCpMinConnections")
		pCpMaxConnections					= resourceProps.getProperty("pCpMaxConnections")
		pCpConnectionTimeout				= resourceProps.getProperty("pCpConnectionTimeout")
		pCpAgedTimeout						= resourceProps.getProperty("pCpAgedTimeout")
		pCpReapTime							= resourceProps.getProperty("pCpReapTime")
		pCpUnusedTimeout					= resourceProps.getProperty("pCpUnusedTimeout")
		pCpPurgePolicy						= resourceProps.getProperty("pCpPurgePolicy")
		pCpTestConnection					= resourceProps.getProperty("pCpTestConnection")
		pCpTestConnectionInterval			= resourceProps.getProperty("pCpTestConnectionInterval")
		pDsPropPreTestSQLString				= resourceProps.getProperty("pDsPropPreTestSQLString")
		pDsPropValidateNewConnection		= resourceProps.getProperty("pDsPropValidateNewConnection")
		pDsPropValidateNewConnectionRetryCount		= resourceProps.getProperty("pDsPropValidateNewConnectionRetryCount")
		pDsPropValidateNewConnectionRetryInterval	= resourceProps.getProperty("pDsPropValidateNewConnectionRetryInterval")	
		pDsPropEnableMultithreadedAccessDetection	= resourceProps.getProperty("pDsPropEnableMultithreadedAccessDetection")
		config_target_name					= resourceProps.getProperty("config.target.name")

		if(DEBUG) {
			logger.debug("DataSource-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("DataSource-create: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("DataSource-create: servername=${servername}")
			logger.debug("DataSource-create: appsrvnodename=${appsrvnodename}")			
			logger.debug("DataSource-create: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("DataSource-create: websphere_install_dir=${websphere_install_dir}")
			logger.debug("DataSource-create: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("DataSource-create: dmgrprotocol=${dmgrprotocol}")
			logger.debug("DataSource-create: dmgrport=${dmgrport}")
			logger.debug("DataSource-create: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("DataSource-create: wsadminOptionsList=${wsadminOptionsList}")			
			logger.debug("DataSource-create: scope=${scope}")
			logger.debug("DataSource-create: config_target_name=${config_target_name}")
			logger.debug("DataSource-create: dataSourceIdentifier=${dataSourceIdentifier}")
			logger.debug("DataSource-create: pDsClassName=${pDsClassName}")
			logger.debug("DataSource-create: pDsClassPath=${pDsClassPath}")
			logger.debug("DataSource-create: pDsConnectionUrl=${pDsConnectionUrl}")
			logger.debug("DataSource-create: pDsDatabase=${pDsDatabase}")
			logger.debug("DataSource-create: pDsDataSourceName=${pDsDataSourceName}")
			logger.debug("DataSource-create: pDsHelperClass=${pDsHelperClass}")
			logger.debug("DataSource-create: pDsHost=${pDsHost}")
			logger.debug("DataSource-create: pDsJaasAliasName=${pDsJaasAliasName}")
			logger.debug("DataSource-create: pDsJdbcProviderName=${pDsJdbcProviderName}")
			logger.debug("DataSource-create: pDsJdbcProviderReuse=${pDsJdbcProviderReuse}")
			logger.debug("DataSource-create: pDsJdbcProviderType=${pDsJdbcProviderType}")
			logger.debug("DataSource-create: pDsJndiName=${pDsJndiName}")
			logger.debug("DataSource-create: pDsOptions=${pDsOptions}")
			logger.debug("DataSource-create: pDsPort=${pDsPort}")
			logger.debug("DataSource-create: pDsStatementCacheSize=${pDsStatementCacheSize}")
			logger.debug("DataSource-create: pDsXADataSource=${pDsXADataSource}")
			logger.debug("DataSource-create: pDsXADataSourceString=${pDsXADataSourceString}")
			logger.debug("DataSource-create: pCpMinConnections=${pCpMinConnections}")
			logger.debug("DataSource-create: pCpMaxConnections=${pCpMaxConnections}")
			logger.debug("DataSource-create: pCpConnectionTimeout=${pCpConnectionTimeout}")
			logger.debug("DataSource-create: pCpAgedTimeout=${pCpAgedTimeout}")
			logger.debug("DataSource-create: pCpReapTime=${pCpReapTime}")
			logger.debug("DataSource-create: pCpUnusedTimeout=${pCpUnusedTimeout}")
			logger.debug("DataSource-create: pCpPurgePolicy=${pCpPurgePolicy}")
			logger.debug("DataSource-create: pCpTestConnection=${pCpTestConnection}")
			logger.debug("DataSource-create: pCpTestConnectionInterval=${pCpTestConnectionInterval}")
			logger.debug("DataSource-create: pDsPropPreTestSQLString=${pDsPropPreTestSQLString}")
			logger.debug("DataSource-create: pDsPropValidateNewConnection=${pDsPropValidateNewConnection}")
			logger.debug("DataSource-create: pDsPropValidateNewConnectionRetryCount=${pDsPropValidateNewConnectionRetryCount}")
			logger.debug("DataSource-create: pDsPropValidateNewConnectionRetryInterval=${pDsPropValidateNewConnectionRetryInterval}")
			logger.debug("DataSource-create: pDsPropEnableMultithreadedAccessDetection=${pDsPropEnableMultithreadedAccessDetection}")
		}
		
		// Determine existence of userId and password for the data source
		ResourceProperties credentialsProps = new ResourceProperties(websphere_scripting_credentials)	// Object used to read the websphere_scripting_credentials properties file
		String dataSourceUserid = credentialsProps.getProperty("DataSource.create_${scope}_${dataSourceIdentifier}_userid")
		String dataSourcePasswd = credentialsProps.getProperty("DataSource.create_${scope}_${dataSourceIdentifier}_passwd")
		// If userId and/or password are null in the credentials.properties file, can't continue because data source needs credentials
		def dataSourceErrorType = ""
		if (dataSourceUserid == "")
			dataSourceErrorType = "userId"
		if (dataSourcePasswd == "")
			dataSourceErrorType += " password"
		if (dataSourceErrorType) {
			method_MSG = "**SEVERE** DataSource-create: Data Source \"${pDsDataSourceName}\" requires userId and password, but ${dataSourceErrorType} not provided; unable to continue."
			method_RC = -99
			logger.error("${method_MSG}")
			return method_RC
		}
		
		// 1. Create a DataSource 
		logger.info("DataSource-create: Create DataSource \"${pDsDataSourceName}\" using the wsadmin${script_ext} command at scope \"${scope_string}\"...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "createDataSource",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "DataSourceCreate_${pDsDataSourceName}_output", "DataSourceCreate_${pDsDataSourceName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("DataSource-create: Command Return code: " + command_retcode)
		if (command_retcode == 0) {			// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the createDataSource.jy Jython proc completed successfully
			if (  command_output.contains("createDataSource: \"${pDsDataSourceName}\" has been successfully created.") ) { 
				method_MSG =  "DataSource-create: ===> createDataSource.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** DataSource-create: createDataSource.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** DataSource-create: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		logger.info("${method_MSG}")
		return method_RC	// Return to driver indicating failure
	}	// END create method		

	// *************************************************************************************
	// Method to delete a DataSource 
	// General flow:
	//	1. Delete a DataSource using the deleteDataSource.jy script
	public int delete(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 				= resourceProps.getProperty("servername")
		scope 					= resourceProps.getProperty("scope")
		pDsDataSourceName 		= resourceProps.getProperty("pDsDataSourceName")
		pDsJdbcProviderName		= resourceProps.getProperty("pDsJdbcProviderName")
		pDsRemoveJdbcProvider	= resourceProps.getProperty("pDsRemoveJdbcProvider")
		config_target_name		= resourceProps.getProperty("config.target.name")

		if(DEBUG) {
			logger.debug("DataSource-delete: globalPropsFileName=${globalPropsFileName}")		
			logger.debug("DataSource-delete: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("DataSource-delete: servername=${servername}")
			logger.debug("DataSource-create: appsrvnodename=${appsrvnodename}")
			logger.debug("DataSource-delete: pDsDataSourceName=${pDsDataSourceName}")
			logger.debug("DataSource-delete: pDsJdbcProviderName=${pDsJdbcProviderName}")
			logger.debug("DataSource-delete: pDsRemoveJdbcProvider=${pDsRemoveJdbcProvider}")
			logger.debug("DataSource-delete: scope=${scope}")
			logger.debug("DataSource-delete: config_target_name=${config_target_name}")
			logger.debug("DataSource-delete: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("DataSource-delete: websphere_install_dir=${websphere_install_dir}")
			logger.debug("DataSource-delete: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("DataSource-delete: dmgrprotocol=${dmgrprotocol}")
			logger.debug("DataSource-delete: dmgrport=${dmgrport}")
			logger.debug("DataSource-delete: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("DataSource-delete: wsadminOptionsList=${wsadminOptionsList}")			
		}
		
		// 1. Delete a DataSource 
		logger.info("DataSource-delete: Delete DataSource \"${pDsDataSourceName}\" using the wsadmin${script_ext} command at scope \"${scope_string}\"...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "deleteDataSource",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "DataSourceDelete_${pDsDataSourceName}_output", "DataSourceDelete_${pDsDataSourceName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("DataSource-delete: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the deleteDataSource.jy Jython proc completed successfully
			if (  command_output.contains("removeDataSource: \"${pDsDataSourceName}\" was successfully removed.")  ||
			 	  command_output.contains("removeDataSource: \"${pDsDataSourceName}\" not found;")  ) { 
				method_MSG =  "DataSource-delete: ===> deleteDataSource.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** DataSource-delete: deleteDataSource.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** DataSource-delete: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		logger.info("${method_MSG}")
		return method_RC	// Return to driver indicating failure
	}	// END delete method

}	// END DataSource class
