// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: JDBCProvider
// Purpose: Encapsulate the attributes and methods used to manage and configure a JDBCProvider
// Methods:
//		create 		- create a JDBCProvider
//		delete 		- delete a JDBCProvider
// ***************************************************************************************************************

class JDBCProvider extends Resource {
	
	// Class variables available to all methods:
	CommandProcess cmdProc
	def globalPropsFileName
	def resourcePropsFileName
	def	servername
	def scope
	def jdbcProviderIdentifier
	def pDsClassName 
	def pDsClassPath
    def pDsJdbcProviderName
    def pDsJdbcProviderReuse
    def pDsJdbcProviderType
    def pDsXADataSource
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command 
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution	

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	public JDBCProvider(GlobalProperties parmGblProps) {
		super(parmGblProps)
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands			
	}	// END CTOR
	
	// *************************************************************************************
	// Method to create a JDBCProvider 
	// General flow:
	//	1. Create a JDBCProvider using the createJDBCProvider.jy script
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName = resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 							= resourceProps.getProperty("servername")
		scope 								= resourceProps.getProperty("scope")
		jdbcProviderIdentifier				= resourceProps.getProperty("jdbcProviderIdentifier")
		pDsClassName 						= resourceProps.getProperty("pDsClassName")
		pDsClassPath						= resourceProps.getProperty("pDsClassPath")
		pDsJdbcProviderName					= resourceProps.getProperty("pDsJdbcProviderName")
		pDsJdbcProviderReuse				= resourceProps.getProperty("pDsJdbcProviderReuse")
		pDsJdbcProviderType					= resourceProps.getProperty("pDsJdbcProviderType")
		pDsXADataSource						= resourceProps.getProperty("pDsXADataSource")
		config_target_name					= resourceProps.getProperty("config.target.name")

		if(DEBUG) {
			logger.debug("JDBCProvider-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("JDBCProvider-create: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("JDBCProvider-create: servername=${servername}")
			logger.debug("JDBCProvider-create: appsrvnodename=${appsrvnodename}")			
			logger.debug("JDBCProvider-create: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("JDBCProvider-create: websphere_install_dir=${websphere_install_dir}")
			logger.debug("JDBCProvider-create: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("JDBCProvider-create: dmgrprotocol=${dmgrprotocol}")
			logger.debug("JDBCProvider-create: dmgrport=${dmgrport}")
			logger.debug("JDBCProvider-create: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("JDBCProvider-create: wsadminOptionsList=${wsadminOptionsList}")			
			logger.debug("JDBCProvider-create: scope=${scope}")
			logger.debug("JDBCProvider-create: config_target_name=${config_target_name}")
			logger.debug("JDBCProvider-create: jdbcProviderIdentifier=${jdbcProviderIdentifier}")
			logger.debug("JDBCProvider-create: pDsClassName=${pDsClassName}")
			logger.debug("JDBCProvider-create: pDsClassPath=${pDsClassPath}")
			logger.debug("JDBCProvider-create: pDsJdbcProviderName=${pDsJdbcProviderName}")
			logger.debug("JDBCProvider-create: pDsJdbcProviderReuse=${pDsJdbcProviderReuse}")
			logger.debug("JDBCProvider-create: pDsJdbcProviderType=${pDsJdbcProviderType}")
			logger.debug("JDBCProvider-create: pDsXADataSource=${pDsXADataSource}")
		}
		
		// 1. Create a JDBCProvider
		logger.info("JDBCProvider-create: Create JDBCProvider \"${pDsJdbcProviderName}\" using the wsadmin${script_ext} command at the \"${scope}\" level...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "createJDBCProvider",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "JDBCProviderCreate_${pDsJdbcProviderName}_output", "JDBCProviderCreate_${pDsJdbcProviderName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("JDBCProvider-create: Command Return code: " + command_retcode)
		if (command_retcode == 0) {			// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the createJDBCProvider.jy Jython proc completed successfully
			if (  command_output.contains("createJDBCProvider: \"${pDsJdbcProviderName}\" has been successfully created.") ||
				  command_output.contains("createJDBCProvider: JDBCProvider with a name of \"${pDsJdbcProviderName}\" already exists at scope") ) { 
				method_MSG =  "JDBCProvider-create: ===> createJDBCProvider.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** JDBCProvider-create: createJDBCProvider.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** JDBCProvider-create: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		logger.info("${method_MSG}")
		return method_RC	// Return to driver indicating failure
	}	// END create method		

	// *************************************************************************************
	// Method to delete a JDBCProvider 
	// General flow:
	//	1. Delete a JDBCProvider definition using the deleteJDBCProvider.jy script
	public int delete(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 				= resourceProps.getProperty("servername")
		scope 					= resourceProps.getProperty("scope")
		pDsJdbcProviderName		= resourceProps.getProperty("pDsJdbcProviderName")
		config_target_name		= resourceProps.getProperty("config.target.name")

		if(DEBUG) {
			logger.debug("JDBCProvider-delete: globalPropsFileName=${globalPropsFileName}")		
			logger.debug("JDBCProvider-delete: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("JDBCProvider-delete: servername=${servername}")
			logger.debug("JDBCProvider-create: appsrvnodename=${appsrvnodename}")
			logger.debug("JDBCProvider-delete: pDsJdbcProviderName=${pDsJdbcProviderName}")
			logger.debug("JDBCProvider-delete: scope=${scope}")
			logger.debug("JDBCProvider-delete: config_target_name=${config_target_name}")
			logger.debug("JDBCProvider-delete: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("JDBCProvider-delete: websphere_install_dir=${websphere_install_dir}")
			logger.debug("JDBCProvider-delete: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("JDBCProvider-delete: dmgrprotocol=${dmgrprotocol}")
			logger.debug("JDBCProvider-delete: dmgrport=${dmgrport}")
			logger.debug("JDBCProvider-delete: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("JDBCProvider-delete: wsadminOptionsList=${wsadminOptionsList}")			
		}
		
		// 1. Delete a JDBCProvider 
		logger.info("JDBCProvider-delete: Delete JDBCProvider \"${pDsJdbcProviderName}\" using the wsadmin${script_ext} command at the \"${scope}\" level...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "deleteJDBCProvider",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "JDBCProviderDelete_${pDsJdbcProviderName}_output", "JDBCProviderDelete_${pDsJdbcProviderName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("JDBCProvider-delete: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the deleteJDBCProvider.jy Jython proc completed successfully
			if (  command_output.contains("deleteJDBCProvider: \"${pDsJdbcProviderName}\" was successfully removed.")  ||
			 	  command_output.contains("deleteJDBCProvider: \"${pDsJdbcProviderName}\" not found;")  ) { 
				method_MSG =  "JDBCProvider-delete: ===> deleteJDBCProvider.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** JDBCProvider-delete: deleteJDBCProvider.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** JDBCProvider-delete: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		logger.info("${method_MSG}")
		return method_RC	// Return to driver indicating failure
	}	// END delete method

}	// END JDBCProvider class
