// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// *********************************************************************************************************************
// Class: Node
// Purpose: Encapsulate the attributes and methods used to perform a full synchronization between dmgr and the nodeagent
// Methods:
//		syncNode	- perform a full synchronization
// *********************************************************************************************************************

class Node extends Resource {
	// Class variables:
	CommandProcess cmdProc
	def globalPropsFileName
	def resourcePropsFileName
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command  
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution							 
	
	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties
	// Global properties are obtained from the base Resource class 
	public Node(GlobalProperties parmGblProps) {
		super(parmGblProps)
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands	
	}	// END CTOR

	// *************************************************************************************
	// Method to perform synchronization of dmgr and nodeagent repositories 
	// General flow:
	//	1. Run the syncNode.jy script
	public int syncNode(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Method fields:
		resourcePropsFileName = resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		config_target_name	  = resourceProps.getProperty("config.target.name")
		if (DEBUG) {
			logger.debug("Node-syncNode: globalPropsFileName=${globalPropsFileName}")
			logger.debug("Node-syncNode: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("Node-syncNode: profilename=${profilename}")
			logger.debug("Node-syncNode: appsrvnodename=${appsrvnodename}")
			logger.debug("Node-syncNode: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("Node-syncNode: config_target_name=${config_target_name}")
			logger.debug("Node-syncNode: script_ext=${script_ext}")
			logger.debug("Node-syncNode: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("Node-syncNode: websphere_install_dir=${websphere_install_dir}")
			logger.debug("Node-syncNode: websphere_profile_dir=${websphere_profile_dir}")
			logger.debug("Node-syncNode: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("Node-syncNode: dmgrprotocol=${dmgrprotocol}")
			logger.debug("Node-syncNode: dmgrport=${dmgrport}")
			logger.debug("Node-syncNode: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("Node-syncNode: wsadminOptionsList=${wsadminOptionsList}")			
		} 
		
		// 1. Run the syncNode.jy script 
		logger.info("Node-syncNode: Synchronize the ${dmgr} and ${nodeagent} repositories using the wsadmin${script_ext} command...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "syncNode",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "NodeSyncNode_${appsrvnodename}_output", "NodeSyncNode_${appsrvnodename}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("Node-syncNode: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the syncNode.jy Jython proc completed successfully
			if (  command_output.contains("syncNode: Synchronization between dmgr and nodeagent for node complete.") ) { 
				method_MSG =  "Node-syncNode: ===> syncNode.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** Node-syncNode: syncNode.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** Node-syncNode: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END syncNode method	

}	// END Node class