// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere.docparts
;

import java.io.File;
import java.util.List;
import java.util.Map.Entry
import java.util.HashMap;
import java.util.regex.Pattern;

import com.sas.config.Context
import com.sas.config.ContextBuilder
import com.sas.config.ResourceLoader
import com.sas.config.DirectoryResourceLoader
import com.sas.config.PropertyContextBuilder
import com.sas.config.Resource
import com.sas.config.websphere.docparts.WebSphereResource

class CompareServerNames implements Comparator<Resource> {
    CompareServerNames() {}
    int compare(Object o1, Object o2) {
        Resource r1 = (Resource) o1
        Resource r2 = (Resource) o2
        String s1 = r1["servername"]
        String s2 = r2["servername"]
        if (s1 == null) 
            s1 = ""
        if (s2 == null)
            s2 = ""
        return s1.compareTo(s2)
    }
}

class WebSphereContextBuilder extends PropertyContextBuilder {

    private String target
    private List<String> scopes = [ "CELL", "NODE", "SERVER" ]
    private Context context = new Context()
	private WebSphereResource websphere_global = null
	
    public WebSphereContextBuilder(String target) {
        this.target = target
    }
    
    @Override
    protected Context buildInternal(File templateFile, Context context) {   
        super.buildInternal(templateFile, context)
        // Provide the list of scopes       
        context.setProperty("scopes", scopes)
		
		for(Entry e : this.context.entrySet()) {
			context.put(e.getKey(), e.getValue())
			println "**** Added " + e.getKey() + " to context."
		}
		
        // Provide the helper objects
        context.setProperty("ResourceFinder", new ResourceFinder(context))
        context.setProperty("CustomProperties", new CustomProperties())
        context.setProperty("SystemProperties", new SystemProperties())
		
		// Miscellaneous "global" Properties
		context.setProperty("fileSeparator", ((websphere_global.getProperty("os_localhost_type") == "win") ? "\\" : "/"))
		context.setProperty("notSet", "not-set")
		context.setProperty("scriptExtension", ((websphere_global.getProperty("os_localhost_type") == "win") ? "bat" : "sh"))
		
        return context
    }

    public void loadResources(File resourceDirectory) {
        DirectoryResourceLoader directoryResourceLoader = new DirectoryResourceLoader()
        directoryResourceLoader.setDirectory(resourceDirectory)
        loadResourcesInternal(directoryResourceLoader)
    }
    
	
	public void loadResources(DirectoryResourceLoader resourceLoader) {
		websphere_global = loadResourceInternal("websphere.global", resourceLoader)
		loadResourceInternal("DmgrProfile", resourceLoader)
		loadResourceInternal("NodeProfile", resourceLoader)
		loadResourcesInternal("LoginModule", resourceLoader)
		loadResourcesInternal("Server", resourceLoader)
		loadResourcesInternal("MailSession", resourceLoader)
		loadResourcesInternal("SIBus", resourceLoader)
		loadResourcesInternal("JMSConnectionFactory", resourceLoader)
		loadResourcesInternal("JMSQueue", resourceLoader)
		loadResourcesInternal("JMSTopic", resourceLoader)
		loadResourcesInternal("ActivationSpec", resourceLoader)
		loadResourcesInternal("JDBCProvider", resourceLoader)
		loadResourcesInternal("DataSource", resourceLoader)	
		loadResourcesInternal("ObjectCache", resourceLoader)
		loadResourcesInternal("Application", resourceLoader)
	}
	
	private void loadResourcesInternal(String resourceType, DirectoryResourceLoader resourceLoader) {
		resourceLoader.setFilenamePattern(~/^${resourceType}/)
		List<Resource> resources = resourceLoader.loadResources()
		Collections.sort(resources, new CompareServerNames())
		List<WebSphereResource> wsResources = new ArrayList<WebSphereResource>()
		for(Resource r : resources) { 
			switch(resourceType) {	
				// Allow for future subclasses of WebsphereResource to be loaded.			
				default:
					wsResources.add(new WebSphereResource(r))
					break
			}
		}			
		context.setProperty(replacePeriodsWithUnderscore(resourceType), wsResources)
	}
	
	private WebSphereResource loadResourceInternal(String resourceType, DirectoryResourceLoader resourceLoader) {
		resourceLoader.setFilenamePattern(~/^${resourceType}/)
		List<Resource> resources = resourceLoader.loadResources()
		WebSphereResource newWASResource = null
		if(resources.size() > 0) {
			switch(resourceType) {
				// Allow for future subclasses of WebsphereResource to be loaded.
				default:
					newWASResource = new WebSphereResource(resources[0])
					context.setProperty(replacePeriodsWithUnderscore(resourceType), newWASResource)	
					break
			}
		}
		return newWASResource
	}
	
	private String replacePeriodsWithUnderscore(String propertyName) {
		if(propertyName != null) {
			propertyName = propertyName.replaceAll('\\.', '_')
		}
		return propertyName
	}
}
