# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import os
import string
from java.net import InetAddress

def handleError(shutdownAdminServer, e, rcode):
  logger.error('Failed to create the domain.')
  if e != '':
     logger.error(e)
  dumpStack()

  writeRC(rcode)
  
  if shutdownAdminServer:
     try:
         nmKill('AdminServer')
     except Exception, e:
         logger.error('Failed to kill the AdminServer')
  try:
     stopNodeManager()
  except Exception, e:
     logger.error('Failed to stopNodeManager')
  import time  
  return rcode

#**************************************************************************************************
# Name: SAScreateDomain
# Purpose: Create a new WebLogic Server domain from a template or reconfigure an existing domain.
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials used in this module:
#@@ credentials resourceDescription: All properties that are related to credentials are stored in the <tt>credentials.properties</tt> file.  The tool will prompt for these properties. This property file does not need to be edited directly.  These values are cleared from the file after the tool completes if the global property <tt>webappsrvScriptingCacheCredentials</tt> is set to <tt>false</tt>.  When stored, they are stored in SAS base-64 encoding, not clear-text.
#@@ credentials          domain.create_WeblogicAdmin_userid:  The WebLogic Server administrator user.
#@@ credentials          domain.create_WeblogicAdmin_passwd:  The WebLogic Server administrator password.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#                        For the domain, this file is not used. All domain properties are 
#                        found in global.properties and loaded before invoking this function.
#
# Properties found in domain.domainname.properties:
#@@ domain resourceDescription:  Most domain configuration properties are defined in global.properties.  The file <tt>domain.<em>domainName</em>.properties</tt> contains properties needed exclusively to create a new domain and not required by operations on other resources.
#@@ domain domainTemplatePath: The full path to the template JAR file that is used to create the domain.
#
# Properties common to all property types:
#@@ common resourceDescription:  The following properties are common to a number of resource types.
#@@ common deleted: A boolean value. If set to <tt>true</tt>, then this resource has been marked as deleted.
#@@ common deletedTargets: A comma separated list of target servers that contain this resource that are marked for deletion.  A delete operation will remove these targets and remove the resource if no targets remain.
#@@ common targets: A comma separated list of servers that this resource instance is targeted to.
#@@ common thisOperation: This field is used internally by SAS Deployment Wizard and SAS Deployment Manager to manage resource files.  It is not used by the SAS Configuration Scripting Tools.
#@@ common thisTarget: This field is used internally by SAS Deployment Wizard and SAS Deployment Manager to manage resource files.  It is not used by the SAS Configuration Scripting Tools.
#
# Global properties:
#@@ global adminHost: The host name of the administration server for this domain.
#@@ global adminJvmOptions: JVM options for the administration server.
#@@ global adminPort: The port number that the administration server listens on.
#@@ global adminServerName: The name of the administration server.  This property is not currently honored.  The name is always <tt>AdminServer</tt>.
#@@ global adminServiceName: The Windows service name for the administration server.  This is used to generate the scripts that install Windows services. 
#@@ global adminSSLPort: The administration server SSL port.  This value can be set in the properties file, but the scripts do not support the configuration of servers with SSL.
#@@ global adminUrl: The URL used by <tt>wlst</tt> to connect to the administration server.
#@@ global applicationStagingDir: This property is not used by the SAS Configuration Scripting Tools. This property is used to generate documentation only.
#@@ global autoConfigure: A boolean value. If set to <tt>false</tt>, then manual configuration is requested and the SAS Deployment Wizard creates a sample domain and configures servers in off-line mode only. All configuration steps run outside of SAS Deployment Wizard and SAS Deployment Manager are automated regardless of this setting.
#@@ global autoDeploy: A boolean value.  If set to <tt>false</tt>, then the SAS Deployment Wizard does not deploy the SAS Web applications.  This property is not used by the SAS Configuration Scripting Tools. This property is used by SAS Deployment Wizard to generate documentation.
#@@ global backupBinDir:  The fully qualified path to a directory that is used to save previous versions of the scripts in the <tt><em>Domain</em>/bin</tt> directory.
#@@ global bitwidthOption: If the <tt>-d64</tt> JVM option is required on this host, then this property is set to contain the JVM option.  This property is used in the commEnvSAS.sh script.
#@@ global configureJMSSecurity: This property is used by SAS Deployment Wizard to generate documentation only. 
#@@ global domainChanges: This property is used to generate a report of configuration changes in the instructions that are generated by SAS Deployment Wizard or SAS Deployment Manager.  It is not used by the SAS Configuration Scripting Tools.  
#@@ global domainDir: The fully qualified path of the domain to be configured.
#@@ global host: The host name for this machine.
#@@ global isAdminHost: A boolean value.  If the administration server is installed on this machine, then the value is <tt>true</tt>.
#@@ global javaHome: The <tt>JAVA_HOME</tt> environment variable that is inherited by the servers and tools that are launched by the SAS Configuration Scripting Tools.
#@@ global javaVendor: The Java runtime vendor.  Valid values include <tt>Sun</tt>, <tt>Oracle</tt>, <tt>IBM</tt>, <tt>BEA</tt>, and <tt>HP</tt>.
#@@ global jdkBugWorkaround: This property defines a JVM option that is required by most recent JDKs.  Do not modify this property.
#@@ global loginModuleFile: The path of the JAAS loginModuleFile that is set as a JVM option for WebLogic managed servers. 
#@@ global midtierMulticastIpJavaOptions: The JVM options that configure multicast communications with the SAS Remote Services application.  These values must match the values used by the SAS Remote Services application. 
#@@ global minimumUlimit: On UNIX systems, this is the minimum acceptable unlimit setting for file descriptors.  The create domain operation fails if a lower setting is detected. 
#@@ global mwHomeDir: The Oracle middleware home directory (<tt>MW_HOME</tt>).  This is typically the parent directory of the WebLogic Server installation directory, but the WebLogic Server installers permit installing WebLogic Server in a different location.
#@@ global nodeManagerPort: The port number that the node manager for this domain is listening on.
#@@ global osLinuxVendor: This property contains a valid Linux vendor name - either <tt>redhat</tt> or <tt>SuSE</tt>.  On other operating systems it is set to <tt>UNKNOWN</tt>.  This value is used in the commEnvSAS.sh script.
#@@ global osLinuxVersion: This property contains a Linux operating system version number.  On other operating systems it is set to <tt>UNKNOWN</tt>.
#@@ global osType: The operating system type. Valid values are <tt>win</tt> or <tt>unx</tt>.
#@@ global pythonCachedirOption: This option is passed to the <tt>wlst</tt> command line.  It controls the location of the Python cache.
#@@ global rcFileName: The return code file used by <tt>wlst</tt> commands to return values in properties and its return code status.
#@@ global returnCodeProperties: The location of the return code properties file that maps return code keys to numeric values.  This property should not be changed.
#@@ global SASDomainName: The name of the WebLogic Server domain that is used for the SAS Web applications.  This property is not used by the SAS Configuration Scripting Tools.  This property is used by SAS Deployment Wizard to generate documentation only.
#@@ global SASWlstScriptHome: The fully qualified path to the SAS Configuration Scripting Tools directory.
#@@ global startScript: The location of the domain setup script. This is setDomainEnv.cmd or setDomainEnv.sh.
#@@ global templateDirectoryName: The fully qualified path to the directory that contains the velocity templates. The templates are used to create scripts in the <tt><em>Domain</em>/bin</tt> directory.
#@@ global webappsrvScriptingCacheCredentials: A boolean value.  If set to <tt>true</tt>, then credential values are cached in the <tt>credentials.properties</tt> file.  If set to <tt>false</tt>, then values are purged from <tt>credentials.properties</tt> when the SAS Configuration Scripting Tools exit.
#@@ global webauthIsComment: Several lines are included in commEnvSAS.cmd or commEnvSAS.sh for the support of Web authentication.  These are commented out unless you have manually modified the SAS deployment to support Web authentication.  This property contains the characters that set these lines as comments, or if Web authentication is configured, this property contains an empty value.
#@@ global weblogicBarName: This name is used to create the name of the Windows services in installNodeMgrSvc.cmd and uninstallNodeMgrSvc.cmd.
#@@ global weblogicHome: The fully qualified path to the WebLogic Server installation.  The <tt>WL_HOME</tt> environment variable is set to this value in generated scripts.
#@@ global weblogicInstalled: A boolean value. <tt>True</tt> indicates that WebLogic Server is installed on the machine that is used for the SAS middle tier. This value is used to generate documentation and is not used by the SAS Configuration Scripting Tools.
#@@ global weblogicNodeManagerServiceName:  The Windows service name for the nodemanager.
#@@ global weblogicProdName: The WebLogic product name that is used as part of the Windows service name for the nodemanager.
#@@ global weblogicSelectedVersion: Used by SAS Deployment Wizard to generate documentation only.  It is not used by the SAS Configuration Scripting Tools.
#@@ global weblogicReconfigured:  A boolean value. This property is set to <tt>true</tt> if the last SAS Deployment Wizard run was a reconfiguration.  This property is used to generate documentation only and is not used by the SAS Configuration Scripting Tools.
#                           
# Output:
#   Generates a new domain or updates the existing domain.
#   Creates a return code file containing properties for status and merged command line options
#**************************************************************************************************

def SAScreateDomain(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    import time

    logger.debug("javaVendor=" + javaVendor)
    logger.debug('adminJvmOptions:' + adminJvmOptions)
    logger.debug('adminHost:' + adminHost)
    logger.debug('domainTemplatePath:' + domainTemplatePath)
    logger.debug('domainDir:' + domainDir)
    logger.debug('adminPort:' + adminPort)
    logger.debug('adminSSLPort:' + adminSSLPort)
    logger.debug('host:' + host)
    logger.debug('nodeManagerPort:' + nodeManagerPort)
    logger.debug('rcFileName:' + rcFileName)
    logger.debug('osType:' + osType)
    logger.debug('javaHome:' + javaHome)
    logger.debug('weblogicHome:' + weblogicHome)
    logger.debug('startScript:' + startScript)
    logger.debug('autoConfigure:' + autoConfigure)
    logger.debug('osMachineType:' + osMachineType)

    adminUser=''
    adminPasswd=''
    rc = 0

    # startTime= time.time()

    changes = ''
    try:

        adminUser = getCredentialUser(credentialsFile, "domain.create_WeblogicAdmin")
        wlstAdminPassword = getCredentialPassword(credentialsFile, "domain.create_WeblogicAdmin")
        try:
            from com.sas.util import SasPasswordString
            adminPasswd = SasPasswordString.decode(wlstAdminPassword)
        except ImportError, e:
            logger.error("Can't find sas.core.jar")
            writeRC(RC_NO_SAS_CORE_JAR)
            return RC_NO_SAS_CORE_JAR

        # Check for space in Domain Directory
        # WebLogic accepts names in directories above the Domain Directory in the path on 
        # Windows, but not in the Domain Directory name itself.
        
        domainDirFile = File(domainDir).getName()
        if domainDirFile.find(' ') > 0:
            logger.error("Domain name must not contain spaces:  '" + domainDirFile + '"')
            writeRC(RC_BAD_DOMAIN_NAME)
            return RC_BAD_DOMAIN_NAME

        linuxURandomFlag = checkForURandomUseOnLinux()
        
        # logger.debug("sun.arch.data.model=" + System.getProperty("sun.arch.data.model"))
        if osMachineType != 'win' and is64BitArchitectureSupported() == false:
            logger.error("A 32-bit JDK has been selected.  A 64-bit JDK is required.")
            writeRC(RC_BAD_JVM_ARCHITECTURE)
            return RC_BAD_JVM_ARCHITECTURE
        if isCorrectNativeLibraryInstalled() == false:
            logger.error("The required 64-bit WebLogic native libraries have not been installed with the selected WebLogic Server")
            writeRC(RC_NO_WLS_NATIVE_LIBRARIES)
            return RC_NO_WLS_NATIVE_LIBRARIES

        HostListenAddress = getHostListenAddress(weblogicHome, javaHome)
        logger.debug('HostListenAddress:' + HostListenAddress)
        localHostName = InetAddress.getLocalHost().getHostName()
        logger.debug('localHostName:' + localHostName)
        try:
            if InetAddress.getByName(HostListenAddress) != InetAddress.getByName(localHostName):
                logger.error('The DemoIdentity.jks keystore contains a certificate with the wrong host name.  Please update the certificate and retry this step.')
                writeRC(RC_BAD_HOSTNAME_IN_CERT)
                return RC_BAD_HOSTNAME_IN_CERT
        except java.net.UnknownHostException, e:
                logger.error('The DemoIdentity.jks keystore contains a certificate with an unknown host name.  Please update the certificate and retry this step.')
                writeRC(RC_BAD_HOSTNAME_IN_CERT)
                return RC_BAD_HOSTNAME_IN_CERT

        # compare the addresses returned for HostListenAddress and host - the names don't have to match
        if InetAddress.getByName(HostListenAddress) != InetAddress.getByName(host):
            logger.error('The global.properties file contains host=' + host + ' which does not match the actual host name ' + localHostName +'.  If this configuration has been moved to another machine, please update this file.  If this is an automated SDW configuration, restart SDW and specify a valid name for this host in the Local Machine Name panel.')
            writeRC(RC_BAD_HOSTNAME_IN_GLOBAL_PROP)
            return RC_BAD_HOSTNAME_IN_GLOBAL_PROP
            
        if os.path.exists(domainDir):
            try:
                logger.info('The Domain exists.  Opening Domain')
                isNewDomain = false
                readDomain(domainDir)  # additional structures are setup
                cd ('/Server/AdminServer')
                originalPort = get ('ListenPort')
            except Exception, e:
                logger.error("Can't open domain " + domainDir)
                writeRC(RC_CANT_READ_DOMAIN)
                return RC_CANT_READ_DOMAIN
        else:
            try:
                logger.debug('Reading the template: ' + domainTemplatePath)
                isNewDomain = true
                readTemplate(domainTemplatePath)
            except Exception, e:
                logger.error("Can't find domain template jar" + domainTemplatePath)
                writeRC(RC_CANT_READ_TEMPLATE)
                return RC_CANT_READ_TEMPLATE
      
            try:
                cd ('Security/' + get('Name') + '/User/weblogic')
                if adminUser != 'weblogic':
                    set('Name', adminUser)
                cmo.setPassword(adminPasswd)
            except Exception, e:
                logger.error("The password must be at least 8 alphanumeric characters with at least one number or special character.")
                writeRC(RC_BAD_PASSWORD)
                return RC_BAD_PASSWORD
            cd ('/Server/AdminServer')
            set ('ListenPort', int(adminPort))
            originalPort = int(adminPort)
            setOption('OverwriteDomain', 'true')
    
        if isNewDomain:
            writeDomain(domainDir)
            closeTemplate()
            readDomain(domainDir)  # additional structures are setup
 
        if string.lower(host) == string.lower(adminHost):
            configureAdminServer = true
            url='t3://' + HostListenAddress + ':' + `originalPort`
        else:
            configureAdminServer = false
            url='t3://' + adminHost + ':' + adminPort

        nodeManagerHome=domainDir + '/nodemanager'
        if os.path.exists(nodeManagerHome) != true:
            os.mkdir(nodeManagerHome)
     
        setNodeManagerDomain()
        
        # if autoConfigure == 'true' and configureAdminServer == true:
        if autoConfigure == 'true':
            # Workaround for a bug in startNodeManager - it tries to set '-d64' on Windows, which is not supported
            if System.getProperty("sun.arch.data.model") == '64' and osType == 'win':
                System.setProperty("sun.arch.data.model", '32')
            if isServerAvailable(HostListenAddress, nodeManagerPort) == true:
                logger.debug('nodemanager is already running.')
                writeRC(RC_NODEMANAGER_RUNNING_NOT_ALLOWED)
                return RC_NODEMANAGER_RUNNING_NOT_ALLOWED
                
            if osType == 'win':
                startNodeManager(verbose='true', NodeManagerHome=nodeManagerHome, ListenPort=nodeManagerPort, ListenAddress=HostListenAddress, CrashRecoveryEnabled='true', QuitEnabled='true')
            else:
                startNodeManager(verbose='true', NodeManagerHome=nodeManagerHome, ListenPort=nodeManagerPort, ListenAddress=HostListenAddress, CrashRecoveryEnabled='true', QuitEnabled='true', NativeVersionEnabled='false')
            if not waitUntilServerIsAvailable(HostListenAddress, nodeManagerPort):
                if checkForURandomUseOnLinux():
                    writeRC(RC_NODEMANAGER_FAILED_TO_START_LNX)
                    return RC_NODEMANAGER_FAILED_TO_START_LNX
                else:
                    raise Exception
            if os.path.exists(nmUserConfigFile) and os.path.exists(nmUserKeyFile):
                nmConnect(userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile, host=host, port=nodeManagerPort, domainName=os.path.basename(domainDir), domainDir=domainDir)
            else:
                domain_name = os.path.basename(domainDir)
                logger.debug('domain_name:' + domain_name)
                nmConnect(adminUser, adminPasswd, host=host, port=nodeManagerPort, domainName=domain_name, domainDir=domainDir)
                logger.debug('nmConnect completed')
                storeUserConfig(nmUserConfigFile,  nmUserKeyFile, nm="true")
                logger.debug('storeUserConfig completed')
 
        if autoConfigure == 'true' and configureAdminServer == true:
            from java.util import Properties
            admProps = Properties()
            admProps.setProperty('AutoRestart', 'true')
            admProps.setProperty('AutoKillIfFailed', 'false')
            admProps.setProperty('RestartDelaySeconds', '0')
            admProps.setProperty('RestartInterval', '3600')
            admProps.setProperty('Arguments', adminJvmOptions)
            admProps.setProperty('RestartMax', '2')
            admProps.setProperty('SSLArguments', "-Dweblogic.security.SSL.ignoreHostnameVerification=false -Dweblogic.ReverseDNSAllowed=false")
            admProps.setProperty('JavaHome', javaHome)
            logger.debug('About to start AdminServer')
            nmStart('AdminServer',props=admProps)
     
        if autoConfigure == 'true':
            if os.path.exists(nmUserConfigFile) and os.path.exists(nmUserKeyFile):
                connect(userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile, url=url)
            else: 
                connect(url=url, username=adminUser, password=adminPasswd)
            logger.debug('connected to AdminServer')
            nmEnroll(domainDir, domainDir + "/nodemanager")
            edit()
            startEdit()
  
        machineChangesList = ['']
        machine = createMachine(host, HostListenAddress, nodeManagerPort, machineChangesList)

        adminServerChangesList = ['']
        mergedCmdLineArguments = [''] 
        mergedCmdLineChanges = ['']
        mergedScriptCmdLineArguments = [''] 
        mergedScriptCmdLineChanges = ['']
        if configureAdminServer == true:
            createServer('AdminServer', machine, adminPort, adminSSLPort, host, host, 'true', adminJvmOptions, 
                mergedCmdLineArguments, mergedScriptCmdLineArguments, mergedCmdLineChanges, 
                mergedScriptCmdLineChanges, adminServerChangesList,  weblogicHome, javaHome, adminUser, 
                adminPasswd, domainDir, startScript, isNewDomain) 
      
        # set security property for SCS
        if configureAdminServer == true:
            logger.debug('About to set security for SCS')
            SASDomainName=os.path.basename(domainDir)
            cd ('/SecurityConfiguration/' + SASDomainName)
            originalEnforceValidBasicAuthCredentials = get ('EnforceValidBasicAuthCredentials')
            if originalEnforceValidBasicAuthCredentials == 0:  originalEnforceValidBasicAuthCredentials = 'false'
            if originalEnforceValidBasicAuthCredentials == 1:  originalEnforceValidBasicAuthCredentials = 'true'
            changes = changes + logChange(SASDomainName, 'EnforceValidBasicAuthCredentials', originalEnforceValidBasicAuthCredentials, 'false')
            logger.debug('About to set security for SCS')
            set ('EnforceValidBasicAuthCredentials', 'false')

        if autoConfigure == 'false':
            updateDomain()
        else:
            if configureAdminServer == true:
                logger.debug('About to set ShowArchivedRealPathEnabled')
                cd ('/WebAppContainer/' + SASDomainName)
                originalShowArchivedRealPathEnabled = get ('ShowArchivedRealPathEnabled')
                if originalShowArchivedRealPathEnabled == 0:  originalShowArchivedRealPathEnabled = 'false'
                if originalShowArchivedRealPathEnabled == 1:  originalShowArchivedRealPathEnabled = 'true'
                changes = changes + logChange(SASDomainName, 'ShowArchivedRealPathEnabled', originalShowArchivedRealPathEnabled, 'true')  
                set ('ShowArchivedRealPathEnabled', 'true')
            save()
            logger.debug('Domain changes are saved')
            try:  # an exception during activate may because the admin port is changed and connection is dropped
                activate(block='true')
            except Exception, e:
                if adminPort != originalPort:
                    logger.debug('The AdminServer port has changed.  Try reconnecting to the admin server.')
                    url='t3://' + HostListenAddress + ':' + adminPort
                    connect(userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile, url=url)
            disconnect()
            logger.debug('disconnected from AdminServer')
            if configureAdminServer == true:    
                nmKill('AdminServer')
            try:  # ignore failure to stop node manager.  This fails if it was running before we started
                stopNodeManager()
            except Exception, e:
                logger.error('failed to stop nodemanager.')
  
        # set production mode
        if isNewDomain:
            closeDomain()
            readDomain(domainDir)
            set('ProductionModeEnabled', 'true')
            updateDomain()
            closeDomain()
            logger.debug('Production Mode enabled')
        
        writeReturnProperty('weblogic.AdminServer.merged.cmdLine', mergedCmdLineArguments[0])
        writeReturnProperty('weblogic.AdminServer.merged.script.cmdLine', mergedScriptCmdLineArguments[0])
        if isNewDomain:
            writeReturnProperty("weblogic.webauth.configured", "false")
            writeRC(RC_OK_NEW)
            rc = RC_OK_NEW
        else:
            if isWebAuthConfigured(domainDir, osType):
                writeReturnProperty("weblogic.webauth.configured", "true")
            else:
                writeReturnProperty("weblogic.webauth.configured", "false")
            if changes != '':
                writeReturnProperty('weblogic.createDomain.changes', changes)
            if machineChangesList[0] != '':
                writeReturnProperty('weblogic.machine.changes', machineChangesList[0])
            if adminServerChangesList[0] != '':
                writeReturnProperty(adminServerChangesList[0].split("=")[0], string.strip(adminServerChangesList[0].split("=")[1]))
            if mergedCmdLineChanges[0] != '':
                writeReturnProperty('weblogic.server.AdminServer.cmdline.changes', mergedCmdLineChanges[0])
            if mergedScriptCmdLineChanges[0] != '':
                writeReturnProperty('weblogic.server.AdminServer.cmdline.script.changes', mergedScriptCmdLineChanges[0])
            writeRC(RC_OK_RECONFIGURE)
            rc = RC_OK_RECONFIGURE
        logger.debug('The domain has been successfully created and configured.')
        return rc      
    except Exception, e:
        dumpStack()
        handleError(configureAdminServer, e, "-1")
        return -1


