# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

#**************************************************************************************************
# Name: SAScreateForeignJNDIProvider
# Purpose: Create a Foreign JNDI Provider or update an existing Foreign JNID Provider within WebLogic.
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation:
#@@ foreignjndiprovider  resourceDescription: Resources are stored in properties files that are named in the pattern <tt>foreignjndiprovider.<em>resource</em>.properties</tt>.  For information about how these properties control WebLogic Server configuration, see the online help in the WebLogic Administration Console.  More information is available in the WebLogic Server documentation found on the Oracle Web site.  The WebLogic Server 10.3.3 documentation for this resource can be found at <a href="http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/jndi/ConfigureForeignJNDIProvider.html">http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/jndi/ConfigureForeignJNDIProvider.html</a>.
#@@ foreignjndiprovider  foreignJNDIName: The foreign JNDI Name for a linked name.
#@@ foreignjndiprovider  foreignLinkName: The user-specified name of the Foreign JNDI Link MBean instance.  
#@@ foreignjndiprovider  foreignProviderName: The user-specified name of the Foreign JNDI MBean instance.
#@@ foreignjndiprovider  foreignProviderURL: The Foreign Provider URL used to make connections to the remote server.
#@@ foreignjndiprovider  initialContextFactory: The initial context factory to use to connect. This class name depends on the JNDI provider and the vendor that are being used. The value corresponds to the standard JNDI property, java.naming.factory.initial.
#@@ foreignjndiprovider  localJNDIName: The local JNDI Name for a linked name.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output:
#   None
#**************************************************************************************************

def SAScreateForeignJNDIProvider(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    # Debug Statements
    logger.debug("adminUrl=" + adminUrl)
    logger.debug("foreignProviderName=" + foreignProviderName)
    logger.debug("foreignProviderURL=" + foreignProviderURL)
    logger.debug("initialContextFactory=" + initialContextFactory)
    logger.debug("foreignLinkName=" + foreignLinkName)
    logger.debug("localJNDIName=" + localJNDIName)
    logger.debug("foreignJNDIName=" + foreignJNDIName)

    editing = None

    try:
        # Connect to administration server
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
        logger.debug('Creating new Foreign JNDI Provider named ' + foreignProviderName + '.')
        edit()
        startEdit()
        editing = true
        cd('/')

        # Create foreign provider if it doesn't exist
        foreignProvider = getMBean('/ForeignJNDIProviders/' + foreignProviderName)
        if foreignProvider is None:
            foreignProvider = create(foreignProviderName, 'ForeignJNDIProvider')
  
        foreignProvider.setInitialContextFactory(initialContextFactory)
        foreignProvider.setProviderURL(foreignProviderURL)

        # Create the JNDI link if it doesn't exist
        cd('/ForeignJNDIProviders/' + foreignProviderName + '/ForeignJNDILinks/')
        jndiLink = getMBean('/ForeignJNDIProviders/' + foreignProviderName + '/ForeignJNDILinks/' + foreignLinkName)
        if jndiLink is None:
            jndiLink = create(foreignLinkName, 'ForeignJNDILink')

        jndiLink.setLocalJNDIName(localJNDIName)
        jndiLink.setRemoteJNDIName(foreignJNDIName)  	

        save()
        activate(block='true')  # Activate changes
        disconnect()
  
        logger.debug('Exiting Foreign Provider creation.')
        writeRC(RC_OK)
        return RC_OK
    except Exception, e:
        logger.error('Failed to create Foreign JDNI Provider' + foreignProviderName + '.')
        logger.error(e)

        writeRC(RC_FAILED_ACTIVATE)
        rc = RC_FAILED_ACTIVATE
    
        # Cancel edit.
        try:
            if editing is not None:
                logger.debug('Canceling edit...')
                cancelEdit('y')
        except WLSTException:
            pass
        return rc
   
    
