# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
#***********************************************************************************
# Name: SASdeleteJmsServer
# Purpose - delete a JMS Server within WebLogic
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
# resource properties used - see createJMSServer.py for descriptions of these resources:
#!! jmsserver jmsServerName
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output: 
#     None
#***********************************************************************************
def SASdeleteJmsServer(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    logger.debug('adminUrl=' + adminUrl)
    logger.debug('jmsServerName=' + jmsServerName)

    # Connect to administration server
    try:
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
    except WLSTException:
        writeRC(RC_FAIL_ADM_CONNECT)
        return RC_FAIL_ADM_CONNECT

    # Check if JMS Server exists
    try:
        cd('/JMSServers/' + jmsServerName)
        logger.debug('The JMS Server with name ' + jmsServerName + ' exists.')
    except WLSTException:
        writeRC(RC_OK_NOT_CONFIGURED)
        logger.debug('The JMS Server with name ' + jmsServerName + ' does not exist.')
        disconnect()
        return RC_OK_NOT_CONFIGURED

    try:
        logger.debug('Deleting JMS Server named ' + jmsServerName + '.')

        edit()
        startEdit()
        cd('/')

        logger.debug('calling delete jms server')   
        jmsServerMBean = delete(jmsServerName,'JMSServer')
        logger.debug('calling delete jdbc store the jms server referenced')   
        removeJDBCStorebyJMSServer(jmsServerName)
        logger.debug('saving')   
        save()
        activate(block='true')
        logger.debug('Exiting JMS Server deletion.')

        writeRC(RC_OK)
        disconnect()
        return RC_OK
    except WLSTException, e:
        logger.error('Failed to delete JMS server ' + jmsServerName  + '.')
        logger.error(e)
        writeRC(RC_FAILURE)
        cancelEdit('y')
        disconnect()
        return RC_FAILURE
  #function to help remove all related JdbcStore mbean(s) which reference the data source.
# that match a specific name
def removeJDBCStorebyJMSServer(jmsName):
    cd('/JDBCStores')
    jdbcStoreList = cmo.getJDBCStores()
    if jdbcStoreList is not None:
       for js in jdbcStoreList:
          if js.getName()==(jmsName + '_JDBCStore'):
              removeJMSServerbyJDBCStore(js.getName())
              logger.debug('Removing JDBCStore Resource: ' + js.getName() + ' referred by the jms server: ' + jmsName)
              delete(js.getName(),'JDBCStore')  