# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
#***********************************************************************************
# Name: SASdeleteMailSession
# Purpose - delete a Mail Session service 
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
# resource properties used - see createMailSession.py for descriptions of these resources:
#!! mailsession deletedTargets
#!! mailsession mailSessionName
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output: 
#     None
#***********************************************************************************

def SASdeleteMailSession(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)
 
    # Debug Statements
    logger.debug('adminUrl=' + adminUrl)
    logger.debug('deletedTargets=' + deletedTargets)
    logger.debug('mailSessionName=' + mailSessionName)

    try:
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile) 
    except WLSTException:
        writeRC(RC_FAIL_ADM_CONNECT)
        return RC_FAIL_ADM_CONNECT

        # Check if mailSession exists
    try:
        cd('/MailSessions/' + mailSessionName)
        logger.debug('The MailSession ' + mailSessionName + ' exists.')
    except WLSTException:
        writeRC(RC_OK_NOT_CONFIGURED)
        logger.debug('The MailSession ' + mailSessionName + ' does not exist.')
        disconnect()
        return RC_OK_NOT_CONFIGURED
 
    try: 
        edit()
        startEdit()
  
        # remove MailSession from target server.
        cd('/MailSessions/' + mailSessionName)
        deletedTargetList = String.split(deletedTargets, ',')
        targets = cmo.getTargets()
        missingTargets = false
        for thisTarget in deletedTargetList:
            targetFound = false
            for t in targets:
                if t.getName() == thisTarget:
                    logger.debug('Removing ' + thisTarget + ' target from MailSession: ' + mailSessionName)
                    cmo.removeTarget(t)
                    targetFound = true
                    break

            if targetFound == false:
                logger.error('Target ' + thisTarget + ' not found for MailSession ' + mailSessionName)
                missingTargets = true;
      
        targets = cmo.getTargets()  # get new copy of targets object
        cd('/')
        if len(targets) > 0:
            logger.debug('MailSession still references ' + `len(targets)` + ' more target(s)')
        else:
            logger.debug('MailSession is no longer referenced.  Deleting.')
            delete(mailSessionName, 'MailSession')
        save()
        activate(block="true")
        disconnect()
        if missingTargets == true:
            rc = RC_OK_NOT_TARGETTED
        else:
            rc = RC_OK
        writeRC(rc)
        return RC_OK
    except WLSTException, e:
        logger.error('Failed to delete mailSession ' + mailSessionName + '.')
        logger.error(e)
        writeRC(RC_FAILURE)
        cancelEdit('y')
        disconnect()
        return RC_FAILURE

  
