# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import string
import os.path
from com.sas.util import SasPasswordString

#**************************************************************************************************
# Name: SASlistJMSServers
# Purpose: List the servers in the domain.
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output:
#   Writes a comma separated list of <JMSServerName>/<ManagedServer> to the return code file in the property 
#   "weblogic.jmsservername.list".
#**************************************************************************************************
def SASlistJMSServers(credentialsFile, resourcePropertiesFile):

    # Debug Statements
    logger.debug('adminUrl:' + adminUrl)

    serverList = None
    try:
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)      
  
        cd('/JMSServers')

        label = ""
        serverNameList = None
        serverList = cmo.getJMSServers()
        if serverList is not None:
            for i in cmo.getJMSServers():
                label = i.getName() + "/" 
                targets = i.getTargets()
                if targets is not None and len(targets) > 0:
                    target = i.getTargets()[0].getName()      
                    label = label + target
                else:
                    label = label + "NO_TARGET_DEFINED"
      
                logger.debug(label)
                if serverNameList is None:
                    serverNameList = label
                else:
                    serverNameList = serverNameList + "," + label  
  
        if serverNameList is None:
            serverNameList = ""
    
        writeReturnProperty("weblogic.jmsservername.list=", serverNameList)
        writeRC(RC_OK)
        disconnect() 
        return RC_OK   
    except Exception, e:
        logger.error('Unable to list JMS servers.')
        logger.error(e)
        writeRC(RC_FAILURE)
        return RC_FAILURE
