# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import os
import time
import string
import java.lang
import java.io
from org.apache.log4j import Logger, PropertyConfigurator, PatternLayout
from com.sas.config.weblogic import WebLogicCredentialsPrep
from com.sas.config.util import CredentialsDialog
from com.sas.config.util import ResourceProperties
from com.sas.config.util import PropertyUtils

execfile(os.path.dirname(sys.argv[0])+'/createDomain.py')
execfile(os.path.dirname(sys.argv[0])+'/customizeDomain.py')
execfile(os.path.dirname(sys.argv[0])+'/startNodemanager.py')
execfile(os.path.dirname(sys.argv[0])+'/stopNodemanager.py')
execfile(os.path.dirname(sys.argv[0])+'/checkManagedServerStatus.py')
execfile(os.path.dirname(sys.argv[0])+'/createServer.py')
execfile(os.path.dirname(sys.argv[0])+'/deleteServer.py')
execfile(os.path.dirname(sys.argv[0])+'/startManagedServer.py')
execfile(os.path.dirname(sys.argv[0])+'/stopManagedServer.py')
execfile(os.path.dirname(sys.argv[0])+'/createMailSession.py')
execfile(os.path.dirname(sys.argv[0])+'/deleteMailSession.py')
execfile(os.path.dirname(sys.argv[0])+'/createDataSource.py')
execfile(os.path.dirname(sys.argv[0])+'/deleteDataSource.py')
execfile(os.path.dirname(sys.argv[0])+'/createJMSServer.py')
execfile(os.path.dirname(sys.argv[0])+'/deleteJMSServer.py')
execfile(os.path.dirname(sys.argv[0])+'/createJMSResource.py')
execfile(os.path.dirname(sys.argv[0])+'/deleteJMSResource.py')
execfile(os.path.dirname(sys.argv[0])+'/createUser.py')
execfile(os.path.dirname(sys.argv[0])+'/deployApplication.py')
execfile(os.path.dirname(sys.argv[0])+'/undeployApplication.py')
execfile(os.path.dirname(sys.argv[0])+'/setPolicy.py')
execfile(os.path.dirname(sys.argv[0])+'/createForeignJNDIProvider.py')
execfile(os.path.dirname(sys.argv[0])+'/listServers.py')
execfile(os.path.dirname(sys.argv[0])+'/listJMSServers.py')
execfile(os.path.dirname(sys.argv[0])+'/addLoginModule.py')
execfile(os.path.dirname(sys.argv[0])+'/sasutils.py')

domainOps = {"create":SAScreateDomain, "customize":SAScustomizeDomain, "listservers":SASlistServers, "listjmsservers":SASlistJMSServers, "createloginmodule":SASaddLoginModule}
nodeManagerOps = {"start":SASstartNodeManager, "stop":SASstopNodeManager}
serverOps = {"check":SAScheckServer, "create":SAScreateServer, "start":SASstartServer, "stop":SASstopServer, "delete":SASdeleteServer}
mailSessionOps = {"create":SAScreateMailSession, "delete":SASdeleteMailSession}
jmsServerOps = {"create":SAScreateJmsServer, "delete":SASdeleteJmsServer}
jmsConnectionFactoryOps = {"create":SAScreateJMSResource, "delete":SASdeleteJMSResource}
jmsQueueOps = {"create":SAScreateJMSResource, "delete":SASdeleteJMSResource}
jmsTopicOps = {"create":SAScreateJMSResource, "delete":SASdeleteJMSResource}
userOps = {"create":SAScreateUser}
policyOps = {"set":SASsetPolicy}
applicationOps = {"compile":SAScompileApplication, "deploy":SASdeployApplication, "undeploy":SASundeployApplication}
dataSourceOps = {"create":SAScreateDataSource, "delete":SASdeleteDataSource}
foreignJndiProviderOps = {"create":SAScreateForeignJNDIProvider}
jdbcProviderOps = {"create":SAScreateJDBCProvider, "delete":SASdeleteJDBCProvider}

resources = {"application":applicationOps,
             "connectionfactory":jmsConnectionFactoryOps,
             "datasource":dataSourceOps,
             "domain":domainOps,
             "foreignjndiprovider":foreignJndiProviderOps,
             "jdbcprovider":jdbcProviderOps,
             "jmsserver":jmsServerOps,
             "mailsession":mailSessionOps,
             "nodemanager":nodeManagerOps,
             "policy":policyOps,
             "queue":jmsQueueOps,
             "server":serverOps,
             "topic":jmsTopicOps,
             "user":userOps
             }

def execOperation(op, type, name):
    logger.debug("credentialsFile=" + credentialsFile)
    rc = -1
    try:
        resource = resources[type]
    except KeyError, e:
        logger.error('Unsupported resource type: ' + type)
        usage()
        return -1
    resourcePropsFile = propertiesHome + '/' + type + "." + name + ".properties"
    if os.path.exists(resourcePropsFile) != true:
        logger.error('Resource file ' + resourcePropsFile + ' not found.')
        return -1
    try:
        import time
        startTime= time.time()

        logger.info('***** executing command ' + op + ' ' + type + ' ' + name + ' *****')
        rc = resource[op](credentialsFile, resourcePropsFile)
        import time
        endTime= time.time()
        logger.info('***** This step took ' + `endTime - startTime` + ' seconds to complete. *****')
        if int(rc) < 0:
            logger.error('operation completed with return code ' + `rc`)
        else:
            logger.info('operation completed with return code ' + `rc`)
    except KeyError, e:
        logger.error('Unsupported operation: ' + op)
        usage()
        return -1
    return int(rc)

def processCredentials(credentialsFile, mode, command, batchFileName):
    print "processCredentials: " + credentialsFile + ' promptMode=' + promptMode
    try:
        needAdminCredentials = true
        if os.path.exists(nmUserConfigFile) and os.path.exists(nmUserKeyFile):
            needAdminCredentials = false
        credPrep = WebLogicCredentialsPrep(credentialsFile, true, promptMode, needAdminCredentials)
        batchFile = None;
        if mode == "BATCH":
            batchFile = java.io.File(batchFileName)
        tasksNeedingCredentials = credPrep.scanCommandsForCredentials(mode, command, batchFile)
        contents = credPrep.processCredentials(tasksNeedingCredentials);
        if len(contents) > 0:
            rc = credPrep.invokeDialog(contents)
    except java.lang.IllegalArgumentException, e:
        logger.error('Credentials file contains improperly formed keys.')
        exit (-1)
    return

def clearCredentials():
    credentialsFile = propertiesHome + '/' + 'credentials.properties'    
    if webappsrvScriptingCacheCredentials == "false":
        credentials = ResourceProperties(credentialsFile)
        credentials.clearAll()
        credentials = PropertyUtils.loadProperties(File(credentialsFile))
        PropertyUtils.writeProps(credentials, File(credentialsFile))
    return

def printFile(fileName):
    fp = open(fileName)
    line = fp.readline()
    while line:
        logger.info(line.strip())
        line = fp.readline()
    fp.close()
    return

def usage():
    printFile("help.txt")
    
# main driver

scriptHome = System.getenv('SAS_WLST_SCRIPT_HOME')
if scriptHome == None:
    scriptHome = '.'

propertiesHome = scriptHome + '/props'

PropertyConfigurator.configure(scriptHome + '/' + "log4j.properties");
logger = Logger.getLogger("sas")
rlogger = Logger.getRootLogger()

loadProperties(os.path.dirname(sys.argv[0]) + '/returnCode.properties')
domainPropertiesFile=propertiesHome + '/' + 'global.properties'
credentialsFile=propertiesHome + '/' + 'credentials.properties'
logger.debug("domainPropertiesFile=" + domainPropertiesFile)

loadProperties(domainPropertiesFile)
setWLSCredentials()

deleteReturnProperties()

args = []
scriptMode="standalone"
promptMode = "TEXT"
processCreds = false

for arg in sys.argv:
    if arg[0:12] == "credentials=":
        credentialsFile = arg[12:len(arg)]
    elif arg[0:11] == "promptMode=":
        promptMode = string.upper(arg[11:len(arg)])
        if not(promptMode == "TEXT" or promptMode == "GRAPHIC"):
            logger.error('promptMode:' + promptMode + ' not valid.')
            usage()
            exit()
    elif arg[0:11] == "scriptMode=":
        scriptMode= arg[11:len(arg)]
    elif arg[0:18] == "processCredentials":
        processCreds = true
    else:
        args.append(arg)

if scriptMode == "standalone":
    logger.info("Running in script mode, forcing autoConfigure and autoDeploy.")    
    autoConfigure="true"
    autoDeploy="true"

logger.debug('promptMode=' + promptMode)  
logger.trace('there are ' + `len(args)` + ' arguments')
logger.debug('Credentials:' + credentialsFile) 
# example: saswlst operation type name
if len(args) == 4:
    operation = args[1]
    resourceType = args[2]
    resourceName = args[3]
    if processCreds == true:
        processCredentials(credentialsFile, "RUN", operation + ' ' + resourceType + ' ' + resourceName, None)
    else:
        execOperation(operation, resourceType, resourceName)
# example: saswlst commandFile
elif len(args) == 2:
    commandFile = args[1]
    if os.path.exists(commandFile) != true:
        logger.warn('command file ' + commandFile + ' does not exist.')
        usage()
        exit()
    if processCreds == true:
        processCredentials(credentialsFile, "BATCH", None, commandFile)
    else:
        logger.info('Processing command file: ' + commandFile)
        fp = open(commandFile)
        line = fp.readline()
        lineNum = 1
        while line:
            line = string.strip(line)
            print line
            # ignore empty lines, lines containing only white space or lines starting with #
            if len(line) == 0 or line[0] == '#' or len(line.strip()) == 0:
                lineNum = lineNum + 1
                line = fp.readline()
                continue
            largs = line.split(' ')
            if len(largs) == 3:
                rc = execOperation(largs[0], largs[1], largs[2])
            else:
                logger.error('Bad command at line ' + `lineNum` + ': ' + line,)
                clearCredentials()
                exit()
            if rc < 0:
                logger.error('The command on line ' + `lineNum` + ' failed')
                logger.error('line =' + line) 
                clearCredentials()
                exit()
            lineNum = lineNum + 1
            line = fp.readline()
# example:  saswlst  (interactive mode)
elif len(args) == 1:
    # use simple pattern layout for interactive console
    logPattern = PatternLayout("%m%n");
    consoleAppender = rlogger.getAppender("stdout")
    consoleAppender.setLayout(logPattern)
    
    logger.info('Enter commands interactively:')
    logger.info('Usage: operation type name')
    line = raw_input('> ')
    while processCreds == true:
        if len(line) == 0 or len(line.strip()) == 0:
            line = raw_input('> ')
            continue
        if line.strip() == 'exit':
            break
        logger.info('command line=' + line)
        largs = line.split(' ')
        if len(largs) == 3:
            processCredentials(credentialsFile, "RUN", largs[0] + ' ' + largs[1] + ' ' + largs[2], None)
            rc = execOperation(largs[0], largs[1], largs[2])
        else:
            logger.error('Usage: operation type name')
            usage()
        line = raw_input('> ')
else:
    usage()

if processCreds == false:
    clearCredentials()
exit()   


